@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/exim_monitor/em_log.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+
 /*************************************************
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2001 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module contains code for scanning the smaill log,
@@ -42,7 +44,7 @@ static char *oklist[] = {
   "\"",
   " ",
   "\n"};
-static int oklist_size = sizeof(oklist) / sizeof(char *);
+static int oklist_size = sizeof(oklist) / sizeof(uschar *);
 #endif
 
 
@@ -61,7 +63,7 @@ static void show_log(char *s, ...)
 int length, newtop;
 va_list ap;
 XawTextBlock b;
-char buffer[log_buffer_len + 24];
+uschar buffer[log_buffer_len + 24];
 
 /* Do nothing if not tailing a log */
 
@@ -70,7 +72,7 @@ if (log_widget == NULL) return;
 /* Initialize the text block structure */
 
 b.firstPos = 0;
-b.ptr = buffer;
+b.ptr = CS buffer;
 b.format = FMT8BIT;
 
 /* We want to know whether the window has been scrolled back or not,
@@ -95,15 +97,15 @@ if (newtop != top)
 /* Format the text that is to be written. */
 
 va_start(ap, s);
-vsprintf(buffer, s, ap);
+vsprintf(CS buffer, s, ap);
 va_end(ap);
-length = (int)strlen(buffer);
+length = Ustrlen(buffer);
 
 /* If we are anonymizing for screen shots, flatten various things. */
 
 #ifdef ANONYMIZE
   {
-  char *p = buffer + 9;
+  uschar *p = buffer + 9;
   if (p[6] == '-' && p[13] == '-') p += 17;
 
   while (p < buffer + length)
@@ -114,8 +116,8 @@ length = (int)strlen(buffer);
 
     for (i = 0; i < oklist_size; i++)
       {
-      int len = (int)strlen(oklist[i]);
-      if (strncmp(p, oklist[i], len) == 0)
+      int len = Ustrlen(oklist[i]);
+      if (Ustrncmp(p, oklist[i], len) == 0)
         {
         p += len;
         break;
@@ -135,7 +137,7 @@ length = (int)strlen(buffer);
 
     /* Leave C= text alone */
 
-    if (strncmp(p, "C=\"", 3) == 0)
+    if (Ustrncmp(p, "C=\"", 3) == 0)
       {
       p += 3;
       while (*p != 0 && *p != '"') p++;
@@ -144,8 +146,8 @@ length = (int)strlen(buffer);
 
     /* Flatten remaining chars */
 
-    if (isdigit((uschar)*p)) *p++ = 'x';
-    else if (isalpha((uschar)*p)) *p++ = 'x';
+    if (isdigit(*p)) *p++ = 'x';
+    else if (isalpha(*p)) *p++ = 'x';
     else *p++ = '$';
     }
   }
@@ -183,11 +185,12 @@ XawTextReplace(log_widget, 999999, 999999, &b);
 size += length;
 
 /* When not scrolled back, we want to keep the bottom line
-always visible. */
+always visible. Put the insert point at the start of it because
+this stops left/right scrolling with some X libraries. */
 
 if (!scrolled)
   {
-  XawTextSetInsertionPoint(log_widget, size - 1);
+  XawTextSetInsertionPoint(log_widget, size - length);
   top = XawTextTopPosition(log_widget);
   }
 }
@@ -208,7 +211,7 @@ version in the log window. */
 void read_log(void)
 {
 struct stat statdata;
-char buffer[log_buffer_len];
+uschar buffer[log_buffer_len];
 
 /* If log is not yet open, skip all of this. */
 
@@ -216,12 +219,12 @@ if (LOG != NULL)
   {
   fseek(LOG, log_position, SEEK_SET);
 
-  while (fgets(buffer, log_buffer_len, LOG) != NULL)
+  while (Ufgets(buffer, log_buffer_len, LOG) != NULL)
     {
-    char *id;
-    char *p = buffer;
+    uschar *id;
+    uschar *p = buffer;
     void *reset_point;
-    int length = (int)strlen(buffer);
+    int length = Ustrlen(buffer);
     int i;
 
     /* Skip totally blank lines (paranoia: there shouldn't be any) */
@@ -241,33 +244,36 @@ if (LOG != NULL)
 
     for (i = stripchart_varstart; i < stripchart_number; i++)
       {
-      if (pcre_exec(stripchart_regex[i], NULL, buffer, length, 0, PCRE_EOPT,
+      if (pcre_exec(stripchart_regex[i], NULL, CS buffer, length, 0, PCRE_EOPT,
             NULL, 0) >= 0)
         stripchart_total[i]++;
       }
 
     /* Munge the log entry and display shortened form on one line.
-    We omit the date and show only the time. */
+    We omit the date and show only the time. Remove any time zone offset. */
 
-    if (pcre_exec(yyyymmdd_regex,NULL,buffer,length,0,PCRE_EOPT,NULL,0) >= 0)
+    if (pcre_exec(yyyymmdd_regex,NULL,CS buffer,length,0,PCRE_EOPT,NULL,0) >= 0)
       {
+      if ((buffer[20] == '+' || buffer[20] == '-') &&
+          isdigit(buffer[21]) && buffer[25] == ' ')
+        memmove(buffer + 20, buffer + 26, Ustrlen(buffer + 26) + 1);
       id = string_copyn(buffer + 20, MESSAGE_ID_LENGTH);
       show_log("%s", buffer+11);
       }
     else
       {
-      id = "";
+      id = US"";
       show_log("%s", buffer);
       }
 
     /* Deal with frozen and unfrozen messages */
 
-    if (strstric(buffer, "frozen", FALSE) != NULL)
+    if (strstric(buffer, US"frozen", FALSE) != NULL)
       {
       queue_item *qq = find_queue(id, queue_noop, 0);
       if (qq != NULL)
         {
-        if (strstric(buffer, "unfrozen", FALSE) != NULL)
+        if (strstric(buffer, US"unfrozen", FALSE) != NULL)
           qq->frozen = FALSE;
         else qq->frozen = TRUE;
         }
@@ -277,17 +283,17 @@ if (LOG != NULL)
     isn't already on the list for this message, with a pointer
     to the parent if we can. */
 
-    if ((p = strstr(buffer, "==")) != NULL)
+    if ((p = Ustrstr(buffer, "==")) != NULL)
       {
       queue_item *qq = find_queue(id, queue_noop, 0);
       if (qq != NULL)
         {
         dest_item *d;
-        char *q, *r;
+        uschar *q, *r;
         p += 2;
-        while (isspace((uschar)*p)) p++;
+        while (isspace(*p)) p++;
         q = p;
-        while (*p != 0 && !isspace((uschar)*p))
+        while (*p != 0 && !isspace(*p))
           {
           if (*p++ != '\"') continue;
           while (*p != 0)
@@ -299,10 +305,15 @@ if (LOG != NULL)
         *p++ = 0;
         if ((r = strstric(q, qualify_domain, FALSE)) != NULL &&
           *(--r) == '@') *r = 0;
-        d = find_dest(qq, q, dest_add);
+
+        /* If we already have this destination, as tested case-insensitively,
+        do not add it to the destinations list. */
+
+        d = find_dest(qq, q, dest_add, TRUE);
+
         if (d->parent == NULL)
           {
-          while (isspace((uschar)*p)) p++;
+          while (isspace(*p)) p++;
           if (*p == '<')
             {
             dest_item *dd;
@@ -311,7 +322,7 @@ if (LOG != NULL)
             *p = 0;
             if ((p = strstric(q, qualify_domain, FALSE)) != NULL &&
               *(--p) == '@') *p = 0;
-            dd = find_dest(qq, q, dest_noop);
+            dd = find_dest(qq, q, dest_noop, FALSE);
             if (dd != NULL && dd != d) d->parent = dd;
             }
           }
@@ -324,10 +335,16 @@ if (LOG != NULL)
 
 
 /* We have to detect when the log file is changed, and switch to the new file.
-In practice, this means that some deliveries might go unrecorded, since they'll
-be written to the old file, but this usually happens in the middle of the
-night, and I don't think the hassle of keeping track of two log files is worth
-it.
+In practice, for non-datestamped files, this means that some deliveries might
+go unrecorded, since they'll be written to the old file, but this usually
+happens in the middle of the night, and I don't think the hassle of keeping
+track of two log files is worth it.
+
+First we check the datestamped name of the log file if necessary; if it is
+different to the file we currently have open, go for the new file. As happens
+in Exim itself, we leave in the following inode check, even when datestamping
+because it does no harm and will cope should a file actually be renamed for
+some reason.
 
 The test for a changed log file is to look up the inode of the file by name and
 compare it with the saved inode of the file we currently are processing. This
@@ -338,9 +355,24 @@ old file remains in existence. This can happen if the old log file is renamed,
 processed in some way, and then deleted. To work round this, also test for a
 link count of zero on the currently open file. */
 
+if (log_datestamping)
+  {
+  uschar log_file_wanted[256];
+  string_format(log_file_wanted, sizeof(log_file_wanted), CS log_file);
+  if (Ustrcmp(log_file_wanted, log_file_open) != 0)
+    {
+    if (LOG != NULL)
+      {
+      fclose(LOG);
+      LOG = NULL;
+      }
+    Ustrcpy(log_file_open, log_file_wanted);
+    }
+  }
+
 if (LOG == NULL ||
     (fstat(fileno(LOG), &statdata) == 0 && statdata.st_nlink == 0) ||
-    (stat(log_file, &statdata) == 0 && log_inode != statdata.st_ino))
+    (Ustat(log_file, &statdata) == 0 && log_inode != statdata.st_ino))
   {
   FILE *TEST;
 
@@ -349,7 +381,7 @@ if (LOG == NULL ||
   is renamed and before the new one exists. Therefore do a
   trial open first to be sure. */
 
-  if ((TEST = fopen(log_file, "r")) != NULL)
+  if ((TEST = fopen(CS log_file_open, "r")) != NULL)
     {
     if (LOG != NULL) fclose(LOG);
     LOG = TEST;
