@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/exim_monitor/em_main.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+
 /*************************************************
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -137,7 +139,7 @@ static Widget above_queue_widget;
 in their libraries, but can provide the same facility by this simple
 alternative function. */
 
-char *
+uschar *
 strerror(int n)
 {
 if (n < 0 || n >= sys_nerr) return "unknown error number";
@@ -148,29 +150,38 @@ return sys_errlist[n];
 
 
 /*************************************************
-*               Debug print routine              *
+*         Handle attempts to write the log       *
 *************************************************/
 
-void debug(char *format, ...)
+/* The message gets written to stderr when log_write() is called from a
+utility. The message always gets '\n' added on the end of it. These calls come
+from modules such as store.c when things go drastically wrong (e.g. malloc()
+failing). In normal use they won't get obeyed.
+
+Arguments:
+  selector  not relevant when running a utility
+  flags     not relevant when running a utility
+  format    a printf() format
+  ...       arguments for format
+
+Returns:    nothing
+*/
+
+void
+log_write(unsigned int selector, int flags, char *format, ...)
 {
 va_list ap;
 va_start(ap, format);
-
-if (DEBUG == NULL)
-  {
-  if ((DEBUG = fopen("DEBUG", "w")) == NULL)
-    {
-    printf("Failed to open DEBUG\n");
-    exit(99);
-    }
-  else printf("Opened debugging file DEBUG\n");
-  }
-
-vfprintf(DEBUG, format, ap);
+vfprintf(stderr, format, ap);
+fprintf(stderr, "\n");
 va_end(ap);
+selector = selector;     /* Keep picky compilers happy */
+flags = flags;
 }
 
 
+
+
 /*************************************************
 *        Extract port from address string        *
 *************************************************/
@@ -189,7 +200,7 @@ Returns:     0 if there is no port, else the port number.
 */
 
 int
-host_extract_port(char *address)
+host_extract_port(uschar *address)
 {
 int skip = -3;                     /* Skip 3 dots in IPv4 addresses */
 address--;
@@ -201,27 +212,11 @@ while (*(++address) != 0)
   }
 if (*address == 0) return 0;
 *address++ = 0;
-return atoi(address);
+return Uatoi(address);
 }
 
 
 
-/*************************************************
-*            Dummy routine                       *
-*************************************************/
-
-/* Eximon calls some functions from the main exim code, and these in turn
-may call log_write in error cases. That just writes to stderr, but the log
-module calls smtp_closedown() on panic crashes, so we must provide a dummy
-function here. One of these days this should all be made much tidier, with
-less inter-dependence. */
-
-void smtp_closedown(char *message)
-{
-message = message;
-}
-
-
 
 /*************************************************
 *                SIGCHLD handler                 *
@@ -263,7 +258,7 @@ call_data = call_data;
 actioned_message[0] = 0;
 dialog_ref_widget = w;
 dialog_action = da_hide;
-create_dialog("Hide addresses ending with", "");
+create_dialog(US"Hide addresses ending with", US"");
 }
 
 void unhideAction(Widget w, XtPointer client_data, XtPointer call_data)
@@ -429,7 +424,7 @@ XRaiseWindow(X_display, w);
 
 /* The various cases here are: hide domain, (no more yet) */
 
-void NonMessageDialogue(char *s)
+void NonMessageDialogue(uschar *s)
 {
 skip_item *sk;
 
@@ -450,10 +445,10 @@ switch(dialog_action)
 
   /* Add item to skip queue */
 
-  sk = (skip_item *)store_malloc(sizeof(skip_item) + (int)strlen(s));
+  sk = (skip_item *)store_malloc(sizeof(skip_item) + Ustrlen(s));
   sk->next = queue_skip;
   queue_skip = sk;
-  strcpy(sk->text, s);
+  Ustrcpy(sk->text, s);
   sk->reveal = time(NULL) + 60 * 60;
   scan_spool_input(TRUE);
   queue_display();
@@ -515,7 +510,7 @@ and display their output if their windows are still open. */
 while (p != NULL)
   {
   int count;
-  char buffer[256];
+  uschar buffer[256];
 
   while ((count = read(p->fd, buffer, 254)) > 0)
     {
@@ -559,12 +554,12 @@ Arguments:
 Returns:    points to the buffer, or NULL
 */
 
-static char *
-numlock_modifiers(Display *display, char *buf)
+static uschar *
+numlock_modifiers(Display *display, uschar *buf)
 {
 XModifierKeymap *m;
 int i, j;
-char *ret = NULL;
+uschar *ret = NULL;
 
 m = XGetModifierMapping(display);
 if (m == NULL)
@@ -583,7 +578,7 @@ for (i = 3; i < 8; i++)
     if (XKeycodeToKeysym(display, m->modifiermap [i*m->max_keypermod + j], 0)
         == XK_Num_Lock)
       {
-      sprintf(buf+strlen(buf), " Mod%d", i-2);
+      sprintf(CS(buf+Ustrlen(buf)), " Mod%d", i-2);
       ret = buf;
       }
     }
@@ -603,8 +598,8 @@ int main(int argc, char **argv)
 {
 int i;
 struct stat statdata;
-char modbuf[] = " Mod1 Mod2 Mod3 Mod4 Mod5";
-char *numlock;
+uschar modbuf[] = " Mod1 Mod2 Mod3 Mod4 Mod5";
+uschar *numlock;
 Widget stripchart_form_widget,
        update_widget,
        quit_widget,
@@ -631,7 +626,7 @@ printf("\nExim Monitor version %s (compiled %s) initializing\n",
 
 /* Initialize various things from the environment and arguments. */
 
-init(argc, argv);
+init(argc, USS argv);
 
 /* Set up the SIGCHLD handler */
 
@@ -639,7 +634,7 @@ signal(SIGCHLD, sigchld_handler);
 
 /* Get the buffer for storing the string for the log display. */
 
-log_display_buffer = (char *)store_malloc(log_buffer_size);
+log_display_buffer = (uschar *)store_malloc(log_buffer_size);
 log_display_buffer[0] = 0;
 
 /* Initialize the data structures for the stripcharts */
@@ -648,16 +643,28 @@ stripchart_init();
 
 /* If log_file contains the empty string, then Exim is running using syslog
 only, and we can't tail the log. If not, open the log file and position to the
-end of it. Save its inode so that we can detect when the file is switched to
-another one. However, allow the monitor to start up without a log file (can
-happen if no messages have been sent today.) */
+end of it. Before doing so, we have to detect whether the log files are
+datestamped, and if so, sort out the name. The string in log_file already has
+%s replaced by "main"; if datestamping is occurring, %D will be present. In
+fact, we don't need to test explicitly - just process the string with
+string_format.
+
+Once opened, save the file's inode so that we can detect when the file is
+switched to another one for non-datestamped files. However, allow the monitor
+to start up without a log file (can happen if no messages have been sent
+today.) */
 
 if (log_file[0] != 0)
   {
-  LOG = fopen(log_file, "r");
+  (void)string_format(log_file_open, sizeof(log_file_open), CS log_file);
+  log_datestamping = string_datestamp_offset >= 0;
+
+  LOG = fopen(CS log_file_open, "r");
+
   if (LOG == NULL)
     {
-    printf("*** eximon warning: can't open log file - will try periodically\n");
+    printf("*** eximon warning: can't open log file %s - will try "
+      "periodically\n", log_file_open);
     }
   else
     {
@@ -713,19 +720,19 @@ state of other modifiers. Thanks to Kevin Ryde for this information, and for
 the function above that discovers which modifier is Num Lock, because it turns
 out that it varies from server to server. */
 
-sprintf(big_buffer,
+sprintf(CS big_buffer,
   "!%s:            menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
    !Lock %s:       menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
   ", menu_event, menu_event);
 
 numlock = numlock_modifiers(X_display, modbuf); /* Get Num Lock modifier(s) */
 
-if (numlock != NULL) sprintf(big_buffer + (int)strlen(big_buffer),
+if (numlock != NULL) sprintf(CS big_buffer + Ustrlen(big_buffer),
   "!%s %s:         menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
    !Lock %s %s:    menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
   ", numlock, menu_event, numlock, menu_event);
 
-sprintf(big_buffer + (int)strlen(big_buffer),
+sprintf(CS big_buffer + Ustrlen(big_buffer),
   "<Btn1Down>:     select-start()\n\
    <Btn1Motion>:   extend-adjust()\n\
    <Btn1Up>:       extend-end(PRIMARY,CUT_BUFFER0)\n\
@@ -738,7 +745,7 @@ sprintf(big_buffer + (int)strlen(big_buffer),
    Ctrl<Key>S:     search(forward)\n\
   ");
 
-queue_trans = XtParseTranslationTable(big_buffer);
+queue_trans = XtParseTranslationTable(CS big_buffer);
 
 text_trans = XtParseTranslationTable(
   "<Btn1Down>:     select-start()\n\
@@ -782,7 +789,7 @@ if (size_stripchart != NULL)
 #ifdef HAVE_STATFS
   if (size_stripchart_name == NULL)
     {
-    size_stripchart_name = size_stripchart + (int)strlen(size_stripchart) - 1;
+    size_stripchart_name = size_stripchart + Ustrlen(size_stripchart) - 1;
     while (size_stripchart_name > size_stripchart &&
       *size_stripchart_name == '/') size_stripchart_name--;
     while (size_stripchart_name > size_stripchart &&
@@ -828,7 +835,7 @@ if (log_file[0] != 0)
     outer_form_widget, log_args, XtNumber(log_args));
   XawTextDisplayCaret(log_widget, TRUE);
   xs_SetValues(log_widget, 6,
-    "editType",  XawtextAppend,
+    "editType",  XawtextEdit,
     "translations", text_trans,
     "string",    log_display_buffer,
     "length",    log_buffer_size,
@@ -837,7 +844,7 @@ if (log_file[0] != 0)
 
   if (log_font != NULL)
     {
-    XFontStruct *f = XLoadQueryFont(X_display, log_font);
+    XFontStruct *f = XLoadQueryFont(X_display, CS log_font);
     if (f != NULL) xs_SetValues(log_widget, 1, "font", f);
     }
 
@@ -867,14 +874,14 @@ queue_widget = XtCreateManagedWidget("queue", asciiTextWidgetClass,
 XawTextDisplayCaret(queue_widget, TRUE);
 
 xs_SetValues(queue_widget, 4,
-  "editType",  XawtextAppend,
+  "editType",  XawtextEdit,
   "height",    queue_depth,
   "width",     queue_width,
   "translations", queue_trans);
 
 if (queue_font != NULL)
   {
-  XFontStruct *f = XLoadQueryFont(X_display, queue_font);
+  XFontStruct *f = XLoadQueryFont(X_display, CS queue_font);
   if (f != NULL) xs_SetValues(queue_widget, 1, "font", f);
   }
 
