@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/exim_monitor/em_strip.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+
 /*************************************************
 *                   Exim Monitor                 *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -25,7 +27,7 @@ static int    *stripchart_last_total;        /* vector of prevous values */
 static int    *stripchart_max;               /* vector of maxima */
 static int    *stripchart_middelay;          /* vector of */
 static int    *stripchart_midmax;            /* vector of */
-static char  **stripchart_name;              /* vector of name strings */
+static uschar  **stripchart_name;              /* vector of name strings */
 static Widget  stripchart_prev_chart = NULL; /* previously created chart */
 static Widget  stripchart_prev_label = NULL; /* previously created label */
 
@@ -43,7 +45,7 @@ stripchart_last_total = (int *)store_malloc(stripchart_number * sizeof(int));
 stripchart_max =        (int *)store_malloc(stripchart_number * sizeof(int));
 stripchart_middelay =   (int *)store_malloc(stripchart_number * sizeof(int));
 stripchart_midmax =     (int *)store_malloc(stripchart_number * sizeof(int));
-stripchart_name =     (char **)store_malloc(stripchart_number * sizeof(char *));
+stripchart_name =     (uschar **)store_malloc(stripchart_number * sizeof(uschar *));
 stripchart_total =      (int *)store_malloc(stripchart_number * sizeof(int));
 }
 
@@ -93,7 +95,7 @@ else if (size_stripchart != NULL && num == 1)
   {
 #ifdef HAVE_STATFS
   struct statvfs statbuf;
-  if (statvfs(size_stripchart, &statbuf) == 0)
+  if (statvfs(CS size_stripchart, &statbuf) == 0)
     {
     int used = statbuf.f_blocks - statbuf.f_bfree;
     int max = used + statbuf.f_bavail;
@@ -135,12 +137,12 @@ while (thresholds[i] > 0)
     if (stripchart_max[num] < thresh ||
        (stripchart_max[num] > thresh && stripchart_delay[num]++ > 20))
       {
-      char buffer[128];
+      uschar buffer[128];
       newmax = (thresh > stripchart_midmax[num])?
         thresh : stripchart_midmax[num];
-      if (newmax == 10) sprintf(buffer, "%s", stripchart_name[num]);
-        else sprintf(buffer, "%s x%d", stripchart_name[num], newmax/10);
-      if (size_stripchart != NULL && num == 1) strcat(buffer, "%");
+      if (newmax == 10) sprintf(CS buffer, "%s", stripchart_name[num]);
+        else sprintf(CS buffer, "%s x%d", stripchart_name[num], newmax/10);
+      if (size_stripchart != NULL && num == 1) Ustrcat(buffer, "%");
       xs_SetValues(stripchart_label[num], 1, "label", buffer);
       oldmax = stripchart_max[num];
       stripchart_max[num] = newmax;
@@ -213,7 +215,7 @@ and for the second if it is a partition size display; its update time is
 initially set to 1 second so that it gives an immediate display of the queue.
 The first time its callback function is obeyed, the update time gets reset. */
 
-void create_stripchart(Widget parent, char *title)
+void create_stripchart(Widget parent, uschar *title)
 {
 Widget chart;
 
