@@ -1,4 +1,5 @@
 #! /bin/sh
+# $Cambridge: exim/exim-src/scripts/Configure-config.h,v 1.1 2004/10/06 15:07:40 ph10 Exp $
 
 # Build the config.h file, using the buildconfig program, first ensuring that
 # it exists.
@@ -29,7 +30,23 @@ st='	 '
 
 (sed -n \
   "/\\\$/d;s/#.*\$//;s/^[$st]*\\([A-Z][^:$st]*\\)[$st]*=[$st]*\\([^$st]*\\)[$st]*\$/\\1=\\2 export \\1/p" \
-  < Makefile ; echo "./buildconfig") | /bin/sh || exit 1
+  < Makefile ; echo "./buildconfig") | /bin/sh
+
+# If buildconfig ends with an error code, it will have output an error
+# message. Ensure that a broken config.h gets deleted.
+
+if [ $? != 0 ] ; then
+  rm -f config.h
+  exit 1
+fi
+
+# Double-check that config.h is complete.
+
+if [ "`tail -1 config.h`" != "/* End of config.h */" ] ; then
+  echo "*** config.h appears to be incomplete"
+  echo "*** unexpected failure in buildconfig program"
+  exit 1
+fi
 
 echo ">>> config.h built"
 echo ""
