@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/auths/call_pam.c,v 1.2 2005/01/04 10:00:43 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -22,14 +24,19 @@ to stop even pickier compilers complaining about infinite loops. */
 static void dummy(int x) { dummy(x-1); }
 #else  /* SUPPORT_PAM */
 
+#ifdef PAM_H_IN_PAM
+#include <pam/pam_appl.h>
+#else
 #include <security/pam_appl.h>
+#endif
 
 /* According to the specification, it should be possible to have an application
 data pointer passed to the conversation function. However, I was unable to get
 this to work on Solaris 2.6, so static variables are used instead. */
 
 static int pam_conv_had_error;
-static char *pam_args;
+static uschar *pam_args;
+static BOOL pam_arg_ended;
 
 
 
@@ -38,7 +45,10 @@ static char *pam_args;
 *************************************************/
 
 /* This function is passed to the PAM authentication function, and it calls it
-back when it wants data from the client.
+back when it wants data from the client. The string list is in pam_args. When
+we reach the end, we pass back an empty string once. If this function is called
+again, it will give an error response. This is protection against something
+crazy happening.
 
 Arguments:
   num_msg        number of messages associated with the call
@@ -57,20 +67,28 @@ pam_converse (int num_msg, PAM_CONVERSE_ARG2_TYPE **msg,
 {
 int i;
 int sep = 0;
-struct pam_response *reply = malloc(sizeof(struct pam_response) * num_msg);
+struct pam_response *reply;
+
+if (pam_arg_ended) return PAM_CONV_ERR;
+
+reply = malloc(sizeof(struct pam_response) * num_msg);
 
 if (reply == NULL) return PAM_CONV_ERR;
 
 for (i = 0; i < num_msg; i++)
   {
-  char *arg;
+  uschar *arg;
   switch (msg[i]->msg_style)
     {
     case PAM_PROMPT_ECHO_ON:
     case PAM_PROMPT_ECHO_OFF:
     arg = string_nextinlist(&pam_args, &sep, big_buffer, big_buffer_size);
-    if (arg == NULL) arg = "";
-    reply[i].resp = string_copy_malloc(arg); /* PAM frees resp */
+    if (arg == NULL)
+      {
+      arg = US"";
+      pam_arg_ended = TRUE;
+      }
+    reply[i].resp = CS string_copy_malloc(arg); /* PAM frees resp */
     reply[i].resp_retcode = PAM_SUCCESS;
     break;
 
@@ -110,13 +128,13 @@ Returns:   OK if authentication succeeded
 */
 
 int
-auth_call_pam(char *s, char **errptr)
+auth_call_pam(uschar *s, uschar **errptr)
 {
 pam_handle_t *pamh = NULL;
 struct pam_conv pamc;
 int pam_error;
 int sep = 0;
-char *user;
+uschar *user;
 
 /* Set up the input data structure: the address of the conversation function,
 and a pointer to application data, which we don't use because I couldn't get it
@@ -125,21 +143,26 @@ to work under Solaris 2.6 - it always arrived in pam_converse() as NULL. */
 pamc.conv = pam_converse;
 pamc.appdata_ptr = NULL;
 
-/* Initialize the static data - the current input data and error flag. */
+/* Initialize the static data - the current input data, the error flag, and the
+flag for data end. */
 
 pam_args = s;
 pam_conv_had_error = FALSE;
+pam_arg_ended = FALSE;
 
-/* The first string in the list is the user. */
+/* The first string in the list is the user. If this is an empty string, we
+fail. PAM doesn't support authentication with an empty user (it prompts for it,
+causing a potential mis-interpretation). */
 
 user = string_nextinlist(&pam_args, &sep, big_buffer, big_buffer_size);
-if (user == NULL) user = "";
+if (user == NULL || user[0] == 0) return FAIL;
 
 /* Start off PAM interaction */
 
-DEBUG(9) debug_printf("Running PAM authentication for user \"%s\"\n", user);
+DEBUG(D_auth)
+  debug_printf("Running PAM authentication for user \"%s\"\n", user);
 
-pam_error = pam_start ("exim", user, &pamc, &pamh);
+pam_error = pam_start ("exim", CS user, &pamc, &pamh);
 
 /* Do the authentication - the pam_authenticate() will call pam_converse() to
 get the data it wants. After successful authentication we call pam_acct_mgmt()
@@ -161,12 +184,12 @@ pam_end(pamh, PAM_SUCCESS);
 
 if (pam_error == PAM_SUCCESS)
   {
-  DEBUG(9) debug_printf("PAM success\n");
+  DEBUG(D_auth) debug_printf("PAM success\n");
   return OK;
   }
 
-*errptr = (char *)pam_strerror(pamh, pam_error);
-DEBUG(9) debug_printf("PAM error: %s\n", *errptr);
+*errptr = (uschar *)pam_strerror(pamh, pam_error);
+DEBUG(D_auth) debug_printf("PAM error: %s\n", *errptr);
 
 if (pam_error == PAM_USER_UNKNOWN ||
     pam_error == PAM_AUTH_ERR ||
