@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/auths/get_data.c,v 1.2 2005/01/04 10:00:43 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -17,7 +19,8 @@ to the SMTP client and read the response line.
 
 Arguments:
    aptr       set to point to the response (which is in big_buffer)
-   challenge  the challenge text (unencoded)
+   challenge  the challenge text (unencoded, may be binary)
+   challen    the length of the challenge text
 
 Returns:      OK on success
               BAD64 if response too large for buffer
@@ -25,11 +28,11 @@ Returns:      OK on success
 */
 
 int
-auth_get_data(uschar **aptr, uschar *challenge)
+auth_get_data(uschar **aptr, uschar *challenge, int challen)
 {
 int c;
 int p = 0;
-smtp_printf("334 %s\r\n", auth_b64encode(challenge, Ustrlen(challenge)));
+smtp_printf("334 %s\r\n", auth_b64encode(challenge, challen));
 while ((c = receive_getc()) != '\n' && c != EOF)
   {
   if (p >= big_buffer_size - 1) return BAD64;
