@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/auths/xtextencode.c,v 1.2 2005/01/04 10:00:43 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -13,8 +15,8 @@
 *************************************************/
 
 /* This function encodes a string of bytes, containing any values whatsoever,
-as "xtext" as defined in RFC 1891 (DSN)  and required by the SMTP AUTH
-extension (RFC 2554).
+as "xtext", as defined in RFC 1891 and required by the SMTP AUTH extension (RFC
+2554).
 
 Arguments:
   clear       points to the clear text bytes
@@ -24,12 +26,12 @@ Returns:      a pointer to the zero-terminated xtext string, which
               is in working store
 */
 
-char *
-auth_xtextencode(char *clear, int len)
+uschar *
+auth_xtextencode(uschar *clear, int len)
 {
-char *code;
+uschar *code;
 uschar *p = (uschar *)clear;
-char *pp;
+uschar *pp;
 int c = len;
 int count = 1;
 register int x;
@@ -48,7 +50,7 @@ while (c-- > 0)
   {
   if ((x = *p++) < 33 || x > 127 || x == '+' || x == '=')
     {
-    sprintf(pp, "+%.02x", x);   /* There's always room */
+    sprintf(CS pp, "+%.02x", x);   /* There's always room */
     pp += 3;
     }
   else *pp++ = x;
