@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/daemon.c,v 1.9 2005/02/17 11:58:25 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with running Exim as a daemon */
@@ -37,8 +39,8 @@ static int   accept_retry_errno;
 static BOOL  accept_retry_select_failed;
 
 static int   queue_run_count = 0;
-static pid_t *queue_pid_slots;
-static smtp_slot *smtp_slots;
+static pid_t *queue_pid_slots = NULL;
+static smtp_slot *smtp_slots = NULL;
 
 static BOOL  write_pid = TRUE;
 
@@ -82,8 +84,8 @@ static void
 main_sigchld_handler(int sig)
 {
 sig = sig;    /* Keep picky compilers happy */
+os_non_restarting_signal(SIGCHLD, SIG_DFL);
 sigchld_seen = TRUE;
-signal(SIGCHLD, SIG_DFL);
 }
 
 
@@ -143,6 +145,11 @@ union sockaddr_46 interface_sockaddr;
 SOCKLEN_T ifsize = sizeof(interface_sockaddr);
 int dup_accept_socket = -1;
 int max_for_this_host = 0;
+int wfsize = 0;
+int wfptr = 0;
+int use_log_write_selector = log_write_selector;
+uschar *whofrom = NULL;
+
 void *reset_point = store_get(0);
 
 /* Make the address available in ASCII representation, and also fish out
@@ -179,6 +186,37 @@ if (smtp_in == NULL)
   goto ERROR_RETURN;
   }
 
+/* Get the data for the local interface address. */
+
+if (getsockname(accept_socket, (struct sockaddr *)(&interface_sockaddr),
+     &ifsize) < 0)
+  {
+  log_write(0, LOG_MAIN|LOG_PANIC, "getsockname() failed: %s",
+    strerror(errno));
+  smtp_printf("421 Local problem: getsockname() failed; please try again later\r\n");
+  goto ERROR_RETURN;
+  }
+
+interface_address = host_ntoa(-1, &interface_sockaddr, NULL, &interface_port);
+DEBUG(D_interface) debug_printf("interface address=%s port=%d\n",
+  interface_address, interface_port);
+
+/* Build a string identifying the remote host and, if requested, the port and
+the local interface data. This is for logging; at the end of this function the
+memory is reclaimed. */
+
+whofrom = string_append(whofrom, &wfsize, &wfptr, 3, "[", sender_host_address, "]");
+
+if ((log_extra_selector & LX_incoming_port) != 0)
+  whofrom = string_append(whofrom, &wfsize, &wfptr, 2, ":", string_sprintf("%d",
+    sender_host_port));
+
+if ((log_extra_selector & LX_incoming_interface) != 0)
+  whofrom = string_append(whofrom, &wfsize, &wfptr, 4, " I=[",
+    interface_address, "]:", string_sprintf("%d", interface_port));
+
+whofrom[wfptr] = 0;    /* Terminate the newly-built string */
+
 /* Check maximum number of connections. We do not check for reserved
 connections or unacceptable hosts here. That is done in the subprocess because
 it might take some time. */
@@ -191,7 +229,7 @@ if (smtp_accept_max > 0 && smtp_accept_count >= smtp_accept_max)
     "please try again later.\r\n");
   log_write(L_connection_reject,
             LOG_MAIN, "Connection from %s refused: too many connections",
-    sender_host_address);
+    whofrom);
   goto ERROR_RETURN;
   }
 
@@ -210,7 +248,7 @@ if (smtp_load_reserve >= 0)
     smtp_printf("421 Too much load; please try again later.\r\n");
     log_write(L_connection_reject,
               LOG_MAIN, "Connection from %s refused: load average = %.2f",
-      sender_host_address, (double)load_average/1000.0);
+      whofrom, (double)load_average/1000.0);
     goto ERROR_RETURN;
     }
   }
@@ -230,7 +268,7 @@ if (smtp_accept_max_per_host != NULL)
     {
     if (!expand_string_forcedfail)
       log_write(0, LOG_MAIN|LOG_PANIC, "expansion of smtp_accept_max_per_host "
-        "failed for host %s: %s", sender_host_address, expand_string_message);
+        "failed for %s: %s", whofrom, expand_string_message);
     }
   /* For speed, interpret a decimal number inline here */
   else
@@ -240,7 +278,7 @@ if (smtp_accept_max_per_host != NULL)
       max_for_this_host = max_for_this_host * 10 + *s++ - '0';
     if (*s != 0)
       log_write(0, LOG_MAIN|LOG_PANIC, "expansion of smtp_accept_max_per_host "
-        "for host %s contains non-digit: %s", sender_host_address, expanded);
+        "for %s contains non-digit: %s", whofrom, expanded);
     }
   }
 
@@ -283,44 +321,32 @@ if ((max_for_this_host > 0) &&
       "from this IP address; please try again later.\r\n");
     log_write(L_connection_reject,
               LOG_MAIN, "Connection from %s refused: too many connections "
-      "from that IP address", sender_host_address);
+      "from that IP address", whofrom);
     goto ERROR_RETURN;
     }
   }
 
 /* OK, the connection count checks have been passed. Before we can fork the
 accepting process, we must first log the connection if requested. This logging
-used to happen in the subprocess, but doing that that means that the value of
+used to happen in the subprocess, but doing that means that the value of
 smtp_accept_count can be out of step by the time it is logged. So we have to do
 the logging here and accept the performance cost. Note that smtp_accept_count
 hasn't yet been incremented to take account of this connection.
 
 In order to minimize the cost (because this is going to happen for every
-connection), do a selector test here. This saves ploughing through the
-generalized logging code each time when the selector is false. */
+connection), do a preliminary selector test here. This saves ploughing through
+the generalized logging code each time when the selector is false. If the
+selector is set, check whether the host is on the list for logging. If not,
+arrange to unset the selector in the subprocess. */
 
 if ((log_write_selector & L_smtp_connection) != 0)
   {
-  if ((log_extra_selector & LX_incoming_port) == 0)
-    log_write(0, LOG_MAIN, "SMTP connection from [%s] "
-      "(TCP/IP connection count = %d)",
-      sender_host_address, smtp_accept_count + 1);
+  uschar *list = hosts_connection_nolog;
+  if (list != NULL && verify_check_host(&list) == OK)
+    use_log_write_selector &= ~L_smtp_connection;
   else
-    log_write(0, LOG_MAIN, "SMTP connection from [%s]:%d "
-      "(TCP/IP connection count = %d)",
-      sender_host_address, sender_host_port, smtp_accept_count + 1);
-  }
-
-/* Get the data for the local interface address. We do this in the
-main process so that we can easily give up if it fails. */
-
-if (getsockname(accept_socket, (struct sockaddr *)(&interface_sockaddr),
-     &ifsize) < 0)
-  {
-  log_write(0, LOG_MAIN|LOG_PANIC, "getsockname() failed: %s",
-    strerror(errno));
-  smtp_printf("421 Local problem: getsockname() failed; please try again later\r\n");
-  goto ERROR_RETURN;
+    log_write(L_smtp_connection, LOG_MAIN, "SMTP connection from %s "
+      "(TCP/IP connection count = %d)", whofrom, smtp_accept_count + 1);
   }
 
 /* Now we can fork the accepting process; do a lookup tidy, just in case any
@@ -335,19 +361,26 @@ if (pid == 0)
   {
   int i;
   int queue_only_reason = 0;
+  int old_pool = store_pool;
+  int save_debug_selector = debug_selector;
   BOOL local_queue_only;
   #ifdef SA_NOCLDWAIT
   struct sigaction act;
   #endif
 
-  /* Get the local interface address into some permanent store, and also set
-  the port. */
+  /* May have been modified for the subprocess */
 
-  uschar *buffer = store_get_perm(sizeof(ip_address_item));
-  interface_address = host_ntoa(-1, &interface_sockaddr, buffer,
-    &interface_port);
-  DEBUG(D_interface) debug_printf("interface address=%s port=%d\n",
-    interface_address, interface_port);
+  log_write_selector = use_log_write_selector;
+
+  /* Get the local interface address into permanent store */
+
+  store_pool = POOL_PERM;
+  interface_address = string_copy(interface_address);
+  store_pool = old_pool;
+
+  /* Check for a tls-on-connect port */
+
+  if (host_is_tls_on_connect_port(interface_port)) tls_on_connect = TRUE;
 
   /* Expand smtp_active_hostname if required. We do not do this any earlier,
   because it may depend on the local interface address (indeed, that is most
@@ -400,15 +433,26 @@ if (pid == 0)
   /* Attempt to get an id from the sending machine via the RFC 1413
   protocol. We do this in the sub-process in order not to hold up the
   main process if there is any delay. Then set up the fullhost information
-  in case there is no HELO/EHLO. */
+  in case there is no HELO/EHLO.
 
+  If debugging is enabled only for the daemon, we must turn if off while
+  finding the id, but turn it on again afterwards so that information about the
+  incoming connection is output. */
+
+  if (debug_daemon) debug_selector = 0;
   verify_get_ident(IDENT_PORT);
   host_build_sender_fullhost();
+  debug_selector = save_debug_selector;
 
   DEBUG(D_any)
     debug_printf("Process %d is handling incoming connection from %s\n",
       (int)getpid(), sender_fullhost);
 
+  /* Now disable debugging permanently if it's required only for the daemon
+  process. */
+
+  if (debug_daemon) debug_selector = 0;
+
   /* If there are too many child processes for immediate delivery,
   set the local_queue_only flag, which is initialized from the
   configured value and may therefore already be TRUE. Leave logging
@@ -452,7 +496,11 @@ if (pid == 0)
       {
       BOOL ok = receive_msg(FALSE);
       search_tidyup();                    /* Close cached databases */
-      if (!ok) _exit(EXIT_SUCCESS);       /* Connection was dropped */
+      if (!ok)                            /* Connection was dropped */
+        {
+        mac_smtp_fflush();
+        _exit(EXIT_SUCCESS);
+        }
       if (message_id[0] == 0) continue;   /* No message was accepted */
       }
     else
@@ -728,6 +776,72 @@ return FALSE;
 
 
 
+/*************************************************
+*         Handle terminating subprocesses        *
+*************************************************/
+
+/* Handle the termination of child processes. Theoretically, this need be done
+only when sigchld_seen is TRUE, but rumour has it that some systems lose
+SIGCHLD signals at busy times, so to be on the safe side, this function is
+called each time round. It shouldn't be too expensive.
+
+Arguments:  none
+Returns:    nothing
+*/
+
+static void
+handle_ending_processes(void)
+{
+int status;
+pid_t pid;
+
+while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
+  {
+  int i;
+  DEBUG(D_any) debug_printf("child %d ended: status=0x%x\n", (int)pid,
+    status);
+
+  /* If it's a listening daemon for which we are keeping track of individual
+  subprocesses, deal with an accepting process that has terminated. */
+
+  if (smtp_slots != NULL)
+    {
+    for (i = 0; i < smtp_accept_max; i++)
+      {
+      if (smtp_slots[i].pid == pid)
+        {
+        if (smtp_slots[i].host_address != NULL)
+          store_free(smtp_slots[i].host_address);
+        smtp_slots[i] = empty_smtp_slot;
+        if (--smtp_accept_count < 0) smtp_accept_count = 0;
+        DEBUG(D_any) debug_printf("%d SMTP accept process%s now running\n",
+          smtp_accept_count, (smtp_accept_count == 1)? "" : "es");
+        break;
+        }
+      }
+    if (i < smtp_accept_max) continue;  /* Found an accepting process */
+    }
+
+  /* If it wasn't an accepting process, see if it was a queue-runner
+  process that we are tracking. */
+
+  if (queue_pid_slots != NULL)
+    {
+    for (i = 0; i < queue_run_max; i++)
+      {
+      if (queue_pid_slots[i] == pid)
+        {
+        queue_pid_slots[i] = 0;
+        if (--queue_run_count < 0) queue_run_count = 0;
+        DEBUG(D_any) debug_printf("%d queue-runner process%s now running\n",
+          queue_run_count, (queue_run_count == 1)? "" : "es");
+        break;
+        }
+      }
+    }
+  }
+}
+
 
 
 /*************************************************
@@ -850,7 +964,9 @@ if (daemon_listen)
   order to perform an "open" on the kernel memory file). */
 
   #ifdef LOAD_AVG_NEEDS_ROOT
-  if (queue_only_load >= 0 || smtp_load_reserve >= 0) (void)os_getloadavg();
+  if (queue_only_load >= 0 || smtp_load_reserve >= 0 ||
+       (deliver_queue_load_max >= 0 && deliver_drop_privilege))
+    (void)os_getloadavg();
   #endif
 
   /* If -oX was used, disable the writing of a pid file unless -oP was
@@ -1074,26 +1190,35 @@ if (daemon_listen)
     }
   }
 
-/* We now close all open file descriptors that we know about, and disconnect
-from the controlling terminal, unless background_daemon is unset. This is
-always unset when debugging, but can also be forced. Most modern Unixes seem to
-have setsid() for getting rid of the controlling terminal. For any OS that
-doesn't, setsid() can be #defined as a no-op, or as something else. */
+/* The variable background_daemon is always false when debugging, but
+can also be forced false in order to keep a non-debugging daemon in the
+foreground. If background_daemon is true, close all open file descriptors that
+we know about, but then re-open stdin, stdout, and stderr to /dev/null.
+
+This is protection against any called functions (in libraries, or in
+Perl, or whatever) that think they can write to stderr (or stdout). Before this
+was added, it was quite likely that an SMTP connection would use one of these
+file descriptors, in which case writing random stuff to it caused chaos.
+
+Then disconnect from the controlling terminal, Most modern Unixes seem to have
+setsid() for getting rid of the controlling terminal. For any OS that doesn't,
+setsid() can be #defined as a no-op, or as something else. */
 
 if (background_daemon)
   {
-  log_close_all();  /* Just in case anything was logged earlier */
-  search_tidyup();  /* Just in case any were used in reading the config. */
-  close(0);         /* Get rid of stdin/stdout/stderr */
+  log_close_all();    /* Just in case anything was logged earlier */
+  search_tidyup();    /* Just in case any were used in reading the config. */
+  close(0);           /* Get rid of stdin/stdout/stderr */
   close(1);
   close(2);
+  exim_nullstd();     /* Connect stdin/stdout/stderr to /dev/null */
   log_stderr = NULL;  /* So no attempt to copy paniclog output */
 
   /* If the parent process of this one has pid == 1, we are re-initializing the
-  daemon as the result of a SIGHUP. In this case, there is no need to do any
-  forking, because the controlling terminal has long gone. Otherwise, fork,
-  in case current process is a process group leader (see 'man setsid' for an
-  explanation). */
+  daemon as the result of a SIGHUP. In this case, there is no need to do
+  anything, because the controlling terminal has long gone. Otherwise, fork, in
+  case current process is a process group leader (see 'man setsid' for an
+  explanation) before calling setsid(). */
 
   if (getppid() != 1)
     {
@@ -1329,7 +1454,7 @@ if (queue_interval > 0 && queue_run_max > 0)
 /* Set up the handler for termination of child processes. */
 
 sigchld_seen = FALSE;
-signal(SIGCHLD, main_sigchld_handler);
+os_non_restarting_signal(SIGCHLD, main_sigchld_handler);
 
 /* If we are to run the queue periodically, pretend the alarm has just gone
 off. This will cause the first queue-runner to get kicked off straight away. */
@@ -1341,7 +1466,9 @@ must be set up. */
 
 if (daemon_listen)
   {
-  int i;
+  int i, j;
+  int smtp_ports = 0;
+  int smtps_ports = 0;
   ip_address_item *ipa;
   uschar *p = big_buffer;
   uschar *qinfo = (queue_interval > 0)?
@@ -1349,44 +1476,74 @@ if (daemon_listen)
     :
     US"no queue runs";
 
-  #ifdef SUPPORT_TLS
-  uschar *sinfo = tls_on_connect ?
-      US"listening for SMTPS on" : US"listening for SMTP on";
-  #else
-  uschar *sinfo = US"listening for SMTP on";
-  #endif
-
   /* Build a list of listening addresses in big_buffer, but limit it to 10
-  items. The style is for backwards compatibility. */
+  items. The style is for backwards compatibility.
 
-  for (i = 0, ipa = addresses; i < 10 && ipa != NULL; i++, ipa = ipa->next)
-     {
-     if (ipa->address[0] == ':' && ipa->address[1] == 0)
+  It is now possible to have some ports listening for SMTPS (the old,
+  deprecated protocol that starts TLS without using STARTTLS), and others
+  listening for standard SMTP. Keep their listings separate. */
+
+  for (j = 0; j < 2; j++)
+    {
+    for (i = 0, ipa = addresses; i < 10 && ipa != NULL; i++, ipa = ipa->next)
        {
-       if (ipa->next != NULL && ipa->next->address[0] == 0 &&
-           ipa->next->port == ipa->port)
-         {
-         (void)sprintf(CS p, " port %d (IPv6 and IPv4)", ipa->port);
-         ipa = ipa->next;
-         }
-       else if (ipa->v6_include_v4)
-         (void)sprintf(CS p, " port %d (IPv6 with IPv4)", ipa->port);
-       else
-         (void)sprintf(CS p, " port %d (IPv6)", ipa->port);
-       }
-     else if (ipa->address[0] == 0)
-       (void)sprintf(CS p, " port %d (IPv4)", ipa->port);
-     else
-       (void)sprintf(CS p, " [%s]:%d", ipa->address, ipa->port);
-     while (*p != 0) p++;
-     }
+       /* First time round, look for SMTP ports; second time round, look for
+       SMTPS ports. For the first one of each, insert leading text. */
 
-  if (ipa != NULL) memcpy(p, " ...", 5);
+       if (host_is_tls_on_connect_port(ipa->port) == (j > 0))
+         {
+         if (j == 0)
+           {
+           if (smtp_ports++ == 0)
+             {
+             memcpy(p, "SMTP on", 8);
+             p += 7;
+             }
+           }
+         else
+           {
+           if (smtps_ports++ == 0)
+             {
+             (void)sprintf(CS p, "%sSMTPS on",
+               (smtp_ports == 0)? "":" and for ");
+             while (*p != 0) p++;
+             }
+           }
+
+         /* Now the information about the port (and sometimes interface) */
+
+         if (ipa->address[0] == ':' && ipa->address[1] == 0)
+           {
+           if (ipa->next != NULL && ipa->next->address[0] == 0 &&
+               ipa->next->port == ipa->port)
+             {
+             (void)sprintf(CS p, " port %d (IPv6 and IPv4)", ipa->port);
+             ipa = ipa->next;
+             }
+           else if (ipa->v6_include_v4)
+             (void)sprintf(CS p, " port %d (IPv6 with IPv4)", ipa->port);
+           else
+             (void)sprintf(CS p, " port %d (IPv6)", ipa->port);
+           }
+         else if (ipa->address[0] == 0)
+           (void)sprintf(CS p, " port %d (IPv4)", ipa->port);
+         else
+           (void)sprintf(CS p, " [%s]:%d", ipa->address, ipa->port);
+         while (*p != 0) p++;
+         }
+       }
+
+    if (ipa != NULL)
+      {
+      memcpy(p, " ...", 5);
+      p += 4;
+      }
+    }
 
   log_write(0, LOG_MAIN,
-    "exim %s daemon started: pid=%d, %s, %s%s",
-    version_string, getpid(), qinfo, sinfo, big_buffer);
-  set_process_info("daemon: %s, %s%s", qinfo, sinfo, big_buffer);
+    "exim %s daemon started: pid=%d, %s, listening for %s",
+    version_string, getpid(), qinfo, big_buffer);
+  set_process_info("daemon: %s, listening for %s", qinfo, big_buffer);
   }
 
 else
@@ -1399,9 +1556,9 @@ else
   }
 
 
-/* Close the log so it can be renamed and moved. This process doesn't write
-to the log again, unless it is about to die or exec and in the latter case
-it closes the log first. */
+/* Close the log so it can be renamed and moved. In the few cases below where
+this long-running process writes to the log (always exceptional conditions), it
+closes the log afterwards, for the same reason. */
 
 log_close_all();
 
@@ -1422,7 +1579,6 @@ for (;;)
   #endif
 
   SOCKLEN_T len = sizeof(accepted);
-  int status;
   pid_t pid;
 
   /* This code is placed first in the loop, so that it gets obeyed at the
@@ -1443,9 +1599,16 @@ for (;;)
       if ((pid = fork()) == 0)
         {
         int sk;
+
         DEBUG(D_any) debug_printf("Starting queue-runner: pid %d\n",
           (int)getpid());
 
+        /* Disable debugging if it's required only for the daemon process. We
+        leave the above message, because it ties up with the "child ended"
+        debugging messages. */
+
+        if (debug_daemon) debug_selector = 0;
+
         /* Close any open listening sockets in the child */
 
         for (sk = 0; sk < listen_socket_count; sk++) close(listen_sockets[sk]);
@@ -1487,6 +1650,7 @@ for (;;)
         {
         log_write(0, LOG_MAIN|LOG_PANIC, "daemon: fork of queue-runner "
           "process failed: %s", strerror(errno));
+        log_close_all();
         }
       else
         {
@@ -1523,7 +1687,7 @@ for (;;)
 
   if (daemon_listen)
     {
-    int sk, lcount;
+    int sk, lcount, select_errno;
     int max_socket = 0;
     BOOL select_failed = FALSE;
     fd_set select_listen;
@@ -1537,16 +1701,44 @@ for (;;)
 
     DEBUG(D_any) debug_printf("Listening...\n");
 
-    if ((lcount = select(max_socket + 1, (SELECT_ARG2_TYPE *)&select_listen,
-         NULL, NULL, NULL)) < 0)
+    /* In rare cases we may have had a SIGCHLD signal in the time between
+    setting the handler (below) and getting back here. If so, pretend that the
+    select() was interrupted so that we reap the child. This might still leave
+    a small window when a SIGCHLD could get lost. However, since we use SIGCHLD
+    only to do the reaping more quickly, it shouldn't result in anything other
+    than a delay until something else causes a wake-up. */
+
+    if (sigchld_seen)
+      {
+      lcount = -1;
+      errno = EINTR;
+      }
+    else
+      {
+      lcount = select(max_socket + 1, (SELECT_ARG2_TYPE *)&select_listen,
+        NULL, NULL, NULL);
+      }
+
+    if (lcount < 0)
       {
       select_failed = TRUE;
       lcount = 1;
       }
 
+    /* Clean up any subprocesses that may have terminated. We need to do this
+    here so that smtp_accept_max_per_host works when a connection to that host
+    has completed, and we are about to accept a new one. When this code was
+    later in the sequence, a new connection could be rejected, even though an
+    old one had just finished. Preserve the errno from any select() failure for
+    the use of the common select/accept error processing below. */
+
+    select_errno = errno;
+    handle_ending_processes();
+    errno = select_errno;
+
     /* Loop for all the sockets that are currently ready to go. If select
-    actually failed, we have set the count to 1 and a flag, so as to use the
-    common error code for select/accept below. */
+    actually failed, we have set the count to 1 and select_failed=TRUE, so as
+    to use the common error code for select/accept below. */
 
     while (lcount-- > 0)
       {
@@ -1594,6 +1786,7 @@ for (;;)
               accept_retry_select_failed? "select" : "accept",
               (accept_retry_count == 1)? "" : "s",
               strerror(accept_retry_errno));
+            log_close_all();
             accept_retry_count = 0;
             accept_retry_errno = errno;
             accept_retry_select_failed = select_failed;
@@ -1611,6 +1804,7 @@ for (;;)
             accept_retry_select_failed? "select" : "accept",
             (accept_retry_count == 1)? "" : "s",
             strerror(accept_retry_errno));
+          log_close_all();
           accept_retry_count = 0;
           }
         }
@@ -1636,57 +1830,7 @@ for (;;)
     tv.tv_sec = queue_interval;
     tv.tv_usec = 0;
     select(0, NULL, NULL, NULL, &tv);
-    }
-
-  /* Handle the termination of a child process. Theoretically, this need
-  be done only when sigchld_seen is TRUE, but rumour has it that some systems
-  lose SIGCHLD signals at busy times, so to be on the safe side, just
-  do it each time round. It shouldn't be too expensive. */
-
-  while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
-    {
-    int i;
-    DEBUG(D_any) debug_printf("child %d ended: status=0x%x\n", (int)pid,
-      status);
-
-    /* If it's a listening daemon, deal with an accepting process that has
-    terminated. */
-
-    if (daemon_listen)
-      {
-      for (i = 0; i < smtp_accept_max; i++)
-        {
-        if (smtp_slots[i].pid == pid)
-          {
-          if (smtp_slots[i].host_address != NULL)
-            store_free(smtp_slots[i].host_address);
-          smtp_slots[i] = empty_smtp_slot;
-          if (--smtp_accept_count < 0) smtp_accept_count = 0;
-          DEBUG(D_any) debug_printf("%d SMTP accept process%s now running\n",
-            smtp_accept_count, (smtp_accept_count == 1)? "" : "es");
-          break;
-          }
-        }
-      if (i < smtp_accept_max) continue;  /* Found an accepting process */
-      }
-
-    /* If it wasn't an accepting process, see if it was a queue-runner
-    process, if we are keeping track of them. */
-
-    if (queue_interval > 0)
-      {
-      for (i = 0; i < queue_run_max; i++)
-        {
-        if (queue_pid_slots[i] == pid)
-          {
-          queue_pid_slots[i] = 0;
-          if (--queue_run_count < 0) queue_run_count = 0;
-          DEBUG(D_any) debug_printf("%d queue-runner process%s now running\n",
-            queue_run_count, (queue_run_count == 1)? "" : "es");
-          break;
-          }
-        }
-      }
+    handle_ending_processes();
     }
 
   /* Re-enable the SIGCHLD handler if it has been run. It can't do it
@@ -1695,7 +1839,7 @@ for (;;)
   if (sigchld_seen)
     {
     sigchld_seen = FALSE;
-    signal(SIGCHLD, main_sigchld_handler);
+    os_non_restarting_signal(SIGCHLD, main_sigchld_handler);
     }
 
   /* Handle being woken by SIGHUP. We know at this point that the result
@@ -1703,7 +1847,8 @@ for (;;)
   closing the listening sockets so that they can be reused. Cancel any pending
   alarm in case it is just about to go off, and set SIGHUP to be ignored so
   that another HUP in quick succession doesn't clobber the new daemon before it
-  gets going. All log files get closed by the close-on-exec flag. */
+  gets going. All log files get closed by the close-on-exec flag; however, if
+  the exec fails, we need to close the logs. */
 
   if (sighup_seen)
     {
@@ -1718,6 +1863,7 @@ for (;;)
     execv(CS exim_path, (char *const *)sighup_argv);
     log_write(0, LOG_MAIN|LOG_PANIC_DIE, "pid %d: exec of %s failed: %s",
       getpid(), exim_path, strerror(errno));
+    log_close_all();
     }
 
   }   /* End of main loop */
