@@ -1,20 +1,21 @@
+/* $Cambridge: exim/exim-src/src/dbfn.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
 #include "exim.h"
-#include <setjmp.h>
 
 
-/* Functions for accessing Exim's database, which consists of a number
-of different DBM files. This module does not contain code for reading DBM
-files for (e.g.) alias expansion. That is all contained within the general
-search functions. As Exim now has support for the ndbm interface and the native
-db 1.85 and 2.x interfaces, all the relevant functions are called as macros.
+/* Functions for accessing Exim's hints database, which consists of a number of
+different DBM files. This module does not contain code for reading DBM files
+for (e.g.) alias expansion. That is all contained within the general search
+functions. As Exim now has support for several DBM interfaces, all the relevant
+functions are called as macros.
 
 All the data in Exim's database is in the nature of *hints*. Therefore it
 doesn't matter if it gets destroyed by accident. These functions are not
@@ -31,28 +32,25 @@ general want to do more than one read or write while holding the lock, there
 are separate open and close functions. However, the calling modules should
 arrange to hold the locks for the bare minimum of time. */
 
-/* Environment save for the longjmp() that catches segv errors while reading.
-This was implemented in the days when locking was on the DBM files themselves;
-this is now believed to have been the cause of such errors, but leave the code
-in existence just in case. */
-
-static jmp_buf read_env;
 
 
 /*************************************************
-*        Signal handler for segv while reading   *
+*         Berkeley DB error callback             *
 *************************************************/
 
-/* This is used only while reading from a dbm file; segv crashes have
-occurred and this is a damage limitation exercise. With the change to locking
-on a separate file, this is believed not to be likely to happen any more. */
+/* For Berkeley DB >= 2, we can define a function to be called in case of DB
+errors. This should help with debugging strange DB problems, e.g. getting "File
+exists" when you try to open a db file. */
 
-static void
-segv_handler(int signal)
+#if defined(USE_DB) && defined(DB_VERSION_STRING)
+void
+dbfn_bdb_error_callback(const char *pfx, char *msg)
 {
-signal = signal;       /* Keep picky compilers happy */
-longjmp(read_env, 1);
+pfx = pfx;
+log_write(0, LOG_MAIN, "Berkeley DB error: %s", msg);
 }
+#endif
+
 
 
 
@@ -64,7 +62,8 @@ longjmp(read_env, 1);
 
 Arguments:
   name     The single-component name of one of Exim's database files.
-  flags    Standard O_xxx flags indicating the type of open required.
+  flags    Either O_RDONLY or O_RDWR, indicating the type of open required;
+             O_RDWR implies "create if necessary"
   dbblock  Points to an open_db block to be filled in.
   lof      If TRUE, write to the log for actual open failures (locking failures
            are always logged).
@@ -77,44 +76,42 @@ Returns:   NULL if the open failed, or the locking failed. After locking
 */
 
 open_db *
-dbfn_open(char *name, int flags, open_db *dbblock, BOOL lof)
+dbfn_open(uschar *name, int flags, open_db *dbblock, BOOL lof)
 {
-int rc;
+int rc, save_errno;
 BOOL read_only = flags == O_RDONLY;
+BOOL created = FALSE;
 flock_t lock_data;
-char buffer[256];
+uschar buffer[256];
 
 /* The first thing to do is to open a separate file on which to lock. This
 ensures that Exim has exclusive use of the database before it even tries to
-open it. Previous versions tried to lock on the open database itself, but that
+open it. Early versions tried to lock on the open database itself, but that
 gave rise to mysterious problems from time to time - it was suspected that some
 DB libraries "do things" on their open() calls which break the interlocking.
-The lock file is never written to, but we open it for writing to ensure it gets
-created, and we also try to make the directory if the initial open fails. */
+The lock file is never written to, but we open it for writing so we can get a
+write lock if required. If it does not exist, we create it. This is done
+separately so we know when we have done it, because when running as root we
+need to change the ownership - see the bottom of this function. We also try to
+make the directory as well, just in case. We won't be doing this many times
+unnecessarily, because usually the lock file will be there. If the directory
+exists, there is no error. */
 
-sprintf(buffer, "%s/db/%s.lockfile", spool_directory, name);
-if ((dbblock->lockfd = open(buffer, O_RDWR|O_CREAT, DB_LOCKFILE_MODE)) < 0)
+sprintf(CS buffer, "%s/db/%s.lockfile", spool_directory, name);
+
+if ((dbblock->lockfd = Uopen(buffer, O_RDWR, EXIMDB_LOCKFILE_MODE)) < 0)
   {
-  (void)directory_make(spool_directory, "db", DB_DIRECTORY_MODE, TRUE);
-  dbblock->lockfd = open(buffer, O_RDWR|O_CREAT, DB_LOCKFILE_MODE);
+  created = TRUE;
+  (void)directory_make(spool_directory, US"db", EXIMDB_DIRECTORY_MODE, TRUE);
+  dbblock->lockfd = Uopen(buffer, O_RDWR|O_CREAT, EXIMDB_LOCKFILE_MODE);
   }
 
-/* If the open for writing fails with permission denied, and this is a
-read-only call, try to open the file for reading instead. This allows group
-read access the databases for utilities like exim_dumpdb to work without having
-to set up write access to the lock file. */
-
 if (dbblock->lockfd < 0)
   {
-  if (errno == EACCES && read_only)
-    dbblock->lockfd = open(buffer, O_RDONLY, DB_LOCKFILE_MODE);
-  if (dbblock->lockfd < 0)
-    {
-    log_write(0, LOG_MAIN, "%s",
-      string_open_failed(errno, "database lock file %s", buffer));
-    errno = 0;      /* Indicates locking failure */
-    return NULL;
-    }
+  log_write(0, LOG_MAIN, "%s",
+    string_open_failed(errno, "database lock file %s", buffer));
+  errno = 0;      /* Indicates locking failure */
+  return NULL;
   }
 
 /* Now we must get a lock on the opened lock file; do this with a blocking
@@ -123,12 +120,13 @@ lock that times out. */
 lock_data.l_type = read_only? F_RDLCK : F_WRLCK;
 lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
 
+DEBUG(D_hints_lookup|D_retry|D_route|D_deliver)
+  debug_printf("locking %s\n", buffer);
+
 sigalrm_seen = FALSE;
-os_non_restarting_signal(SIGALRM, sigalrm_handler);
-alarm(DB_LOCK_TIMEOUT_EXIM);
+alarm(EXIMDB_LOCK_TIMEOUT);
 rc = fcntl(dbblock->lockfd, F_SETLKW, &lock_data);
 alarm(0);
-signal(SIGALRM, SIG_IGN);
 
 if (sigalrm_seen) errno = ETIMEDOUT;
 if (rc < 0)
@@ -141,38 +139,94 @@ if (rc < 0)
   return NULL;
   }
 
-DEBUG(9) debug_printf("locked %s\n", buffer);
+DEBUG(D_hints_lookup) debug_printf("locked %s\n", buffer);
 
 /* At this point we have an opened and locked separate lock file, that is,
-exclusive access to the database, so we can go ahead and open it. */
+exclusive access to the database, so we can go ahead and open it. If we are
+expected to create it, don't do so at first, again so that we can detect
+whether we need to change its ownership (see comments about the lock file
+above.) */
 
-sprintf(buffer, "%s/db/%s", spool_directory, name);
-EXIM_DBOPEN(buffer, flags, DB_MODE, &(dbblock->dbptr));
+sprintf(CS buffer, "%s/db/%s", spool_directory, name);
+EXIM_DBOPEN(buffer, flags, EXIMDB_MODE, &(dbblock->dbptr));
 
-/* If we're using GDBM, then opening for reading doesn't create a new file.
-Arguably, this is correct behaviour. If this happens, we just create the
-file, leaving it open for both reading and writing. */
+if (dbblock->dbptr == NULL && errno == ENOENT && flags == O_RDWR)
+  {
+  DEBUG(D_hints_lookup)
+    debug_printf("%s appears not to exist: trying to create\n", buffer);
+  created = TRUE;
+  EXIM_DBOPEN(buffer, flags|O_CREAT, EXIMDB_MODE, &(dbblock->dbptr));
+  }
 
-if (dbblock->dbptr == NULL && errno == ENOENT && flags == O_RDONLY)
-  EXIM_DBOPEN(buffer, O_RDWR|O_CREAT, DB_MODE, &(dbblock->dbptr));
+save_errno = errno;
+
+/* If we are running as root and this is the first access to the database, its
+files will be owned by root. We want them to be owned by exim. We detect this
+situation by noting above when we had to create the lock file or the database
+itself. Because the different dbm libraries use different extensions for their
+files, I don't know of any easier way of arranging this than scanning the
+directory for files with the appropriate base name. At least this deals with
+the lock file at the same time. Also, the directory will typically have only
+half a dozen files, so the scan will be quick.
+
+This code is placed here, before the test for successful opening, because there
+was a case when a file was created, but the DBM library still returned NULL
+because of some problem. It also sorts out the lock file if that was created
+but creation of the database file failed. */
+
+if (created && geteuid() == root_uid)
+  {
+  DIR *dd;
+  struct dirent *ent;
+  uschar *lastname = Ustrrchr(buffer, '/') + 1;
+  int namelen = Ustrlen(name);
+
+  *lastname = 0;
+  dd = opendir(CS buffer);
+
+  while ((ent = readdir(dd)) != NULL)
+    {
+    if (Ustrncmp(ent->d_name, name, namelen) == 0)
+      {
+      struct stat statbuf;
+      Ustrcpy(lastname, ent->d_name);
+      if (Ustat(buffer, &statbuf) >= 0 && statbuf.st_uid != exim_uid)
+        {
+        DEBUG(D_hints_lookup) debug_printf("ensuring %s is owned by exim\n", buffer);
+        Uchown(buffer, exim_uid, exim_gid);
+        }
+      }
+    }
+
+  closedir(dd);
+  }
 
 /* If the open has failed, return NULL, leaving errno set. If lof is TRUE,
 log the event - also for debugging - but not if the file just doesn't exist. */
 
 if (dbblock->dbptr == NULL)
   {
-  int save_errno = errno;
-  if ((lof || debug_level >= 9) && errno != ENOENT)
-    log_write(0, LOG_MAIN, "%s",
-      string_open_failed(errno, "DB file %s", buffer));
+  if (save_errno != ENOENT)
+    {
+    if (lof)
+      log_write(0, LOG_MAIN, "%s", string_open_failed(save_errno, "DB file %s",
+        buffer));
+    else
+      DEBUG(D_hints_lookup)
+        debug_printf("%s", CS string_open_failed(save_errno, "DB file %s\n",
+          buffer));
+    }
   close(dbblock->lockfd);
   errno = save_errno;
   return NULL;
   }
 
-/* All is well */
+DEBUG(D_hints_lookup)
+  debug_printf("opened hints database %s: flags=%x\n", buffer, flags);
+
+/* Pass back the block containing the opened database handle and the open fd
+for the lock. */
 
-DEBUG(9) debug_printf("opened DB file %s: flags=%x\n", buffer, flags);
 return dbblock;
 }
 
@@ -222,51 +276,26 @@ Returns: a pointer to the retrieved record, or
 */
 
 void *
-dbfn_read_with_length(open_db *dbblock, char *key, int *length)
+dbfn_read_with_length(open_db *dbblock, uschar *key, int *length)
 {
 void *yield;
 EXIM_DATUM key_datum, result_datum;
 
-DEBUG(9) debug_printf("dbfn_read: key=%s\n", key);
+DEBUG(D_hints_lookup) debug_printf("dbfn_read: key=%s\n", key);
 
 EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require the datum */
 EXIM_DATUM_INIT(result_datum);      /* to be cleared before use. */
-EXIM_DATUM_DATA(key_datum) = key;
-EXIM_DATUM_SIZE(key_datum) = (int)strlen(key) + 1;
+EXIM_DATUM_DATA(key_datum) = CS key;
+EXIM_DATUM_SIZE(key_datum) = Ustrlen(key) + 1;
 
-/* There have been problems on various systems that manifest themselves
-as SEGV crashes inside DBM libraries. As yet it is unknown whether these
-are the fault of Exim or not. However, in order to limit the amount of damage
-that such crashes cause, we attempt to catch SEGV signals here, and if one
-happens, log the incident and behave as if the record did not exist in the
-database. To do this we have to use the longjmp() mechanism. */
+if (!EXIM_DBGET(dbblock->dbptr, key_datum, result_datum)) return NULL;
 
-if (setjmp(read_env) == 0)
-  {
-  BOOL found;
-  signal(SIGSEGV, segv_handler);
-  found = EXIM_DBGET(dbblock->dbptr, key_datum, result_datum);
-  signal(SIGSEGV, SIG_DFL);
-  if (!found) return NULL;
-  yield = store_get(EXIM_DATUM_SIZE(result_datum));
-  memcpy(yield, EXIM_DATUM_DATA(result_datum), EXIM_DATUM_SIZE(result_datum));
-  if (length != NULL) *length = EXIM_DATUM_SIZE(result_datum);
-  EXIM_DATUM_FREE(result_datum);    /* Some DBM libs require freeing */
-  return yield;
-  }
+yield = store_get(EXIM_DATUM_SIZE(result_datum));
+memcpy(yield, EXIM_DATUM_DATA(result_datum), EXIM_DATUM_SIZE(result_datum));
+if (length != NULL) *length = EXIM_DATUM_SIZE(result_datum);
 
-/* Caught a SEGV. Returning NULL from here is often interperted as "no retry
-data for this domain", and that might cause frequent subsequent attempts to
-read the same data. That in turn could cause a heck of a lot of data to get
-written to the logs. Therefore, insert a short delay. */
-
-else
-  {
-  log_write(0, LOG_MAIN|LOG_PANIC, "SEGV while reading \"%s\" from "
-    "dbm file: record assumed not to exist", key);
-  sleep(1);
-  return NULL;
-  }
+EXIM_DATUM_FREE(result_datum);    /* Some DBM libs require freeing */
+return yield;
 }
 
 
@@ -287,19 +316,19 @@ Returns:    the yield of the underlying dbm or db "write" function. If this
 */
 
 int
-dbfn_write(open_db *dbblock, char *key, void *ptr, int length)
+dbfn_write(open_db *dbblock, uschar *key, void *ptr, int length)
 {
 EXIM_DATUM key_datum, value_datum;
 dbdata_generic *gptr = (dbdata_generic *)ptr;
 gptr->time_stamp = time(NULL);
 
-DEBUG(9) debug_printf("dbfn_write: key=%s\n", key);
+DEBUG(D_hints_lookup) debug_printf("dbfn_write: key=%s\n", key);
 
 EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require the datum */
 EXIM_DATUM_INIT(value_datum);       /* to be cleared before use. */
-EXIM_DATUM_DATA(key_datum) = key;
-EXIM_DATUM_SIZE(key_datum) = (int)strlen(key) + 1;
-EXIM_DATUM_DATA(value_datum) = (char *)ptr;
+EXIM_DATUM_DATA(key_datum) = CS key;
+EXIM_DATUM_SIZE(key_datum) = Ustrlen(key) + 1;
+EXIM_DATUM_DATA(value_datum) = CS ptr;
 EXIM_DATUM_SIZE(value_datum) = length;
 return EXIM_DBPUT(dbblock->dbptr, key_datum, value_datum);
 }
@@ -319,12 +348,12 @@ Returns: the yield of the underlying dbm or db "delete" function.
 */
 
 int
-dbfn_delete(open_db *dbblock, char *key)
+dbfn_delete(open_db *dbblock, uschar *key)
 {
 EXIM_DATUM key_datum;
 EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require clearing */
-EXIM_DATUM_DATA(key_datum) = key;
-EXIM_DATUM_SIZE(key_datum) = (int)strlen(key) + 1;
+EXIM_DATUM_DATA(key_datum) = CS key;
+EXIM_DATUM_SIZE(key_datum) = Ustrlen(key) + 1;
 return EXIM_DBDEL(dbblock->dbptr, key_datum);
 }
 
@@ -346,11 +375,11 @@ Returns:   the next record from the file, or
            NULL if there are no more
 */
 
-char *
+uschar *
 dbfn_scan(open_db *dbblock, BOOL start, EXIM_CURSOR **cursor)
 {
 EXIM_DATUM key_datum, value_datum;
-char *yield;
+uschar *yield;
 value_datum = value_datum;    /* dummy; not all db libraries use this */
 
 /* Some dbm require an initialization */
@@ -361,7 +390,7 @@ EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require the datum */
 EXIM_DATUM_INIT(value_datum);       /* to be cleared before use. */
 
 yield = (EXIM_DBSCAN(dbblock->dbptr, key_datum, value_datum, start, *cursor))?
-  EXIM_DATUM_DATA(key_datum) : NULL;
+  US EXIM_DATUM_DATA(key_datum) : NULL;
 
 /* Some dbm require a termination */
 
@@ -379,25 +408,8 @@ return yield;
 
 #ifdef STAND_ALONE
 
-
-void
-stat_file(char *name)
-{
-struct stat statbuf;
-if (stat(name, &statbuf) != 0)
-  {
-  printf("Failed to stat %s: %s\n", name, strerror(errno));
-  return;
-  }
-printf("%s: uid = %ld gid = %ld mode = %04o size=%d blksize=%d\n",
-  name, (long int)(statbuf.st_uid), (long int)(statbuf.st_gid),
-  (int)statbuf.st_mode & 07777,
-  (int)statbuf.st_size, (int)statbuf.st_blksize);
-}
-
-
 int
-main(int argc, char **argv)
+main(int argc, char **cargv)
 {
 open_db dbblock[8];
 int max_db = sizeof(dbblock)/sizeof(open_db);
@@ -405,8 +417,9 @@ int current = -1;
 int showtime = 0;
 int i;
 dbdata_wait *dbwait = NULL;
-char buffer[256];
-char structbuffer[1024];
+uschar **argv = USS cargv;
+uschar buffer[256];
+uschar structbuffer[1024];
 
 if (argc != 2)
   {
@@ -419,7 +432,7 @@ if (argc != 2)
 /* Initialize */
 
 spool_directory = argv[1];
-debug_level = 10;
+debug_selector = D_all - D_memory;
 debug_file = stderr;
 big_buffer = malloc(big_buffer_size);
 
@@ -441,44 +454,39 @@ printf("probably ndbm\n");
 #elif defined(USE_TDB)
 printf("using tdb\n");
 #else
-printf("probably GDBM\n");
+  #ifdef USE_GDBM
+  printf("probably GDBM (native mode)\n");
+  #else
+  printf("probably GDBM (compatibility mode)\n");
+  #endif
 #endif
 
-/* Show the effects of certain macros */
-
-#ifdef USE_DB
-printf("USE_DB is defined\n");
-#else
-printf("USE_DB is not defined\n");
-#endif
-
-
 /* Test the functions */
 
 printf("\nTest the functions\n> ");
 
-while (fgets(buffer, 256, stdin) != NULL)
+while (Ufgets(buffer, 256, stdin) != NULL)
   {
-  int len = (int)strlen(buffer);
+  int len = Ustrlen(buffer);
   int count = 1;
   clock_t start = 1;
   clock_t stop = 0;
-  char *cmd = buffer;
+  uschar *cmd = buffer;
   while (len > 0 && isspace((uschar)buffer[len-1])) len--;
   buffer[len] = 0;
 
   if (isdigit((uschar)*cmd))
     {
-    count = atoi(cmd);
+    count = Uatoi(cmd);
     while (isdigit((uschar)*cmd)) cmd++;
     while (isspace((uschar)*cmd)) cmd++;
     }
 
-  if (strncmp(cmd, "open", 4) == 0)
+  if (Ustrncmp(cmd, "open", 4) == 0)
     {
     int i;
     open_db *odb;
-    char *s = cmd + 4;
+    uschar *s = cmd + 4;
     while (isspace((uschar)*s)) s++;
 
     for (i = 0; i < max_db; i++)
@@ -491,7 +499,7 @@ while (fgets(buffer, 256, stdin) != NULL)
       }
 
     start = clock();
-    odb = dbfn_open(s, O_RDWR|O_CREAT, dbblock + i, TRUE);
+    odb = dbfn_open(s, O_RDWR, dbblock + i, TRUE);
     stop = clock();
 
     if (odb != NULL)
@@ -499,13 +507,24 @@ while (fgets(buffer, 256, stdin) != NULL)
       current = i;
       printf("opened %d\n", current);
       }
+    /* Other error cases will have written messages */
+    else if (errno == ENOENT)
+      {
+      printf("open failed: %s%s\n", strerror(errno),
+        #ifdef USE_DB
+        " (or other Berkeley DB error)"
+        #else
+        ""
+        #endif
+        );
+      }
     }
 
-  else if (strncmp(cmd, "write", 5) == 0)
+  else if (Ustrncmp(cmd, "write", 5) == 0)
     {
     int rc = 0;
-    char *key = cmd + 5;
-    char *data;
+    uschar *key = cmd + 5;
+    uschar *data;
 
     if (current < 0)
       {
@@ -520,19 +539,19 @@ while (fgets(buffer, 256, stdin) != NULL)
     while (isspace((uschar)*data)) data++;
 
     dbwait = (dbdata_wait *)(&structbuffer);
-    strcpy(dbwait->text, data);
+    Ustrcpy(dbwait->text, data);
 
     start = clock();
     while (count-- > 0)
       rc = dbfn_write(dbblock + current, key, dbwait,
-        (int)strlen(data) + sizeof(dbdata_wait));
+        Ustrlen(data) + sizeof(dbdata_wait));
     stop = clock();
     if (rc != 0) printf("Failed: %s\n", strerror(errno));
     }
 
-  else if (strncmp(cmd, "read", 4) == 0)
+  else if (Ustrncmp(cmd, "read", 4) == 0)
     {
-    char *key = cmd + 4;
+    uschar *key = cmd + 4;
     if (current < 0)
       {
       printf("No current database\n");
@@ -543,12 +562,12 @@ while (fgets(buffer, 256, stdin) != NULL)
     while (count-- > 0)
       dbwait = (dbdata_wait *)dbfn_read_with_length(dbblock+ current, key, NULL);
     stop = clock();
-    printf("%s\n", (dbwait == NULL)? "<not found>" : dbwait->text);
+    printf("%s\n", (dbwait == NULL)? "<not found>" : CS dbwait->text);
     }
 
-  else if (strncmp(cmd, "delete", 6) == 0)
+  else if (Ustrncmp(cmd, "delete", 6) == 0)
     {
-    char *key = cmd + 6;
+    uschar *key = cmd + 6;
     if (current < 0)
       {
       printf("No current database\n");
@@ -558,12 +577,12 @@ while (fgets(buffer, 256, stdin) != NULL)
     dbfn_delete(dbblock + current, key);
     }
 
-  else if (strncmp(cmd, "scan", 4) == 0)
+  else if (Ustrncmp(cmd, "scan", 4) == 0)
     {
     EXIM_CURSOR *cursor;
     BOOL startflag = TRUE;
-    char *key;
-    char keybuffer[256];
+    uschar *key;
+    uschar keybuffer[256];
     if (current < 0)
       {
       printf("No current database\n");
@@ -573,7 +592,7 @@ while (fgets(buffer, 256, stdin) != NULL)
     while ((key = dbfn_scan(dbblock + current, startflag, &cursor)) != NULL)
       {
       startflag = FALSE;
-      strcpy(keybuffer, key);
+      Ustrcpy(keybuffer, key);
       dbwait = (dbdata_wait *)dbfn_read_with_length(dbblock + current,
         keybuffer, NULL);
       printf("%s: %s\n", keybuffer, dbwait->text);
@@ -582,11 +601,11 @@ while (fgets(buffer, 256, stdin) != NULL)
     printf("End of scan\n");
     }
 
-  else if (strncmp(cmd, "close", 5) == 0)
+  else if (Ustrncmp(cmd, "close", 5) == 0)
     {
-    char *s = cmd + 5;
+    uschar *s = cmd + 5;
     while (isspace((uschar)*s)) s++;
-    i = atoi(s);
+    i = Uatoi(s);
     if (i >= max_db || dbblock[i].dbptr == NULL) printf("Not open\n"); else
       {
       start = clock();
@@ -597,34 +616,41 @@ while (fgets(buffer, 256, stdin) != NULL)
       }
     }
 
-  else if (strncmp(cmd, "file", 4) == 0)
+  else if (Ustrncmp(cmd, "file", 4) == 0)
     {
-    char *s = cmd + 4;
+    uschar *s = cmd + 4;
     while (isspace((uschar)*s)) s++;
-    i = atoi(s);
+    i = Uatoi(s);
     if (i >= max_db || dbblock[i].dbptr == NULL) printf("Not open\n");
       else current = i;
     }
 
-  else if (strncmp(cmd, "time", 4) == 0) showtime = ~showtime;
-
-  else if (strcmp(cmd, "q") == 0 || strncmp(cmd, "quit", 4) == 0) break;
-
-  else if (strncmp(cmd, "help", 4) == 0)
+  else if (Ustrncmp(cmd, "time", 4) == 0)
     {
-    printf("close <number>\n");
-    printf("file  <number>\n");
-    printf("open  <name>\n");
-    printf("quit\n");
-    printf("read  <key>\n");
-    printf("write <key> <rest-of-line>\n");
+    showtime = ~showtime;
+    printf("Timing %s\n", showtime? "on" : "off");
+    }
+
+  else if (Ustrcmp(cmd, "q") == 0 || Ustrncmp(cmd, "quit", 4) == 0) break;
+
+  else if (Ustrncmp(cmd, "help", 4) == 0)
+    {
+    printf("close  [<number>]              close file [<number>]\n");
+    printf("delete <key>                   remove record from current file\n");
+    printf("file   <number>                make file <number> current\n");
+    printf("open   <name>                  open db file\n");
+    printf("q[uit]                         exit program\n");
+    printf("read   <key>                   read record from current file\n");
+    printf("scan                           scan current file\n");
+    printf("time                           time display on/off\n");
+    printf("write  <key> <rest-of-line>    write record to current file\n");
     }
 
   else printf("Eh?\n");
 
   if (showtime && stop >= start)
     printf("start=%d stop=%d difference=%d\n", (int)start, (int)stop,
-     ( int)(stop - start));
+     (int)(stop - start));
 
   printf("> ");
   }
