@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/dns.c,v 1.5 2005/02/17 11:58:26 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for interfacing with the DNS. */
@@ -41,9 +43,11 @@ dns_init(BOOL qualify_single, BOOL search_parents)
 {
 if ((_res.options & RES_INIT) == 0)
   {
+  DEBUG(D_resolver) _res.options |= RES_DEBUG;     /* For Cygwin */
   res_init();
-  DEBUG(11) _res.options |= RES_DEBUG;
+  DEBUG(D_resolver) _res.options |= RES_DEBUG;
   }
+
 _res.options &= ~(RES_DNSRCH | RES_DEFNAMES);
 _res.options |= (qualify_single? RES_DEFNAMES : 0) |
                 (search_parents? RES_DNSRCH : 0);
@@ -53,6 +57,95 @@ if (dns_retry > 0) _res.retry = dns_retry;
 
 
 
+/*************************************************
+*       Build key name for PTR records           *
+*************************************************/
+
+/* This function inverts an IP address and adds the relevant domain, to produce
+a name that can be used to look up PTR records.
+
+Arguments:
+  string     the IP address as a string
+  buffer     a suitable buffer, long enough to hold the result
+
+Returns:     nothing
+*/
+
+void
+dns_build_reverse(uschar *string, uschar *buffer)
+{
+uschar *p = string + Ustrlen(string);
+uschar *pp = buffer;
+
+/* Handle IPv4 address */
+
+#if HAVE_IPV6
+if (Ustrchr(string, ':') == NULL)
+#endif
+  {
+  int i;
+  for (i = 0; i < 4; i++)
+    {
+    uschar *ppp = p;
+    while (ppp > string && ppp[-1] != '.') ppp--;
+    Ustrncpy(pp, ppp, p - ppp);
+    pp += p - ppp;
+    *pp++ = '.';
+    p = ppp - 1;
+    }
+  Ustrcpy(pp, "in-addr.arpa");
+  }
+
+/* Handle IPv6 address; convert to binary so as to fill out any
+abbreviation in the textual form. */
+
+#if HAVE_IPV6
+else
+  {
+  int i;
+  int v6[4];
+  (void)host_aton(string, v6);
+
+  /* The original specification for IPv6 reverse lookup was to invert each
+  nibble, and look in the ip6.int domain. The domain was subsequently
+  changed to ip6.arpa. */
+
+  for (i = 3; i >= 0; i--)
+    {
+    int j;
+    for (j = 0; j < 32; j += 4)
+      {
+      sprintf(CS pp, "%x.", (v6[i] >> j) & 15);
+      pp += 2;
+      }
+    }
+  Ustrcpy(pp, "ip6.arpa.");
+
+  /* Another way of doing IPv6 reverse lookups was proposed in conjunction
+  with A6 records. However, it fell out of favour when they did. The
+  alternative was to construct a binary key, and look in ip6.arpa. I tried
+  to make this code do that, but I could not make it work on Solaris 8. The
+  resolver seems to lose the initial backslash somehow. However, now that
+  this style of reverse lookup has been dropped, it doesn't matter. These
+  lines are left here purely for historical interest. */
+
+  /**************************************************
+  Ustrcpy(pp, "\\[x");
+  pp += 3;
+
+  for (i = 0; i < 4; i++)
+    {
+    sprintf(pp, "%08X", v6[i]);
+    pp += 8;
+    }
+  Ustrcpy(pp, "].ip6.arpa.");
+  **************************************************/
+
+  }
+#endif
+}
+
+
 
 
 /*************************************************
@@ -160,18 +253,21 @@ Argument:   record type
 Returns:    pointer to string
 */
 
-char *
+uschar *
 dns_text_type(int t)
 {
 switch(t)
   {
-  case T_A:    return "A";
-  case T_MX:   return "MX";
-  case T_AAAA: return "AAAA";
-  case T_A6:   return "A6";
-  case T_TXT:  return "TXT";
-  case T_PTR:  return "PTR";
-  default:     return "?";
+  case T_A:     return US"A";
+  case T_MX:    return US"MX";
+  case T_AAAA:  return US"AAAA";
+  case T_A6:    return US"A6";
+  case T_TXT:   return US"TXT";
+  case T_PTR:   return US"PTR";
+  case T_SRV:   return US"SRV";
+  case T_NS:    return US"NS";
+  case T_CNAME: return US"CNAME";
+  default:      return US"?";
   }
 }
 
@@ -182,8 +278,9 @@ switch(t)
 *************************************************/
 
 /* We cache failed lookup results so as not to experience timeouts many
-times for the same domain. For successful lookups, we rely on resolver and/or
-name server caching.
+times for the same domain. We need to retain the resolver options because they
+may change. For successful lookups, we rely on resolver and/or name server
+caching.
 
 Arguments:
   name       the domain name
@@ -194,10 +291,11 @@ Returns:     the return code
 */
 
 static int
-dns_return(char *name, int type, int rc)
+dns_return(uschar *name, int type, int rc)
 {
-tree_node *node = store_malloc(sizeof(tree_node) + 280);
-sprintf(node->name, "%.255s-%s", name, dns_text_type(type));
+tree_node *node = store_get_perm(sizeof(tree_node) + 290);
+sprintf(CS node->name, "%.255s-%s-%lx", name, dns_text_type(type),
+  _res.options);
 node->data.val = rc;
 (void)tree_insertnode(&tree_dns_fails, node);
 return rc;
@@ -222,30 +320,38 @@ Arguments:
   type      type of DNS record required (T_A, T_MX, etc)
 
 Returns:    DNS_SUCCEED   successful lookup
-            DNS_NOMATCH   name not found, or no data found for the given type,
+            DNS_NOMATCH   name not found (NXDOMAIN)
                           or name contains illegal characters (if checking)
+            DNS_NODATA    domain exists, but no data for this type (NODATA)
             DNS_AGAIN     soft failure, try again later
             DNS_FAIL      DNS failure
 */
 
 int
-dns_basic_lookup(dns_answer *dnsa, char *name, int type)
+dns_basic_lookup(dns_answer *dnsa, uschar *name, int type)
 {
+#ifndef STAND_ALONE
+int rc;
+uschar *save;
+#endif
+
 tree_node *previous;
-char node_name[280];
+uschar node_name[290];
 
 /* DNS lookup failures of any kind are cached in a tree. This is mainly so that
 a timeout on one domain doesn't happen time and time again for messages that
 have many addresses in the same domain. We rely on the resolver and name server
 caching for successful lookups. */
 
-sprintf(node_name, "%.255s-%s", name, dns_text_type(type));
+sprintf(CS node_name, "%.255s-%s-%lx", name, dns_text_type(type),
+  _res.options);
 previous = tree_search(tree_dns_fails, node_name);
 if (previous != NULL)
   {
-  DEBUG(8) debug_printf("DNS lookup of %s: using cached value %s\n",
-    node_name,
+  DEBUG(D_dns) debug_printf("DNS lookup of %.255s-%s: using cached value %s\n",
+    name, dns_text_type(type),
       (previous->data.val == DNS_NOMATCH)? "DNS_NOMATCH" :
+      (previous->data.val == DNS_NODATA)? "DNS_NODATA" :
       (previous->data.val == DNS_AGAIN)? "DNS_AGAIN" :
       (previous->data.val == DNS_FAIL)? "DNS_FAIL" : "??");
   return previous->data.val;
@@ -257,18 +363,25 @@ test the handling of DNS errors. */
 
 if (running_in_test_harness)
   {
-  if (strcmp(name, "test.again.dns") == 0)
+  uschar *endname = name + Ustrlen(name);
+  if (Ustrcmp(endname - 14, "test.again.dns") == 0)
     {
-    DEBUG(8) debug_printf("DNS lookup of %s (%s) bypassed for testing\n",
+    int delay = Uatoi(name);  /* digits at the start of the name */
+    DEBUG(D_dns) debug_printf("Real DNS lookup of %s (%s) bypassed for testing\n",
       name, dns_text_type(type));
-    DEBUG(8) debug_printf("returning DNS_AGAIN\n");
+    if (delay > 0)
+      {
+      DEBUG(D_dns) debug_printf("delaying %d seconds\n", delay);
+      sleep(delay);
+      }
+    DEBUG(D_dns) debug_printf("returning DNS_AGAIN\n");
     return dns_return(name, type, DNS_AGAIN);
     }
-  if (strcmp(name, "test.fail.dns") == 0)
+  if (Ustrcmp(endname - 13, "test.fail.dns") == 0)
     {
-    DEBUG(8) debug_printf("DNS lookup of %s (%s) bypassed for testing\n",
+    DEBUG(D_dns) debug_printf("Real DNS lookup of %s (%s) bypassed for testing\n",
       name, dns_text_type(type));
-    DEBUG(8) debug_printf("returning DNS_FAIL\n");
+    DEBUG(D_dns) debug_printf("returning DNS_FAIL\n");
     return dns_return(name, type, DNS_FAIL);
     }
   }
@@ -282,22 +395,35 @@ regex has substrings that are used - the default uses a conditional.
 
 This test is omitted for PTR records. These occur only in calls from the dnsdb
 lookup, which constructs the names itself, so they should be OK. Besides,
-bitstring labels don't conform to normal name syntax. */
+bitstring labels don't conform to normal name syntax. (But the aren't used any
+more.)
+
+For SRV records, we omit the initial _smtp._tcp. components at the start. */
 
 #ifndef STAND_ALONE   /* Omit this for stand-alone tests */
 
-if (check_dns_names && type != T_PTR)
+if (check_dns_names_pattern[0] != 0 && type != T_PTR)
   {
+  uschar *checkname = name;
   int ovector[3*(EXPAND_MAXN+1)];
 
   if (regex_check_dns_names == NULL)
     regex_check_dns_names =
       regex_must_compile(check_dns_names_pattern, FALSE, TRUE);
 
-  if (pcre_exec(regex_check_dns_names, NULL, name, (int)strlen(name), 0,
-      PCRE_EOPT, ovector, sizeof(ovector)/sizeof(int)) < 0)
+  /* For an SRV lookup, skip over the first two components (the service and
+  protocol names, which both start with an underscore). */
+
+  if (type == T_SRV)
     {
-    DEBUG(3)
+    while (*checkname++ != '.');
+    while (*checkname++ != '.');
+    }
+
+  if (pcre_exec(regex_check_dns_names, NULL, CS checkname, Ustrlen(checkname),
+      0, PCRE_EOPT, ovector, sizeof(ovector)/sizeof(int)) < 0)
+    {
+    DEBUG(D_dns)
       debug_printf("DNS name syntax check failed: %s (%s)\n", name,
         dns_text_type(type));
     host_find_failed_syntax = TRUE;
@@ -307,51 +433,61 @@ if (check_dns_names && type != T_PTR)
 
 #endif /* STAND_ALONE */
 
-/* Call the resolver */
+/* Call the resolver; for an overlong response, res_search() will return the
+number of bytes the message would need, so we need to check for this case.
+The effect is to truncate overlong data. */
 
-dnsa->answerlen = res_search(name, C_IN, type, dnsa->answer, MAXPACKET);
+dnsa->answerlen = res_search(CS name, C_IN, type, dnsa->answer, MAXPACKET);
+if (dnsa->answerlen > MAXPACKET) dnsa->answerlen = MAXPACKET;
 
 if (dnsa->answerlen < 0) switch (h_errno)
   {
   case HOST_NOT_FOUND:
-  DEBUG(8) debug_printf("DNS lookup of %s (%s) gave HOST_NOT_FOUND\n"
+  DEBUG(D_dns) debug_printf("DNS lookup of %s (%s) gave HOST_NOT_FOUND\n"
     "returning DNS_NOMATCH\n", name, dns_text_type(type));
   return dns_return(name, type, DNS_NOMATCH);
 
   case TRY_AGAIN:
-  DEBUG(8) debug_printf("DNS lookup of %s (%s) gave TRY_AGAIN\n",
+  DEBUG(D_dns) debug_printf("DNS lookup of %s (%s) gave TRY_AGAIN\n",
     name, dns_text_type(type));
 
   /* Cut this out for various test programs */
   #ifndef STAND_ALONE
-  if (!match_isinlist(name, &dns_again_means_nonexist, FALSE, TRUE, NULL))
+  save = deliver_domain;
+  deliver_domain = name;  /* set $domain */
+  rc = match_isinlist(name, &dns_again_means_nonexist, 0, NULL, NULL,
+    MCL_DOMAIN, TRUE, NULL);
+  deliver_domain = save;
+  if (rc != OK)
     {
-    DEBUG(8) debug_printf("returning DNS_AGAIN\n");
+    DEBUG(D_dns) debug_printf("returning DNS_AGAIN\n");
     return dns_return(name, type, DNS_AGAIN);
     }
-  DEBUG(8) debug_printf("%s is in dns_again_means_nonexist: returning "
+  DEBUG(D_dns) debug_printf("%s is in dns_again_means_nonexist: returning "
     "DNS_NOMATCH\n", name);
-  #endif
-
   return dns_return(name, type, DNS_NOMATCH);
 
+  #else   /* For stand-alone tests */
+  return dns_return(name, type, DNS_AGAIN);
+  #endif
+
   case NO_RECOVERY:
-  DEBUG(8) debug_printf("DNS lookup of %s (%s) gave NO_RECOVERY\n"
+  DEBUG(D_dns) debug_printf("DNS lookup of %s (%s) gave NO_RECOVERY\n"
     "returning DNS_FAIL\n", name, dns_text_type(type));
   return dns_return(name, type, DNS_FAIL);
 
   case NO_DATA:
-  DEBUG(8) debug_printf("DNS lookup of %s (%s) gave NO_DATA\n"
-    "returning DNS_NOMATCH\n", name, dns_text_type(type));
-  return dns_return(name, type, DNS_NOMATCH);
+  DEBUG(D_dns) debug_printf("DNS lookup of %s (%s) gave NO_DATA\n"
+    "returning DNS_NODATA\n", name, dns_text_type(type));
+  return dns_return(name, type, DNS_NODATA);
 
   default:
-  DEBUG(8) debug_printf("DNS lookup of %s (%s) gave unknown DNS error %d\n"
+  DEBUG(D_dns) debug_printf("DNS lookup of %s (%s) gave unknown DNS error %d\n"
     "returning DNS_FAIL\n", name, dns_text_type(type), h_errno);
   return dns_return(name, type, DNS_FAIL);
   }
 
-DEBUG(8) debug_printf("DNS lookup of %s (%s) succeeded\n",
+DEBUG(D_dns) debug_printf("DNS lookup of %s (%s) succeeded\n",
   name, dns_text_type(type));
 
 return DNS_SUCCEED;
@@ -387,22 +523,23 @@ Arguments:
                           contents are different (i.e. it must be preset)
 
 Returns:                DNS_SUCCEED   successful lookup
-                        DNS_NOMATCH   name not found, or no data found
+                        DNS_NOMATCH   name not found
+                        DNS_NODATA    no data found
                         DNS_AGAIN     soft failure, try again later
                         DNS_FAIL      DNS failure
 */
 
 int
-dns_lookup(dns_answer *dnsa, char *name, int type, char **fully_qualified_name)
+dns_lookup(dns_answer *dnsa, uschar *name, int type, uschar **fully_qualified_name)
 {
 int i;
-char *orig_name = name;
+uschar *orig_name = name;
 
 /* Loop to follow CNAME chains so far, but no further... */
 
 for (i = 0; i < 10; i++)
   {
-  char data[256];
+  uschar data[256];
   dns_record *rr, cname_rr, type_rr;
   dns_scan dnss;
   int datalen, rc;
@@ -431,21 +568,29 @@ for (i = 0; i < 10; i++)
     }
 
   /* If a CNAME was found, take the fully qualified name from it; otherwise
-  from the first data record, if present. */
+  from the first data record, if present. For testing, there is a magic name
+  that gets its casing adjusted, because my resolver doesn't seem to pass back
+  upper case letters in domain names. */
 
   if (fully_qualified_name != NULL)
     {
     if (cname_rr.data != NULL)
       {
-      if (strcmp(cname_rr.name, *fully_qualified_name) != 0 &&
+      if (Ustrcmp(cname_rr.name, *fully_qualified_name) != 0 &&
           cname_rr.name[0] != '*')
-        *fully_qualified_name = string_copy(cname_rr.name);
+        *fully_qualified_name = string_copy_dnsdomain(cname_rr.name);
       }
     else if (type_rr.data != NULL)
       {
-      if (strcmp(type_rr.name, *fully_qualified_name) != 0 &&
-          type_rr.name[0] != '*')
-        *fully_qualified_name = string_copy(type_rr.name);
+      if (running_in_test_harness &&
+          Ustrcmp(type_rr.name, "uppercase.test.ex") == 0)
+        *fully_qualified_name = US"UpperCase.test.ex";
+      else
+        {
+        if (Ustrcmp(type_rr.name, *fully_qualified_name) != 0 &&
+            type_rr.name[0] != '*')
+          *fully_qualified_name = string_copy_dnsdomain(type_rr.name);
+        }
       }
     }
 
@@ -473,6 +618,68 @@ return DNS_FAIL;
 }
 
 
+
+
+
+
+/************************************************
+*    Do a DNS lookup and handle virtual types   *
+************************************************/
+
+/* This function handles some invented "lookup types" that synthesize feature
+not available in the basic types. The special types all have negative values.
+Positive type values are passed straight on to dns_lookup().
+
+Arguments:
+  dnsa                  pointer to dns_answer structure
+  name                  domain name to look up
+  type                  DNS record type (T_A, T_MX, etc or a "special")
+  fully_qualified_name  if not NULL, return the returned name here if its
+                          contents are different (i.e. it must be preset)
+
+Returns:                DNS_SUCCEED   successful lookup
+                        DNS_NOMATCH   name not found
+                        DNS_NODATA    no data found
+                        DNS_AGAIN     soft failure, try again later
+                        DNS_FAIL      DNS failure
+*/
+
+int
+dns_special_lookup(dns_answer *dnsa, uschar *name, int type,
+  uschar **fully_qualified_name)
+{
+if (type >= 0) return dns_lookup(dnsa, name, type, fully_qualified_name);
+
+/* The "mx hosts only" type doesn't require any special action here */
+
+if (type == T_MXH) return dns_lookup(dnsa, name, T_MX, fully_qualified_name);
+
+/* Find nameservers for the domain or the nearest enclosing zone, excluding the
+root servers. */
+
+if (type == T_ZNS)
+  {
+  uschar *d = name;
+  while (d != 0)
+    {
+    int rc = dns_lookup(dnsa, d, T_NS, fully_qualified_name);
+    if (rc != DNS_NOMATCH && rc != DNS_NODATA) return rc;
+    while (*d != 0 && *d != '.') d++;
+    if (*d++ == 0) break;
+    }
+  return DNS_NOMATCH;
+  }
+
+/* Control should never reach here */
+
+return DNS_FAIL;
+}
+
+
+
+/* Support for A6 records has been commented out since they were demoted to
+experimental status at IETF 51. */
+
 #if HAVE_IPV6 && defined(SUPPORT_A6)
 
 /*************************************************
@@ -586,7 +793,7 @@ adjust the hanging pointer, and we are done. */
 if (prefix_len == 0)
   {
   dns_address *new = store_get(sizeof(dns_address) + 50);
-  inet_ntop(AF_INET6, bitvec, new->address, 50);
+  inet_ntop(AF_INET6, bitvec, CS new->address, 50);
   new->next = NULL;
   **yptrptr = new;
   *yptrptr = &(new->next);
@@ -636,9 +843,9 @@ if (dns_lookup(&cdnsa, chainptr, T_A6, NULL) == DNS_SUCCEED)
 
 /* The record type is either T_A for an IPv4 address or T_AAAA (or T_A6 when
 supported) for an IPv6 address. In the A6 case, there may be several addresses,
-generated by following chains. A recursive function does all the hard work.
-A6 records now look like passing into history, so the code is only included
-when explicitly asked for.
+generated by following chains. A recursive function does all the hard work. A6
+records now look like passing into history, so the code is only included when
+explicitly asked for.
 
 Argument:
   dnsa       the DNS answer block
@@ -680,7 +887,7 @@ else if (rr->type == T_A6)
 else
   {
   yield = store_get(sizeof(dns_address) + 50);
-  inet_ntop(AF_INET6, (uschar *)(rr->data), yield->address, 50);
+  inet_ntop(AF_INET6, (uschar *)(rr->data), CS yield->address, 50);
   yield->next = NULL;
   }
 #endif  /* HAVE_IPV6 */
