@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/drtables.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -47,6 +49,14 @@ be NULL for methods that don't need them. */
 #include "lookups/dnsdb.h"
 #endif
 
+#ifdef LOOKUP_DSEARCH
+#include "lookups/dsearch.h"
+#endif
+
+#ifdef LOOKUP_IBASE
+#include "lookups/ibase.h"
+#endif
+
 #ifdef LOOKUP_LDAP
 #include "lookups/ldap.h"
 #endif
@@ -67,6 +77,14 @@ be NULL for methods that don't need them. */
 #include "lookups/nisplus.h"
 #endif
 
+#ifdef LOOKUP_ORACLE
+#include "lookups/oracle.h"
+#endif
+
+#ifdef LOOKUP_PASSWD
+#include "lookups/passwd.h"
+#endif
+
 #ifdef LOOKUP_PGSQL
 #include "lookups/pgsql.h"
 #endif
@@ -75,6 +93,9 @@ be NULL for methods that don't need them. */
 #include "lookups/testdb.h"
 #endif
 
+#ifdef LOOKUP_WHOSON
+#include "lookups/whoson.h"
+#endif
 
 /* The second field in each item below is a set of bit flags:
 
@@ -82,22 +103,24 @@ be NULL for methods that don't need them. */
                              else single-key (+ file) style
   lookup_absfile        => an absolute file name is required,
                              (for single-key style only)
-*/
 
+This list must be in alphabetical order of lookup name because it is
+searched by binary chop, having got rather large for the original linear
+searching. */
 
 lookup_info lookup_list[] = {
 
 /* cdb lookup in single file */
 
   {
-  "cdb",			 /* lookup name */
-  lookup_absfile,		 /* uses absolute file name */
+  US"cdb",                       /* lookup name */
+  lookup_absfile,                /* uses absolute file name */
 #ifdef LOOKUP_CDB
-  cdb_open,			 /* open function */
-  cdb_check,			 /* check function */
-  cdb_find,			 /* find function */
-  cdb_close,			 /* close function */
-  NULL,				 /* no tidy function */
+  cdb_open,                      /* open function */
+  cdb_check,                     /* check function */
+  cdb_find,                      /* find function */
+  cdb_close,                     /* close function */
+  NULL,                          /* no tidy function */
   NULL                           /* no quoting function */
 #else
   NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
@@ -108,7 +131,7 @@ lookup_info lookup_list[] = {
 but the code is called dbmdb to avoid name clashes. */
 
   {
-  "dbm",                         /* lookup name */
+  US"dbm",                       /* lookup name */
   lookup_absfile,                /* uses absolute file name */
 #ifdef LOOKUP_DBM
   dbmdb_open,                    /* open function */
@@ -126,7 +149,7 @@ but the code is called dbmdb to avoid name clashes. */
 of the key strings. */
 
   {
-  "dbmnz",                       /* lookup name */
+  US"dbmnz",                     /* lookup name */
   lookup_absfile,                /* uses absolute file name */
 #ifdef LOOKUP_DBM
   dbmdb_open,      /* sic */     /* open function */
@@ -143,7 +166,7 @@ of the key strings. */
 /* Using DNS TXT records as a database */
 
   {
-  "dnsdb",                       /* lookup name */
+  US"dnsdb",                     /* lookup name */
   lookup_querystyle,             /* query style */
 #ifdef LOOKUP_DNSDB
   dnsdb_open,                    /* open function */
@@ -157,10 +180,62 @@ of the key strings. */
 #endif
   },
 
+/* Search of files in a directory */
+
+  {
+  US"dsearch",                   /* lookup name */
+  lookup_absfile,                /* uses absolute file name */
+#ifdef LOOKUP_DSEARCH
+  dsearch_open,                  /* open function */
+  dsearch_check,                 /* check function */
+  dsearch_find,                  /* find function */
+  dsearch_close,                 /* close function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
+/* Interbase lookup */
+
+  {
+  US"ibase",                     /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+#ifdef LOOKUP_IBASE
+  ibase_open,                    /* open function */
+  NULL,                          /* no check function */
+  ibase_find,                    /* find function */
+  NULL,                          /* no close function */
+  ibase_tidy,                    /* tidy function */
+  ibase_quote                    /* quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
+/* Linear search of single file with ip-addresses and networks; shares many
+functions with lsearch. */
+
+  {
+  US"iplsearch",                 /* lookup name */
+  lookup_absfile,                /* uses absolute file name */
+#ifdef LOOKUP_LSEARCH
+  lsearch_open,                  /* open function */
+  lsearch_check,                 /* check function */
+  iplsearch_find,                /* find function */
+  lsearch_close,                 /* close function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
 /* LDAP lookup, allowing data from only one entry to be returned */
 
   {
-  "ldap",                        /* lookup name */
+  US"ldap",                      /* lookup name */
   lookup_querystyle,             /* query-style lookup */
 #ifdef LOOKUP_LDAP
   eldap_open,                    /* open function */
@@ -174,27 +249,10 @@ of the key strings. */
 #endif
   },
 
-/* LDAP lookup, allowing data from more than one entry to be returned */
-
-  {
-  "ldapm",                       /* lookup name */
-  lookup_querystyle,             /* query-style lookup */
-#ifdef LOOKUP_LDAP
-  eldap_open,       /* sic */    /* open function */
-  NULL,                          /* check function */
-  eldapm_find,                   /* find function */
-  NULL,                          /* no close function */
-  eldap_tidy,       /* sic */    /* tidy function */
-  eldap_quote       /* sic */    /* quoting function */
-#else
-  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
-#endif
-  },
-
 /* LDAP lookup, allowing the DN from more one entry to be returned */
 
   {
-  "ldapdn",                       /* lookup name */
+  US"ldapdn",                     /* lookup name */
   lookup_querystyle,             /* query-style lookup */
 #ifdef LOOKUP_LDAP
   eldap_open,       /* sic */    /* open function */
@@ -208,10 +266,27 @@ of the key strings. */
 #endif
   },
 
+/* LDAP lookup, allowing data from more than one entry to be returned */
+
+  {
+  US"ldapm",                     /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+#ifdef LOOKUP_LDAP
+  eldap_open,       /* sic */    /* open function */
+  NULL,                          /* check function */
+  eldapm_find,                   /* find function */
+  NULL,                          /* no close function */
+  eldap_tidy,       /* sic */    /* tidy function */
+  eldap_quote       /* sic */    /* quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
 /* Linear search of single file */
 
   {
-  "lsearch",                     /* lookup name */
+  US"lsearch",                   /* lookup name */
   lookup_absfile,                /* uses absolute file name */
 #ifdef LOOKUP_LSEARCH
   lsearch_open,                  /* open function */
@@ -228,7 +303,7 @@ of the key strings. */
 /* MYSQL lookup */
 
   {
-  "mysql",                       /* lookup name */
+  US"mysql",                     /* lookup name */
   lookup_querystyle,             /* query-style lookup */
 #ifdef LOOKUP_MYSQL
   mysql_open,                    /* open function */
@@ -245,7 +320,7 @@ of the key strings. */
 /* NIS lookup, excluding trailing 0 from key */
 
   {
-  "nis",                         /* lookup name */
+  US"nis",                       /* lookup name */
   0,                             /* not abs file, not query style*/
 #ifdef LOOKUP_NIS
   nis_open,                      /* open function */
@@ -262,7 +337,7 @@ of the key strings. */
 /* NIS lookup, including trailing 0 in key */
 
   {
-  "nis0",                        /* lookup name */
+  US"nis0",                      /* lookup name */
   0,                             /* not absfile, not query style */
 #ifdef LOOKUP_NIS
   nis_open,    /* sic */         /* open function */
@@ -279,7 +354,7 @@ of the key strings. */
 /* NIS+ lookup */
 
   {
-  "nisplus",                     /* lookup name */
+  US"nisplus",                   /* lookup name */
   lookup_querystyle,             /* query-style lookup */
 #ifdef LOOKUP_NISPLUS
   nisplus_open,                  /* open function */
@@ -293,10 +368,62 @@ of the key strings. */
 #endif
   },
 
+/* Linear search of single file, with wildcarding but no pattern expansion.
+Shares many functions with lsearch. */
+
+  {
+  US"nwildlsearch",              /* lookup name */
+  lookup_absfile,                /* uses absolute file name */
+#ifdef LOOKUP_LSEARCH
+  lsearch_open,                  /* open function */
+  lsearch_check,                 /* check function */
+  nwildlsearch_find,             /* find function */
+  lsearch_close,                 /* close function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
+/* Oracle lookup */
+
+  {
+  US"oracle",                    /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+#ifdef LOOKUP_ORACLE
+  oracle_open,                   /* open function */
+  NULL,                          /* check function */
+  oracle_find,                   /* find function */
+  NULL,                          /* no close function */
+  oracle_tidy,                   /* tidy function */
+  oracle_quote                   /* quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
+/* passwd lookup */
+
+  {
+  US"passwd",                    /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+#ifdef LOOKUP_PASSWD
+  passwd_open,                   /* open function */
+  NULL,                          /* no check function */
+  passwd_find,                   /* find function */
+  NULL,                          /* no close function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL   /* lookup not present */
+#endif
+  },
+
 /* PGSQL lookup */
 
   {
-  "pgsql",                       /* lookup name */
+  US"pgsql",                     /* lookup name */
   lookup_querystyle,             /* query-style lookup */
 #ifdef LOOKUP_PGSQL
   pgsql_open,                    /* open function */
@@ -317,7 +444,7 @@ of "lookup type not available". */
 
 #ifdef LOOKUP_TESTDB
   {
-  "testdb",                      /* lookup name */
+  US"testdb",                    /* lookup name */
   lookup_querystyle,             /* query-style lookup */
   testdb_open,                   /* open function */
   NULL,                          /* check function */
@@ -328,87 +455,145 @@ of "lookup type not available". */
   },
 #endif
 
-/* Terminating entry */
+/* "Whoson" lookup */
 
-{ "", 0, NULL, NULL, NULL, NULL, NULL, NULL }
+  {
+  US"whoson",                    /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+#ifdef LOOKUP_WHOSON
+  whoson_open,                   /* open function */
+  NULL,                          /* check function */
+  whoson_find,                   /* find function */
+  NULL,                          /* no close function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL   /* lookup not present */
+#endif
+  },
+
+/* Linear search of single file, with wildcarding and pattern expansion. Shares
+many functions with lsearch. */
+
+  {
+  US"wildlsearch",               /* lookup name */
+  lookup_absfile,                /* uses absolute file name */
+#ifdef LOOKUP_LSEARCH
+  lsearch_open,                  /* open function */
+  lsearch_check,                 /* check function */
+  wildlsearch_find,              /* find function */
+  lsearch_close,                 /* close function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  }
 };
 
+/* Number of entries in the list */
+
+int lookup_list_count = sizeof(lookup_list)/sizeof(lookup_info);
+
+
 
 /* Table of information about all possible authentication mechamisms. All
 entries are always present if any mechanism is declared, but the functions are
 set to NULL for those that are not compiled into the binary. */
 
-#ifdef HAVE_AUTH
-
 #ifdef AUTH_CRAM_MD5
 #include "auths/cram_md5.h"
 #endif
 
+#ifdef AUTH_CYRUS_SASL
+#include "auths/cyrus_sasl.h"
+#endif
+
 #ifdef AUTH_PLAINTEXT
 #include "auths/plaintext.h"
 #endif
 
+#ifdef AUTH_SPA
+#include "auths/spa.h"
+#endif
+
 auth_info auths_available[] = {
 
 /* Checking by an expansion condition on plain text */
 
 #ifdef AUTH_CRAM_MD5
   {
-  "cram_md5",				     /* lookup name */
-  auth_cram_md5_server,                      /* server function */
-  auth_cram_md5_init,                        /* init function */
-  auth_cram_md5_client,                      /* client function */
+  US"cram_md5",                              /* lookup name */
   auth_cram_md5_options,
   &auth_cram_md5_options_count,
   &auth_cram_md5_option_defaults,
-  sizeof(auth_cram_md5_options_block)
+  sizeof(auth_cram_md5_options_block),
+  auth_cram_md5_init,                        /* init function */
+  auth_cram_md5_server,                      /* server function */
+  auth_cram_md5_client                       /* client function */
+  },
+#endif
+
+#ifdef AUTH_CYRUS_SASL
+  {
+  US"cyrus_sasl",			     /* lookup name */
+  auth_cyrus_sasl_options,
+  &auth_cyrus_sasl_options_count,
+  &auth_cyrus_sasl_option_defaults,
+  sizeof(auth_cyrus_sasl_options_block),
+  auth_cyrus_sasl_init,                      /* init function */
+  auth_cyrus_sasl_server,                    /* server function */
+  NULL                                       /* client function */
   },
 #endif
 
 #ifdef AUTH_PLAINTEXT
   {
-  "plaintext",				     /* lookup name */
-  auth_plaintext_server,                     /* server function */
-  auth_plaintext_init,                       /* init function */
-  auth_plaintext_client,                     /* client function */
+  US"plaintext",                             /* lookup name */
   auth_plaintext_options,
   &auth_plaintext_options_count,
   &auth_plaintext_option_defaults,
-  sizeof(auth_plaintext_options_block)
+  sizeof(auth_plaintext_options_block),
+  auth_plaintext_init,                       /* init function */
+  auth_plaintext_server,                     /* server function */
+  auth_plaintext_client                      /* client function */
   },
 #endif
 
-{ "", NULL, NULL, NULL, NULL, 0, NULL, 0  }
+#ifdef AUTH_SPA
+  {
+  US"spa",                                   /* lookup name */
+  auth_spa_options,
+  &auth_spa_options_count,
+  &auth_spa_option_defaults,
+  sizeof(auth_spa_options_block),
+  auth_spa_init,                             /* init function */
+  auth_spa_server,                           /* server function */
+  auth_spa_client                            /* client function */
+  },
+#endif
+
+{ US"", NULL, NULL, NULL, 0, NULL, NULL, NULL  }
 };
 
-#endif  /* HAVE_AUTH */
 
-
-/* Tables of information about which directors, routers and transports are
-included in the exim binary. */
+/* Tables of information about which routers and transports are included in the
+exim binary. */
 
 /* Pull in the necessary header files */
 
-#include "directors/dfunctions.h"
+#include "routers/rf_functions.h"
 
-#ifdef DIRECTOR_ALIASFILE
-#include "directors/aliasfile.h"
+#ifdef ROUTER_ACCEPT
+#include "routers/accept.h"
 #endif
 
-#ifdef DIRECTOR_FORWARDFILE
-#include "directors/forwardfile.h"
+#ifdef ROUTER_DNSLOOKUP
+#include "routers/dnslookup.h"
 #endif
 
-#ifdef DIRECTOR_LOCALUSER
-#include "directors/localuser.h"
-#endif
-
-#ifdef DIRECTOR_SMARTUSER
-#include "directors/smartuser.h"
-#endif
-
-#ifdef ROUTER_DOMAINLIST
-#include "routers/domainlist.h"
+#ifdef ROUTER_MANUALROUTE
+#include "routers/manualroute.h"
 #endif
 
 #ifdef ROUTER_IPLITERAL
@@ -419,14 +604,14 @@ included in the exim binary. */
 #include "routers/iplookup.h"
 #endif
 
-#ifdef ROUTER_LOOKUPHOST
-#include "routers/lookuphost.h"
-#endif
-
 #ifdef ROUTER_QUERYPROGRAM
 #include "routers/queryprogram.h"
 #endif
 
+#ifdef ROUTER_REDIRECT
+#include "routers/redirect.h"
+#endif
+
 #ifdef TRANSPORT_APPENDFILE
 #include "transports/appendfile.h"
 #endif
@@ -435,10 +620,6 @@ included in the exim binary. */
 #include "transports/autoreply.h"
 #endif
 
-#ifdef TRANSPORT_DEBUG
-#include "transports/debug.h"
-#endif
-
 #ifdef TRANSPORT_LMTP
 #include "transports/lmtp.h"
 #endif
@@ -454,122 +635,99 @@ included in the exim binary. */
 
 /* Now set up the structures, terminated by an entry with a null name. */
 
-director_info directors_available[] = {
-#ifdef DIRECTOR_ALIASFILE
-  {
-  "aliasfile",
-  aliasfile_director_entry,
-  aliasfile_director_init,
-  NULL,    /* no tidyup entry */
-  aliasfile_director_options,
-  &aliasfile_director_options_count,
-  &aliasfile_director_option_defaults,
-  sizeof(aliasfile_director_options_block)
-  },
-#endif
-#ifdef DIRECTOR_FORWARDFILE
-  {
-  "forwardfile",
-  forwardfile_director_entry,
-  forwardfile_director_init,
-  NULL,    /* no tidyup entry */
-  forwardfile_director_options,
-  &forwardfile_director_options_count,
-  &forwardfile_director_option_defaults,
-  sizeof(forwardfile_director_options_block)
-  },
-#endif
-#ifdef DIRECTOR_LOCALUSER
-  {
-  "localuser",
-  localuser_director_entry,
-  localuser_director_init,
-  NULL,    /* no tidyup entry */
-  localuser_director_options,
-  &localuser_director_options_count,
-  &localuser_director_option_defaults,
-  sizeof(localuser_director_options_block)
-  },
-#endif
-#ifdef DIRECTOR_SMARTUSER
-  {
-  "smartuser",
-  smartuser_director_entry,
-  smartuser_director_init,
-  NULL,    /* no tidyup entry */
-  smartuser_director_options,
-  &smartuser_director_options_count,
-  &smartuser_director_option_defaults,
-  sizeof(smartuser_director_options_block)
-  },
-#endif
-{ "", NULL, NULL, NULL, NULL, 0, NULL, 0 }
-};
-
-
-
 router_info routers_available[] = {
-#ifdef ROUTER_DOMAINLIST
+#ifdef ROUTER_ACCEPT
   {
-  "domainlist",
-  domainlist_router_entry,
-  domainlist_router_init,
-  NULL,    /* no tidyup entry */
-  domainlist_router_options,
-  &domainlist_router_options_count,
-  &domainlist_router_option_defaults,
-  sizeof(domainlist_router_options_block)
+  US"accept",
+  accept_router_options,
+  &accept_router_options_count,
+  &accept_router_option_defaults,
+  sizeof(accept_router_options_block),
+  accept_router_init,
+  accept_router_entry,
+  NULL,     /* no tidyup entry */
+  ri_yestransport
+  },
+#endif
+#ifdef ROUTER_DNSLOOKUP
+  {
+  US"dnslookup",
+  dnslookup_router_options,
+  &dnslookup_router_options_count,
+  &dnslookup_router_option_defaults,
+  sizeof(dnslookup_router_options_block),
+  dnslookup_router_init,
+  dnslookup_router_entry,
+  NULL,     /* no tidyup entry */
+  ri_yestransport
   },
 #endif
 #ifdef ROUTER_IPLITERAL
   {
-  "ipliteral",
-  ipliteral_router_entry,
-  ipliteral_router_init,
-  NULL,    /* no tidyup entry */
+  US"ipliteral",
   ipliteral_router_options,
   &ipliteral_router_options_count,
   &ipliteral_router_option_defaults,
-  sizeof(ipliteral_router_options_block)
+  sizeof(ipliteral_router_options_block),
+  ipliteral_router_init,
+  ipliteral_router_entry,
+  NULL,     /* no tidyup entry */
+  ri_yestransport
   },
 #endif
 #ifdef ROUTER_IPLOOKUP
   {
-  "iplookup",
-  iplookup_router_entry,
-  iplookup_router_init,
-  NULL,    /* no tidyup entry */
+  US"iplookup",
   iplookup_router_options,
   &iplookup_router_options_count,
   &iplookup_router_option_defaults,
-  sizeof(iplookup_router_options_block)
+  sizeof(iplookup_router_options_block),
+  iplookup_router_init,
+  iplookup_router_entry,
+  NULL,     /* no tidyup entry */
+  ri_notransport
   },
 #endif
-#ifdef ROUTER_LOOKUPHOST
+#ifdef ROUTER_MANUALROUTE
   {
-  "lookuphost",
-  lookuphost_router_entry,
-  lookuphost_router_init,
-  NULL,    /* no tidyup entry */
-  lookuphost_router_options,
-  &lookuphost_router_options_count,
-  &lookuphost_router_option_defaults,
-  sizeof(lookuphost_router_options_block)
+  US"manualroute",
+  manualroute_router_options,
+  &manualroute_router_options_count,
+  &manualroute_router_option_defaults,
+  sizeof(manualroute_router_options_block),
+  manualroute_router_init,
+  manualroute_router_entry,
+  NULL,     /* no tidyup entry */
+  0
   },
 #endif
 #ifdef ROUTER_QUERYPROGRAM
   {
-  "queryprogram",
-  queryprogram_router_entry,
-  queryprogram_router_init,
-  NULL,    /* no tidyup entry */
+  US"queryprogram",
   queryprogram_router_options,
   &queryprogram_router_options_count,
   &queryprogram_router_option_defaults,
-  sizeof(queryprogram_router_options_block)
+  sizeof(queryprogram_router_options_block),
+  queryprogram_router_init,
+  queryprogram_router_entry,
+  NULL,     /* no tidyup entry */
+  0
   },
 #endif
-{ "", NULL, NULL, NULL, NULL, 0, NULL, 0 }
+#ifdef ROUTER_REDIRECT
+  {
+  US"redirect",
+  redirect_router_options,
+  &redirect_router_options_count,
+  &redirect_router_option_defaults,
+  sizeof(redirect_router_options_block),
+  redirect_router_init,
+  redirect_router_entry,
+  NULL,     /* no tidyup entry */
+  ri_notransport
+  },
+#endif
+{ US"", NULL, NULL, NULL, 0, NULL, NULL, NULL, 0 }
 };
 
 
@@ -577,75 +735,75 @@ router_info routers_available[] = {
 transport_info transports_available[] = {
 #ifdef TRANSPORT_APPENDFILE
   {
-  "appendfile",                                /* driver name */
-  appendfile_transport_entry,                  /* main entry point */
-  appendfile_transport_init,                   /* init entry point */
-  NULL,                                        /* no tidyup entry */
+  US"appendfile",                              /* driver name */
   appendfile_transport_options,                /* local options table */
   &appendfile_transport_options_count,         /* number of entries */
   &appendfile_transport_option_defaults,       /* private options defaults */
   sizeof(appendfile_transport_options_block),  /* size of private block */
+  appendfile_transport_init,                   /* init entry point */
+  appendfile_transport_entry,                  /* main entry point */
+  NULL,                                        /* no tidyup entry */
+  NULL,                                        /* no closedown entry */
   TRUE,                                        /* local flag */
-  NULL                                         /* no closedown entry */
   },
 #endif
 #ifdef TRANSPORT_AUTOREPLY
   {
-  "autoreply",                                 /* driver name */
-  autoreply_transport_entry,                   /* main entry point */
-  autoreply_transport_init,                    /* init entry point */
-  NULL,                                        /* no tidyup entry */
+  US"autoreply",                               /* driver name */
   autoreply_transport_options,                 /* local options table */
   &autoreply_transport_options_count,          /* number of entries */
   &autoreply_transport_option_defaults,        /* private options defaults */
   sizeof(autoreply_transport_options_block),   /* size of private block */
-  TRUE,                                        /* local flag */
-  NULL                                         /* no closedown entry */
+  autoreply_transport_init,                    /* init entry point */
+  autoreply_transport_entry,                   /* main entry point */
+  NULL,                                        /* no tidyup entry */
+  NULL,                                        /* no closedown entry */
+  TRUE                                         /* local flag */
   },
 #endif
 #ifdef TRANSPORT_LMTP
   {
-  "lmtp",                                      /* driver name */
-  lmtp_transport_entry,                        /* main entry point */
-  lmtp_transport_init,                         /* init entry point */
-  NULL,                                        /* no tidyup entry */
+  US"lmtp",                                    /* driver name */
   lmtp_transport_options,                      /* local options table */
   &lmtp_transport_options_count,               /* number of entries */
   &lmtp_transport_option_defaults,             /* private options defaults */
   sizeof(lmtp_transport_options_block),        /* size of private block */
-  TRUE,                                        /* local flag */
-  NULL                                         /* no closedown entry */
+  lmtp_transport_init,                         /* init entry point */
+  lmtp_transport_entry,                        /* main entry point */
+  NULL,                                        /* no tidyup entry */
+  NULL,                                        /* no closedown entry */
+  TRUE                                         /* local flag */
   },
 #endif
 #ifdef TRANSPORT_PIPE
   {
-  "pipe",                                      /* driver name */
-  pipe_transport_entry,                        /* main entry point */
-  pipe_transport_init,                         /* init entry point */
-  NULL,                                        /* no tidyup entry */
+  US"pipe",                                    /* driver name */
   pipe_transport_options,                      /* local options table */
   &pipe_transport_options_count,               /* number of entries */
   &pipe_transport_option_defaults,             /* private options defaults */
   sizeof(pipe_transport_options_block),        /* size of private block */
-  TRUE,                                        /* local flag */
-  NULL                                         /* no closedown entry */
+  pipe_transport_init,                         /* init entry point */
+  pipe_transport_entry,                        /* main entry point */
+  NULL,                                        /* no tidyup entry */
+  NULL,                                        /* no closedown entry */
+  TRUE                                         /* local flag */
   },
 #endif
 #ifdef TRANSPORT_SMTP
   {
-  "smtp",                                      /* driver name */
-  smtp_transport_entry,                        /* main entry point */
-  smtp_transport_init,                         /* init entry point */
-  NULL,                                        /* no tidyup entry */
+  US"smtp",                                    /* driver name */
   smtp_transport_options,                      /* local options table */
   &smtp_transport_options_count,               /* number of entries */
   &smtp_transport_option_defaults,             /* private options defaults */
   sizeof(smtp_transport_options_block),        /* size of private block */
-  FALSE,                                       /* local flag */
-  smtp_transport_closedown                     /* close down passed channel */
+  smtp_transport_init,                         /* init entry point */
+  smtp_transport_entry,                        /* main entry point */
+  NULL,                                        /* no tidyup entry */
+  smtp_transport_closedown,                    /* close down passed channel */
+  FALSE                                        /* local flag */
   },
 #endif
-{ "", NULL, NULL, NULL, NULL, 0, NULL, 0, FALSE }
+{ US"", NULL, NULL, NULL, 0, NULL, NULL, NULL, NULL, FALSE }
 };
 
 /* End of drtables.c */
