@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/exim_dbmbuild.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -24,13 +26,20 @@ different.
 
 Input lines beginning with # are ignored, as are blank lines. Entries begin
 with a key terminated by a colon or end of line or whitespace and continue with
-indented lines. */
+indented lines. Keys may be quoted if they contain colons or whitespace or #
+characters. */
 
 
 #include "exim.h"
 
 
-#define maxsize 10000
+#define max_insize   20000
+#define max_outsize 100000
+
+/* This is global because it's defined in the headers and compilers grumble
+if it is made static. */
+
+uschar *hex_digits = US"0123456789abcdef";
 
 
 #ifdef STRERROR_FROM_ERRLIST
@@ -47,6 +56,74 @@ return sys_errlist[n];
 #endif /* STRERROR_FROM_ERRLIST */
 
 
+/* For Berkeley DB >= 2, we can define a function to be called in case of DB
+errors. This should help with debugging strange DB problems, e.g. getting "File
+exists" when you try to open a db file. */
+
+#if defined(USE_DB) && defined(DB_VERSION_STRING)
+void
+dbfn_bdb_error_callback(const char *pfx, char *msg)
+{
+pfx = pfx;
+printf("Berkeley DB error: %s\n", msg);
+}
+#endif
+
+
+
+/*************************************************
+*          Interpret escape sequence             *
+*************************************************/
+
+/* This function is copied from the main Exim code.
+
+Arguments:
+  pp       points a pointer to the initiating "\" in the string;
+           the pointer gets updated to point to the final character
+Returns:   the value of the character escape
+*/
+
+int
+string_interpret_escape(uschar **pp)
+{
+int ch;
+uschar *p = *pp;
+ch = *(++p);
+if (isdigit(ch) && ch != '8' && ch != '9')
+  {
+  ch -= '0';
+  if (isdigit(p[1]) && p[1] != '8' && p[1] != '9')
+    {
+    ch = ch * 8 + *(++p) - '0';
+    if (isdigit(p[1]) && p[1] != '8' && p[1] != '9')
+      ch = ch * 8 + *(++p) - '0';
+    }
+  }
+else switch(ch)
+  {
+  case 'n':  ch = '\n'; break;
+  case 'r':  ch = '\r'; break;
+  case 't':  ch = '\t'; break;
+  case 'x':
+  ch = 0;
+  if (isxdigit(p[1]))
+    {
+    ch = ch * 16 +
+      Ustrchr(hex_digits, tolower(*(++p))) - hex_digits;
+    if (isxdigit(p[1])) ch = ch * 16 +
+      Ustrchr(hex_digits, tolower(*(++p))) - hex_digits;
+    }
+  break;
+  }
+*pp = p;
+return ch;
+}
+
+
+/*************************************************
+*               Main Program                     *
+*************************************************/
+
 int main(int argc, char **argv)
 {
 int started;
@@ -59,27 +136,27 @@ BOOL lowercase = TRUE;
 BOOL warn = TRUE;
 BOOL duperr = TRUE;
 BOOL lastdup = FALSE;
-#if !defined (USE_DB) && !defined(USE_TDB)
+#if !defined (USE_DB) && !defined(USE_TDB) && !defined(USE_GDBM)
 int is_db = 0;
 struct stat statbuf;
 #endif
 FILE *f;
 EXIM_DB *d;
 EXIM_DATUM key, content;
-char *bptr;
-char  keybuffer[256];
-char  temp_dbmname[256];
-char  real_dbmname[256];
-char *buffer = malloc(maxsize);
-char *line = malloc(maxsize);
+uschar *bptr;
+uschar  keybuffer[256];
+uschar  temp_dbmname[256];
+uschar  real_dbmname[256];
+uschar *buffer = malloc(max_outsize);
+uschar *line = malloc(max_insize);
 
 while (argc > 1)
   {
-  if      (strcmp(argv[arg], "-nolc") == 0)     lowercase = FALSE;
-  else if (strcmp(argv[arg], "-nowarn") == 0)   warn = FALSE;
-  else if (strcmp(argv[arg], "-lastdup") == 0)  lastdup = TRUE;
-  else if (strcmp(argv[arg], "-noduperr") == 0) duperr = FALSE;
-  else if (strcmp(argv[arg], "-nozero") == 0)   add_zero = 0;
+  if      (Ustrcmp(argv[arg], "-nolc") == 0)     lowercase = FALSE;
+  else if (Ustrcmp(argv[arg], "-nowarn") == 0)   warn = FALSE;
+  else if (Ustrcmp(argv[arg], "-lastdup") == 0)  lastdup = TRUE;
+  else if (Ustrcmp(argv[arg], "-noduperr") == 0) duperr = FALSE;
+  else if (Ustrcmp(argv[arg], "-nozero") == 0)   add_zero = 0;
   else break;
   arg++;
   argc--;
@@ -91,9 +168,9 @@ if (argc != 3)
   exit(1);
   }
 
-if (strcmp(argv[arg], "-") == 0) f = stdin; else
+if (Ustrcmp(argv[arg], "-") == 0) f = stdin; else
   {
-  f = fopen(argv[arg], "r");
+  f = fopen(argv[arg], "rb");
   if (f == NULL)
     {
     printf("exim_dbmbuild: unable to open %s: %s\n", argv[arg], strerror(errno));
@@ -104,16 +181,16 @@ if (strcmp(argv[arg], "-") == 0) f = stdin; else
 /* By default Berkeley db does not put extensions on... which
 can be painful! */
 
-#if defined(USE_DB) || defined(USE_TDB)
-if (strcmp(argv[arg], argv[arg+1]) == 0)
+#if defined(USE_DB) || defined(USE_TDB) || defined(USE_GDBM)
+if (Ustrcmp(argv[arg], argv[arg+1]) == 0)
   {
   printf("exim_dbmbuild: input and output filenames are the same\n");
   exit(1);
   }
 #endif
 
-strcpy(temp_dbmname, argv[arg+1]);
-strcat(temp_dbmname, ".dbmbuild_temp");
+Ustrcpy(temp_dbmname, argv[arg+1]);
+Ustrcat(temp_dbmname, ".dbmbuild_temp");
 
 /* It is apparently necessary to open with O_RDWR for this to work
 with gdbm-1.7.3, though no reading is actually going to be done. */
@@ -131,9 +208,9 @@ if (d == NULL)
 /* Unless using native db calls, see if we have created <name>.db; if not,
 assume .dir & .pag */
 
-#if !defined(USE_DB) && !defined(USE_TDB)
-sprintf(real_dbmname, "%s.db", temp_dbmname);
-is_db = stat(real_dbmname, &statbuf) == 0;
+#if !defined(USE_DB) && !defined(USE_TDB) && !defined(USE_GDBM)
+sprintf(CS real_dbmname, "%s.db", temp_dbmname);
+is_db = Ustat(real_dbmname, &statbuf) == 0;
 #endif
 
 /* Now do the business */
@@ -141,30 +218,46 @@ is_db = stat(real_dbmname, &statbuf) == 0;
 bptr = buffer;
 started = 0;
 
-while (fgets(line, maxsize, f) != NULL)
+while (Ufgets(line, max_insize, f) != NULL)
   {
-  char *p;
+  uschar *p;
+  int len = Ustrlen(line);
+
+  p = line + len;
+
+  if (len >= max_insize - 1 && p[-1] != '\n')
+    {
+    printf("Overlong line read: max permitted length is %d\n", max_insize - 1);
+    return 1;
+    }
 
   if (line[0] == '#') continue;
-  p = line + (int)strlen(line);
-  while (p > line && isspace((uschar)p[-1])) p--;
+  while (p > line && isspace(p[-1])) p--;
   *p = 0;
   if (line[0] == 0) continue;
 
   /* A continuation line is valid only if there was a previous first
   line. */
 
-  if (isspace((uschar)line[0]))
+  if (isspace(line[0]))
     {
-    char *s = line;
+    uschar *s = line;
     if (!started)
       {
       printf("Unexpected continuation line ignored\n%s\n\n", line);
       continue;
       }
-    while (isspace((uschar)*s)) s++;
+    while (isspace(*s)) s++;
     *(--s) = ' ';
-    strcpy (bptr, s);
+
+    if (bptr - buffer + p - s >= max_outsize - 1)
+      {
+      printf("Continued set of lines is too long: max permitted length is %d\n",
+        max_outsize -1);
+      return 1;
+      }
+
+    Ustrcpy(bptr, s);
     bptr += p - s;
     }
 
@@ -176,12 +269,13 @@ while (fgets(line, maxsize, f) != NULL)
   else
     {
     int i, rc;
-    char *s = line;
+    uschar *s = line;
+    uschar *keystart;
 
     if (started)
       {
       EXIM_DATUM_INIT(content);
-      EXIM_DATUM_DATA(content) = buffer;
+      EXIM_DATUM_DATA(content) = CS buffer;
       EXIM_DATUM_SIZE(content) = bptr - buffer + add_zero;
 
       switch(rc = EXIM_DBPUTB(d, key, content))
@@ -208,10 +302,31 @@ while (fgets(line, maxsize, f) != NULL)
       bptr = buffer;
       }
 
-    while (*s != 0 && *s != ':' && !isspace((uschar)*s)) s++;
     EXIM_DATUM_INIT(key);
-    EXIM_DATUM_DATA(key) = keybuffer;
-    EXIM_DATUM_SIZE(key) = s - line + add_zero;
+    EXIM_DATUM_DATA(key) = CS keybuffer;
+
+    /* Deal with quoted keys. Escape sequences always make one character
+    out of several, so we can re-build in place. */
+
+    if (*s == '\"')
+      {
+      uschar *t = s++;
+      keystart = t;
+      while (*s != 0 && *s != '\"')
+        {
+        if (*s == '\\') *t++ = string_interpret_escape(&s);
+          else *t++ = *s;
+        s++;
+        }
+      if (*s != 0) s++;               /* Past terminating " */
+      EXIM_DATUM_SIZE(key) = t - keystart + add_zero;
+      }
+    else
+      {
+      keystart = s;
+      while (*s != 0 && *s != ':' && !isspace(*s)) s++;
+      EXIM_DATUM_SIZE(key) = s - keystart + add_zero;
+      }
 
     if (EXIM_DATUM_SIZE(key) > 256)
       {
@@ -224,26 +339,26 @@ while (fgets(line, maxsize, f) != NULL)
     if (lowercase)
       {
       for (i = 0; i < EXIM_DATUM_SIZE(key) - add_zero; i++)
-        keybuffer[i] = tolower(line[i]);
+        keybuffer[i] = tolower(keystart[i]);
       }
     else
       {
       for (i = 0; i < EXIM_DATUM_SIZE(key) - add_zero; i++)
-        keybuffer[i] = line[i];
+        keybuffer[i] = keystart[i];
       }
 
     keybuffer[i] = 0;
     started = 1;
 
-    while (isspace((uschar)*s))s++;
+    while (isspace(*s))s++;
     if (*s == ':')
       {
       s++;
-      while (isspace((uschar)*s))s++;
+      while (isspace(*s))s++;
       }
     if (*s != 0)
       {
-      strcpy(bptr, s);
+      Ustrcpy(bptr, s);
       bptr += p - s;
       }
     else buffer[0] = 0;
@@ -254,7 +369,7 @@ if (started)
   {
   int rc;
   EXIM_DATUM_INIT(content);
-  EXIM_DATUM_DATA(content) = buffer;
+  EXIM_DATUM_DATA(content) = CS buffer;
   EXIM_DATUM_SIZE(content) = bptr - buffer + add_zero;
 
   switch(rc = EXIM_DBPUTB(d, key, content))
@@ -296,10 +411,10 @@ if (yield == 0 || yield == 1)
     printf("%d duplicate key%s \n", dupcount, (dupcount > 1)? "s" : "");
     }
 
-  #if defined(USE_DB) || defined(USE_TDB)
-  strcpy(real_dbmname, temp_dbmname);
-  strcpy(buffer, argv[arg+1]);
-  if (rename(real_dbmname, buffer) != 0)
+  #if defined(USE_DB) || defined(USE_TDB) || defined(USE_GDBM)
+  Ustrcpy(real_dbmname, temp_dbmname);
+  Ustrcpy(buffer, argv[arg+1]);
+  if (Urename(real_dbmname, buffer) != 0)
     {
     printf("Unable to rename %s as %s\n", real_dbmname, buffer);
     return 1;
@@ -310,9 +425,9 @@ if (yield == 0 || yield == 1)
 
   if (is_db)
     {
-    sprintf(real_dbmname, "%s.db", temp_dbmname);
-    sprintf(buffer, "%s.db", argv[arg+1]);
-    if (rename(real_dbmname, buffer) != 0)
+    sprintf(CS real_dbmname, "%s.db", temp_dbmname);
+    sprintf(CS buffer, "%s.db", argv[arg+1]);
+    if (Urename(real_dbmname, buffer) != 0)
       {
       printf("Unable to rename %s as %s\n", real_dbmname, buffer);
       return 1;
@@ -323,24 +438,24 @@ if (yield == 0 || yield == 1)
 
   else
     {
-    sprintf(real_dbmname, "%s.dir", temp_dbmname);
-    sprintf(buffer, "%s.dir", argv[arg+1]);
-    if (rename(real_dbmname, buffer) != 0)
+    sprintf(CS real_dbmname, "%s.dir", temp_dbmname);
+    sprintf(CS buffer, "%s.dir", argv[arg+1]);
+    if (Urename(real_dbmname, buffer) != 0)
       {
       printf("Unable to rename %s as %s\n", real_dbmname, buffer);
       return 1;
       }
 
-    sprintf(real_dbmname, "%s.pag", temp_dbmname);
-    sprintf(buffer, "%s.pag", argv[arg+1]);
-    if (rename(real_dbmname, buffer) != 0)
+    sprintf(CS real_dbmname, "%s.pag", temp_dbmname);
+    sprintf(CS buffer, "%s.pag", argv[arg+1]);
+    if (Urename(real_dbmname, buffer) != 0)
       {
       printf("Unable to rename %s as %s\n", real_dbmname, buffer);
       return 1;
       }
     }
 
-  #endif /* USE_DB || USE_TDB */
+  #endif /* USE_DB || USE_TDB || USE_GDBM */
   }
 
 /* Otherwise unlink the temporary files. */
@@ -348,20 +463,20 @@ if (yield == 0 || yield == 1)
 else
   {
   printf("dbmbuild abandoned\n");
-  #if defined(USE_DB) || defined(USE_TDB)
-  unlink(temp_dbmname);
+  #if defined(USE_DB) || defined(USE_TDB) || defined(USE_GDBM)
+  Uunlink(temp_dbmname);
   #else
   if (is_db)
     {
-    sprintf(real_dbmname, "%s.db", temp_dbmname);
-    unlink(real_dbmname);
+    sprintf(CS real_dbmname, "%s.db", temp_dbmname);
+    Uunlink(real_dbmname);
     }
   else
     {
-    sprintf(real_dbmname, "%s.dir", temp_dbmname);
-    unlink(real_dbmname);
-    sprintf(real_dbmname, "%s.pag", temp_dbmname);
-    unlink(real_dbmname);
+    sprintf(CS real_dbmname, "%s.dir", temp_dbmname);
+    Uunlink(real_dbmname);
+    sprintf(CS real_dbmname, "%s.pag", temp_dbmname);
+    Uunlink(real_dbmname);
     }
   #endif /* USE_DB || USE_TDB */
   }
