@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/lookups/dbmdb.c,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -88,13 +90,14 @@ the keylength in order to include the terminating zero. */
 
 int
 dbmdb_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
 EXIM_DB *d = (EXIM_DB *)handle;
 EXIM_DATUM key, data;
 
 filename = filename;    /* Keep picky compilers happy */
 errmsg = errmsg;
+do_cache = do_cache;
 
 EXIM_DATUM_INIT(key);               /* Some DBM libraries require datums to */
 EXIM_DATUM_INIT(data);              /* be cleared before use. */
@@ -120,9 +123,10 @@ return FAIL;
 
 int
 dbmnz_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
-return dbmdb_find(handle, filename, keystring, length-1, result, errmsg);
+return dbmdb_find(handle, filename, keystring, length-1, result, errmsg,
+  do_cache);
 }
 
 
