@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/lookups/lsearch.c,v 1.3 2005/01/11 15:51:03 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -78,8 +80,22 @@ internal_lsearch_find(void *handle, uschar *filename, uschar *keystring,
 FILE *f = (FILE *)handle;
 BOOL last_was_eol = TRUE;
 BOOL this_is_eol = TRUE;
+int old_pool = store_pool;
+void *reset_point = NULL;
 uschar buffer[4096];
 
+/* Wildcard searches may use up some store, because of expansions. We don't
+want them to fill up our search store. What we do is set the pool to the main
+pool and get a point to reset to later. Wildcard searches could also issue
+lookups, but internal_search_find will take care of that, and the cache will be
+safely stored in the search pool again. */
+
+if(type == LSEARCH_WILD || type == LSEARCH_NWILD)
+  {
+  store_pool = POOL_MAIN;
+  reset_point = store_get(0);
+  }
+
 filename = filename;  /* Keep picky compilers happy */
 errmsg = errmsg;
 
@@ -197,7 +213,7 @@ for (last_was_eol = TRUE;
       int maskoffset;
       int save = buffer[linekeylength];
       buffer[linekeylength] = 0;
-      if (!string_is_ip_address(buffer, &maskoffset) ||
+      if (string_is_ip_address(buffer, &maskoffset) == 0 ||
           !host_is_in_net(keystring, buffer, maskoffset)) continue;
       buffer[linekeylength] = save;
       }
@@ -215,6 +231,14 @@ for (last_was_eol = TRUE;
     while (isspace((uschar)*s)) s++;
     }
 
+  /* Reset dynamic store, if we need to, and revert to the search pool */
+
+  if (reset_point != NULL)
+    {
+    store_reset(reset_point);
+    store_pool = old_pool;
+    }
+
   /* Now we want to build the result string to contain the data. There can be
   two kinds of continuation: (a) the physical line may not all have fitted into
   the buffer, and (b) there may be logical continuation lines, for which we
@@ -264,10 +288,19 @@ for (last_was_eol = TRUE;
     }
 
   yield[ptr] = 0;
+  store_reset(yield + ptr + 1);
   *result = yield;
   return OK;
   }
 
+/* Reset dynamic store, if we need to */
+
+if (reset_point != NULL)
+  {
+  store_reset(reset_point);
+  store_pool = old_pool;
+  }
+
 return FAIL;
 }
 
@@ -280,8 +313,9 @@ return FAIL;
 
 int
 lsearch_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
+do_cache = do_cache;  /* Keep picky compilers happy */
 return internal_lsearch_find(handle, filename, keystring, length, result,
   errmsg, LSEARCH_PLAIN);
 }
@@ -296,8 +330,9 @@ return internal_lsearch_find(handle, filename, keystring, length, result,
 
 int
 wildlsearch_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
+do_cache = do_cache;  /* Keep picky compilers happy */
 return internal_lsearch_find(handle, filename, keystring, length, result,
   errmsg, LSEARCH_WILD);
 }
@@ -312,8 +347,9 @@ return internal_lsearch_find(handle, filename, keystring, length, result,
 
 int
 nwildlsearch_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
+do_cache = do_cache;  /* Keep picky compilers happy */
 return internal_lsearch_find(handle, filename, keystring, length, result,
   errmsg, LSEARCH_NWILD);
 }
@@ -329,17 +365,20 @@ return internal_lsearch_find(handle, filename, keystring, length, result,
 
 int
 iplsearch_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
+do_cache = do_cache;  /* Keep picky compilers happy */
 if ((length == 1 && keystring[0] == '*') ||
-    string_is_ip_address(keystring, NULL))
+    string_is_ip_address(keystring, NULL) > 0)
   {
   return internal_lsearch_find(handle, filename, keystring, length, result,
     errmsg, LSEARCH_IP);
   }
 else
   {
-  *errmsg = string_sprintf("\"%s\" is not a valid iplsearch key", keystring);
+  *errmsg = string_sprintf("\"%s\" is not a valid iplsearch key (an IP "
+    "address, with optional CIDR mask, is wanted): "
+    "in a host list, use net-iplsearch as the search type", keystring);
   return DEFER;
   }
 }
