@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/lookups/mysql.c,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Thanks to Paul Kelly for contributing the original code for these
@@ -72,9 +74,6 @@ while ((cn = mysql_connections) != NULL)
   mysql_connections = cn->next;
   DEBUG(D_lookup) debug_printf("close MYSQL connection: %s\n", cn->server);
   mysql_close(cn->handle);
-  store_free(cn->server);
-  store_free(cn->handle);
-  store_free(cn);
   }
 }
 
@@ -93,6 +92,7 @@ Arguments:
   resultptr    where to store the result
   errmsg       where to point an error message
   defer_break  TRUE if no more servers are to be tried after DEFER
+  do_cache     set false if data is changed
 
 The server string is of the form "host/dbname/user/password". The host can be
 host:port. This string is in a nextinlist temporary buffer, so can be
@@ -103,7 +103,7 @@ Returns:       OK, FAIL, or DEFER
 
 static int
 perform_mysql_search(uschar *query, uschar *server, uschar **resultptr,
-  uschar **errmsg, BOOL *defer_break)
+  uschar **errmsg, BOOL *defer_break, BOOL *do_cache)
 {
 MYSQL *mysql_handle = NULL;        /* Keep compilers happy */
 MYSQL_RES *mysql_result = NULL;
@@ -137,7 +137,7 @@ for (i = 3; i > 0; i--)
     }
   *pp++ = 0;
   sdata[i] = pp;
-  if (i == 3) server_copy = string_copy_malloc(server);  /* sans password */
+  if (i == 3) server_copy = string_copy(server);  /* sans password */
   }
 sdata[0] = server;   /* What's left at the start */
 
@@ -197,7 +197,7 @@ if (cn == NULL)
 
   /* Get store for a new handle, initialize it, and connect to the server */
 
-  mysql_handle = store_malloc(sizeof(MYSQL));
+  mysql_handle = store_get(sizeof(MYSQL));
   mysql_init(mysql_handle);
   if (mysql_real_connect(mysql_handle,
       /*  host        user         passwd     database */
@@ -206,15 +206,13 @@ if (cn == NULL)
     {
     *errmsg = string_sprintf("MYSQL connection failed: %s",
       mysql_error(mysql_handle));
-    store_free(mysql_handle);
-    store_free(server_copy);
     *defer_break = FALSE;
     goto MYSQL_EXIT;
     }
 
   /* Add the connection to the cache */
 
-  cn = store_malloc(sizeof(mysql_connection));
+  cn = store_get(sizeof(mysql_connection));
   cn->server = server_copy;
   cn->handle = mysql_handle;
   cn->next = mysql_connections;
@@ -243,7 +241,9 @@ if (mysql_query(mysql_handle, CS query) != 0)
 namely INSERT, UPDATE, or DELETE, an error occurs here. However, this situation
 can be detected by calling mysql_field_count(). If its result is zero, no data
 was expected (this is all explained clearly in the MySQL manual). In this case,
-we return the number of rows affected by the command. */
+we return the number of rows affected by the command. In this event, we do NOT
+want to cache the result; also the whole cache for the handle must be cleaned
+up. Setting do_cache FALSE requests this. */
 
 if ((mysql_result = mysql_use_result(mysql_handle)) == NULL)
   {
@@ -251,6 +251,7 @@ if ((mysql_result = mysql_use_result(mysql_handle)) == NULL)
     {
     DEBUG(D_lookup) debug_printf("MYSQL: query was not one that returns data\n");
     result = string_sprintf("%d", mysql_affected_rows(mysql_handle));
+    *do_cache = FALSE;
     goto MYSQL_EXIT;
     }
   *errmsg = string_sprintf("MYSQL: lookup result failed: %s\n",
@@ -285,36 +286,8 @@ while ((mysql_row_data = mysql_fetch_row(mysql_result)) != NULL)
 
   else for (i = 0; i < num_fields; i++)
     {
-    result = string_cat(result, &ssize, &offset, US fields[i].name,
-      Ustrlen(fields[i].name));
-    result = string_cat(result, &ssize, &offset, US"=", 1);
-
-    /* Quote the value if it contains spaces or is empty */
-
-    if (mysql_row_data[i] == NULL)    /* NULL value */
-      {
-      result = string_cat(result, &ssize, &offset, US"\"\"", 2);
-      }
-
-    else if (mysql_row_data[i][0] == 0 ||
-             Ustrchr(mysql_row_data[i], ' ') != NULL)
-      {
-      int j;
-      result = string_cat(result, &ssize, &offset, US"\"", 1);
-      for (j = 0; j < lengths[i]; j++)
-        {
-        if (mysql_row_data[i][j] == '\"' || mysql_row_data[i][j] == '\\')
-          result = string_cat(result, &ssize, &offset, US"\\", 1);
-        result = string_cat(result, &ssize, &offset, US mysql_row_data[i]+j, 1);
-        }
-      result = string_cat(result, &ssize, &offset, US"\"", 1);
-      }
-    else
-      {
-      result = string_cat(result, &ssize, &offset, US mysql_row_data[i],
-        lengths[i]);
-      }
-    result = string_cat(result, &ssize, &offset, US" ", 1);
+    result = lf_quote(US fields[i].name, US mysql_row_data[i], lengths[i],
+      result, &ssize, &offset);
     }
   }
 
@@ -327,9 +300,14 @@ if (result == NULL)
   yield = FAIL;
   *errmsg = US"MYSQL: no data found";
   }
-else result[offset] = 0;
+else
+  {
+  result[offset] = 0;
+  store_reset(result + offset + 1);
+  }
 
-/* Get here by goto from various error checks. */
+/* Get here by goto from various error checks and from the case where no data
+was read (e.g. an update query). */
 
 MYSQL_EXIT:
 
@@ -340,7 +318,7 @@ if (mysql_result != NULL) mysql_free_result(mysql_result);
 
 /* Non-NULL result indicates a sucessful result */
 
-if(result != NULL)
+if (result != NULL)
   {
   *resultptr = result;
   return OK;
@@ -365,7 +343,7 @@ deferred with a retryable error. */
 
 int
 mysql_find(void *handle, uschar *filename, uschar *query, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
 int sep = 0;
 uschar *server;
@@ -377,7 +355,8 @@ DEBUG(D_lookup) debug_printf("MYSQL query: %s\n", query);
 while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
   {
   BOOL defer_break = FALSE;
-  int rc = perform_mysql_search(query, server, result, errmsg, &defer_break);
+  int rc = perform_mysql_search(query, server, result, errmsg, &defer_break,
+    do_cache);
   if (rc != DEFER || defer_break) return rc;
   }
 
