@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/lookups/nis.c,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -55,11 +57,12 @@ code. */
 
 int
 nis_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
 int rc;
 uschar *nis_data;
 int nis_data_length;
+do_cache = do_cache;   /* Placate picky compilers */
 if ((rc = yp_match(CS handle, CS filename, CS keystring, length,
     CSS &nis_data, &nis_data_length)) == 0)
   {
@@ -80,11 +83,12 @@ return (rc == YPERR_KEY || rc == YPERR_MAP)? FAIL : DEFER;
 
 int
 nis0_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
 int rc;
 uschar *nis_data;
 int nis_data_length;
+do_cache = do_cache;   /* Placate picky compilers */
 if ((rc = yp_match(CS handle, CS filename, CS keystring, length + 1,
     CSS &nis_data, &nis_data_length)) == 0)
   {
