@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/lookups/nisplus.c,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -58,7 +60,7 @@ equals sign. */
 
 int
 nisplus_find(void *handle, uschar *filename, uschar *query, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
 int i;
 int ssize = 0;
@@ -73,6 +75,8 @@ struct table_obj *ta;
 uschar *p = query + length;
 uschar *yield = NULL;
 
+do_cache = do_cache;   /* Placate picky compilers */
+
 /* Search backwards for a colon to see if a result field name
 has been given. */
 
@@ -207,7 +211,11 @@ field name, ensure the concatenated data is zero-terminated. */
 if (field_name != NULL)
   *errmsg = string_sprintf("NIS+ field %s not found for %s", field_name,
     query);
-else yield[offset] = 0;
+else
+  {
+  yield[offset] = 0;
+  store_reset(yield + offset + 1);
+  }
 
 /* Restore the colon in the query, and free result store before
 finishing. */
