@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/lookups/passwd.c,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -35,7 +37,7 @@ return (void *)(-1);     /* Just return something non-null */
 
 int
 passwd_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
 struct passwd *pw;
 
@@ -43,6 +45,7 @@ handle = handle;         /* Keep picky compilers happy */
 filename = filename;
 length = length;
 errmsg = errmsg;
+do_cache = do_cache;
 
 if (!route_finduser(keystring, &pw, NULL)) return FAIL;
 *result = string_sprintf("*:%d:%d:%s:%s:%s", (int)pw->pw_uid, (int)pw->pw_gid,
