@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/lookups/pgsql.c,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Thanks to Petr Cech for contributing the original code for these
@@ -70,8 +72,6 @@ while ((cn = pgsql_connections) != NULL)
   pgsql_connections = cn->next;
   DEBUG(D_lookup) debug_printf("close PGSQL connection: %s\n", cn->server);
   PQfinish(cn->handle);
-  store_free(cn->server);
-  store_free(cn);
   }
 }
 
@@ -107,13 +107,14 @@ Arguments:
   resultptr    where to store the result
   errmsg       where to point an error message
   defer_break  TRUE if no more servers are to be tried after DEFER
+  do_cache     set FALSE if data is changed
 
 Returns:       OK, FAIL, or DEFER
 */
 
 static int
 perform_pgsql_search(uschar *query, uschar *server, uschar **resultptr,
-  uschar **errmsg, BOOL *defer_break)
+  uschar **errmsg, BOOL *defer_break, BOOL *do_cache)
 {
 PGconn *pg_conn = NULL;
 PGresult *pg_result = NULL;
@@ -145,7 +146,7 @@ for (i = 2; i >= 0; i--)
     }
   *pp++ = 0;
   sdata[i] = pp;
-  if (i == 2) server_copy = string_copy_malloc(server);  /* sans password */
+  if (i == 2) server_copy = string_copy(server);  /* sans password */
   }
 
 /* The total server string has now been truncated so that what is left at the
@@ -238,17 +239,17 @@ if (cn == NULL)
 
   if(PQstatus(pg_conn) == CONNECTION_BAD)
     {
+    store_reset(server_copy);
     *errmsg = string_sprintf("PGSQL connection failed: %s",
       PQerrorMessage(pg_conn));
     PQfinish(pg_conn);
-    store_free(server_copy);
     *defer_break = FALSE;
     goto PGSQL_EXIT;
     }
 
   /* Add the connection to the cache */
 
-  cn = store_malloc(sizeof(pgsql_connection));
+  cn = store_get(sizeof(pgsql_connection));
   cn->server = server_copy;
   cn->handle = pg_conn;
   cn->next = pgsql_connections;
@@ -271,9 +272,12 @@ else
     case PGRES_EMPTY_QUERY:
     case PGRES_COMMAND_OK:
     /* The command was successful but did not return any data since it was
-     * not SELECT but either an INSERT, UPDATE or DELETE statement. */
+     * not SELECT but either an INSERT, UPDATE or DELETE statement. Tell the
+     * high level code to not cache this query, and clean the current cache for
+     * this handle by setting *do_cache FALSE. */
     result = string_copy(US PQcmdTuples(pg_result));
     offset = Ustrlen(result);
+    *do_cache = FALSE;
     DEBUG(D_lookup) debug_printf("PGSQL: command does not return any data "
       "but was successful. Rows affected: %s\n", result);
 
@@ -314,30 +318,9 @@ for (i = 0; i < num_tuples; i++)
     int j;
     for (j = 0; j < num_fields; j++)
       {
-      uschar *tmp = US PQfname(pg_result, j);
-      result = string_cat(result, &ssize, &offset, tmp, Ustrlen(tmp));
-      result = string_cat(result, &ssize, &offset, US"=", 1);
-
-      /* Quote the value if it contains spaces or is empty */
-
-      tmp = US PQgetvalue(pg_result, i, j);
-      if (*tmp == 0 || (Ustrchr(tmp, ' ') != NULL))
-        {
-        int k, t_len = Ustrlen(tmp);
-        result = string_cat(result, &ssize, &offset, US"\"", 1);
-        for(k = 0; k < t_len; k++)
-          {
-          if (tmp[k] == '\"' || tmp[k] == '\\')
-             result = string_cat(result, &ssize, &offset, US"\\", 1);
-          result = string_cat(result, &ssize, &offset, tmp+k, 1);
-          }
-        result = string_cat(result, &ssize, &offset, US"\"", 1);
-        }
-      else
-        {
-        result = string_cat(result, &ssize, &offset, tmp, Ustrlen(tmp));
-        }
-      result = string_cat(result, &ssize, &offset, US" ", 1);
+      uschar *tmp = US PQgetvalue(pg_result, i, j);
+      result = lf_quote(US PQfname(pg_result, j), tmp, Ustrlen(tmp), result,
+        &ssize, &offset);
       }
     }
   }
@@ -351,7 +334,11 @@ if (result == NULL)
   yield = FAIL;
   *errmsg = US"PGSQL: no data found";
   }
-else result[offset] = 0;
+else
+  {
+  result[offset] = 0;
+  store_reset(result + offset + 1);
+  }
 
 /* Get here by goto from various error checks. */
 
@@ -389,7 +376,7 @@ deferred with a retryable error. */
 
 int
 pgsql_find(void *handle, uschar *filename, uschar *query, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
 int sep = 0;
 uschar *server;
@@ -402,7 +389,8 @@ while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
         != NULL)
   {
   BOOL defer_break;
-  int rc = perform_pgsql_search(query, server, result, errmsg, &defer_break);
+  int rc = perform_pgsql_search(query, server, result, errmsg, &defer_break,
+    do_cache);
   if (rc != DEFER || defer_break) return rc;
   }
 
