@@ -1,13 +1,16 @@
+/* $Cambridge: exim/exim-src/src/lookups/whoson.h,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the whoson lookup */
 
 extern void *whoson_open(uschar *, uschar **);
-extern int   whoson_find(void *, uschar *, uschar *, int, uschar **, uschar **);
+extern int   whoson_find(void *, uschar *, uschar *, int, uschar **, uschar **,
+               BOOL *);
 
 /* End of lookups/whoson.h */
