@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/macros.h,v 1.9 2005/02/17 11:58:26 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -70,7 +72,7 @@ as unsigned. */
 a no-op once an SSL session is in progress. */
 
 #ifdef SUPPORT_TLS
-#define mac_smtp_fflush() if (!tls_active) fflush(smtp_out);
+#define mac_smtp_fflush() if (tls_active < 0) fflush(smtp_out);
 #else
 #define mac_smtp_fflush() fflush(smtp_out);
 #endif
@@ -129,12 +131,12 @@ enough to hold all the headers from a normal kind of message. */
 into big_buffer_size and in some circumstances increased. It should be at least
 as long as the maximum path length. */
 
-#if defined PATH_MAX && PATH_MAX > 1024
+#if defined PATH_MAX && PATH_MAX > 16384
 #define BIG_BUFFER_SIZE PATH_MAX
-#elif defined MAXPATHLEN && MAXPATHLEN > 1024
+#elif defined MAXPATHLEN && MAXPATHLEN > 16384
 #define BIG_BUFFER_SIZE MAXPATHLEN
 #else
-#define BIG_BUFFER_SIZE 1024
+#define BIG_BUFFER_SIZE 16384
 #endif
 
 /* This limits the length of data returned by local_scan(). Because it is
@@ -225,6 +227,12 @@ enum {
   CEE_EXEC_PANIC            /* Panic-die if exec fails */
 };
 
+/* Bit values for filter_test */
+
+#define FTEST_NONE     0    /* Not filter testing */
+#define FTEST_USER     1    /* Testing user filter */
+#define FTEST_SYSTEM   2    /* Testing system filter */
+
 /* Returns from the routing, transport and authentication functions (not all
 apply to all of them). Some other functions also use these convenient values,
 and some additional values are used only by non-driver functions.
@@ -232,10 +240,10 @@ and some additional values are used only by non-driver functions.
 OK, FAIL, DEFER, and ERROR are also declared in local_scan.h for use in the
 local_scan() function. Do not change them unilaterally. */
 
-#define  OK            0     /* Successful match */
-#define  DEFER         1     /* Defer - some problem */
-#define  FAIL          2     /* Matching failed */
-#define  ERROR         3     /* Internal or config error */
+#define  OK            0    /* Successful match */
+#define  DEFER         1    /* Defer - some problem */
+#define  FAIL          2    /* Matching failed */
+#define  ERROR         3    /* Internal or config error */
 /***********/
 #define DECLINE        4    /* Declined to handle the address, pass to next
                                  router unless no_more is set */
@@ -251,6 +259,13 @@ local_scan() function. Do not change them unilaterally. */
 #define FAIL_SEND     13    /* send() failed in authenticator */
 #define FAIL_DROP     14    /* Fail and drop connection (used in ACL) */
 
+/* Returns from the deliver_message() function */
+
+#define DELIVER_ATTEMPTED_NORMAL   0  /* Tried a normal delivery */
+#define DELIVER_MUA_SUCCEEDED      1  /* Success when mua_wrapper is set */
+#define DELIVER_MUA_FAILED         2  /* Failure when mua_wrapper is set */
+#define DELIVER_NOT_ATTEMPTED      3  /* Not tried (no msg or is locked */
+
 /* Returns from DNS lookup functions. */
 
 enum { DNS_SUCCEED, DNS_NOMATCH, DNS_NODATA, DNS_AGAIN, DNS_FAIL };
@@ -352,16 +367,17 @@ only in the name table to set all options in both bit maps. */
 #define LX_incoming_port               0x80000020
 #define LX_outgoing_port               0x80000040
 #define LX_queue_time                  0x80000080
-#define LX_received_sender             0x80000100
-#define LX_received_recipients         0x80000200
-#define LX_rejected_header             0x80000400
-#define LX_return_path_on_delivery     0x80000800
-#define LX_sender_on_delivery          0x80001000
-#define LX_smtp_confirmation           0x80002000
-#define LX_subject                     0x80004000
-#define LX_tls_certificate_verified    0x80008000
-#define LX_tls_cipher                  0x80010000
-#define LX_tls_peerdn                  0x80020000
+#define LX_queue_time_overall          0x80000100
+#define LX_received_sender             0x80000200
+#define LX_received_recipients         0x80000400
+#define LX_rejected_header             0x80000800
+#define LX_return_path_on_delivery     0x80001000
+#define LX_sender_on_delivery          0x80002000
+#define LX_smtp_confirmation           0x80004000
+#define LX_subject                     0x80008000
+#define LX_tls_certificate_verified    0x80010000
+#define LX_tls_cipher                  0x80020000
+#define LX_tls_peerdn                  0x80040000
 
 #define L_default     (L_connection_reject        | \
                        L_delay_delivery           | \
@@ -423,6 +439,7 @@ to conflict with system errno values. */
 #define ERRNO_CALLOUTDEFER   (-41)   /* When verifying */
 #define ERRNO_AUTHFAIL       (-42)   /* When required by client */
 #define ERRNO_CONNECTTIMEOUT (-43)   /* Used internally in smtp transport */
+#define ERRNO_RCPT4XX        (-44)   /* RCPT gave 4xx error */
 
 /* These must be last, so all retry deferments can easily be identified */
 
@@ -457,23 +474,25 @@ and some forbid, reflecting the "allow" and "forbid" options in the redirect
 router, which were chosen to represent the standard situation for users'
 .forward files. */
 
-#define RDO_BLACKHOLE  0x00000001  /* Forbid :blackhole: */
-#define RDO_DEFER      0x00000002  /* Allow :defer: or "defer" */
-#define RDO_EACCES     0x00000004  /* Ignore EACCES */
-#define RDO_ENOTDIR    0x00000008  /* Ignore ENOTDIR */
-#define RDO_EXISTS     0x00000010  /* Forbid "exists" in expansion in filter */
-#define RDO_FAIL       0x00000020  /* Allow :fail: or "fail" */
-#define RDO_FILTER     0x00000040  /* Allow a filter script */
-#define RDO_FREEZE     0x00000080  /* Allow "freeze" */
-#define RDO_INCLUDE    0x00000100  /* Forbid :include: */
-#define RDO_LOG        0x00000200  /* Forbid "log" */
-#define RDO_LOOKUP     0x00000400  /* Forbid "lookup" in expansion in filter */
-#define RDO_PERL       0x00000800  /* Forbid "perl" in expansion in filter */
-#define RDO_READFILE   0x00001000  /* Forbid "readfile" in exp in filter */
-#define RDO_READSOCK   0x00002000  /* Forbid "readsocket" in exp in filter */
-#define RDO_RUN        0x00004000  /* Forbid "run" in expansion in filter */
-#define RDO_REALLOG    0x00008000  /* Really do log (not testing/verifying) */
-#define RDO_REWRITE    0x00010000  /* Rewrite generated addresses */
+#define RDO_BLACKHOLE    0x00000001  /* Forbid :blackhole: */
+#define RDO_DEFER        0x00000002  /* Allow :defer: or "defer" */
+#define RDO_EACCES       0x00000004  /* Ignore EACCES */
+#define RDO_ENOTDIR      0x00000008  /* Ignore ENOTDIR */
+#define RDO_EXISTS       0x00000010  /* Forbid "exists" in expansion in filter */
+#define RDO_FAIL         0x00000020  /* Allow :fail: or "fail" */
+#define RDO_FILTER       0x00000040  /* Allow a filter script */
+#define RDO_FREEZE       0x00000080  /* Allow "freeze" */
+#define RDO_INCLUDE      0x00000100  /* Forbid :include: */
+#define RDO_LOG          0x00000200  /* Forbid "log" */
+#define RDO_LOOKUP       0x00000400  /* Forbid "lookup" in expansion in filter */
+#define RDO_PERL         0x00000800  /* Forbid "perl" in expansion in filter */
+#define RDO_READFILE     0x00001000  /* Forbid "readfile" in exp in filter */
+#define RDO_READSOCK     0x00002000  /* Forbid "readsocket" in exp in filter */
+#define RDO_RUN          0x00004000  /* Forbid "run" in expansion in filter */
+#define RDO_REALLOG      0x00008000  /* Really do log (not testing/verifying) */
+#define RDO_REWRITE      0x00010000  /* Rewrite generated addresses */
+#define RDO_EXIM_FILTER  0x00020000  /* Forbid Exim filters */
+#define RDO_SIEVE_FILTER 0x00040000  /* Forbid Sieve filters */
 
 /* This is the set that apply to expansions in filters */
 
@@ -487,7 +506,7 @@ automated, but I haven't bothered. Keep this list in step with the above! */
 enum { RDON_BLACKHOLE, RDON_DEFER, RDON_EACCES, RDON_ENOTDIR, RDON_EXISTS,
   RDON_FAIL, RDON_FILTER, RDON_FREEZE, RDON_INCLUDE, RDON_LOG, RDON_LOOKUP,
   RDON_PERL, RDON_READFILE, RDON_READSOCK, RDON_RUN, RDON_REALLOG,
-  RDON_REWRITE };
+  RDON_REWRITE, RDON_EXIM_FILTER, RDON_SIEVE_FILTER };
 
 /* Results of filter or forward file processing. Some are only from a filter;
 some are only from a forward file. */
@@ -526,6 +545,15 @@ return values from header_checkname(). */
 #define htype_envelope_to   'e'
 #define htype_subject       's'
 
+/* These values are used only when adding new headers from an ACL; they too
+never get onto the spool. The type of the added header is set by reference
+to the header name, by calling header_checkname(). */
+
+#define htype_add_top       'a'
+#define htype_add_rec       'r'
+#define htype_add_bot       'z'
+#define htype_add_rfc       'f'
+
 /* Types of item in options lists. These are the bottom 8 bits of the "type"
 field, which is an int. The opt_void value is used for entries in tables that
 point to special types of value that are accessed only indirectly (e.g. the
@@ -567,14 +595,15 @@ enum { v_none, v_sender, v_recipient, v_expn };
 
 /* Option flags for verify_address() */
 
-#define vopt_fake_sender         0x0001   /* for verify=sender=<address> */
-#define vopt_is_recipient        0x0002
-#define vopt_qualify             0x0004
-#define vopt_expn                0x0008
-#define vopt_callout_postmaster  0x0010   /* during callout */
-#define vopt_callout_random      0x0020   /* during callout */
-#define vopt_callout_no_cache    0x0040   /* disable callout cache */
-#define vopt_callout_recipsender 0x0080   /* use real sender to verify recip */
+#define vopt_fake_sender          0x0001   /* for verify=sender=<address> */
+#define vopt_is_recipient         0x0002
+#define vopt_qualify              0x0004
+#define vopt_expn                 0x0008
+#define vopt_callout_postmaster   0x0010   /* during callout */
+#define vopt_callout_random       0x0020   /* during callout */
+#define vopt_callout_no_cache     0x0040   /* disable callout cache */
+#define vopt_callout_recipsender  0x0080   /* use real sender to verify recip */
+#define vopt_callout_recippmaster 0x0100   /* use postmaster to verify recip */
 
 /* Values for fields in callout cache records */
 
@@ -655,9 +684,11 @@ enum {
 
 /* Flags for host_find_bydns() */
 
-#define HOST_FIND_BY_SRV       0x001
-#define HOST_FIND_BY_MX        0x002
-#define HOST_FIND_BY_A         0x004
+#define HOST_FIND_BY_SRV          0x0001
+#define HOST_FIND_BY_MX           0x0002
+#define HOST_FIND_BY_A            0x0004
+#define HOST_FIND_QUALIFY_SINGLE  0x0008
+#define HOST_FIND_SEARCH_PARENTS  0x0010
 
 /* Actions applied to specific messages. */
 
@@ -705,13 +736,32 @@ be added */
 
 enum { MCL_STRING, MCL_DOMAIN, MCL_HOST, MCL_ADDRESS, MCL_LOCALPART };
 
-/* Codes for the places from which ACLs can be called. Keep in step with the
-tables of names and response codes in globals.c. */
+/* Codes for the places from which ACLs can be called. These are cunningly
+ordered to make it easy to implement tests for certain ACLs when processing
+"control" modifiers, by means of a maximum "where" value. Do not modify this
+order without checking carefully! Furthermore, remember to keep these in step
+with the tables of names and response codes in globals.c. */
 
-enum { ACL_WHERE_AUTH,     ACL_WHERE_CONNECT, ACL_WHERE_DATA, ACL_WHERE_ETRN,
-       ACL_WHERE_EXPN,     ACL_WHERE_HELO,    ACL_WHERE_MAIL,
-       ACL_WHERE_MAILAUTH, ACL_WHERE_RCPT,
-       ACL_WHERE_STARTTLS, ACL_WHERE_VRFY,    ACL_WHERE_NOTSMTP };
+enum { ACL_WHERE_RCPT,       /* Some controls are for RCPT only */
+       ACL_WHERE_MAIL,       /* )                                           */
+       ACL_WHERE_PREDATA,    /* ) There are several tests for "in message", */
+                             /* ) implemented by <= WHERE_NOTSMTP           */
+#ifdef WITH_CONTENT_SCAN     /* )                                           */
+       ACL_WHERE_MIME,       /* )                                           */
+#endif                       /* )                                           */
+       ACL_WHERE_DATA,       /* )                                           */
+       ACL_WHERE_NOTSMTP,    /* )                                           */
+
+       ACL_WHERE_AUTH,       /* These remaining ones are not currently    */
+       ACL_WHERE_CONNECT,    /* required to be in a special order so they */
+       ACL_WHERE_ETRN,       /* are just alphabetical.                    */
+       ACL_WHERE_EXPN,
+       ACL_WHERE_HELO,
+       ACL_WHERE_MAILAUTH,
+       ACL_WHERE_QUIT,
+       ACL_WHERE_STARTTLS,
+       ACL_WHERE_VRFY
+     };
 
 /* Situations for spool_write_header() */
 
