@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/match.c,v 1.5 2005/02/17 11:58:26 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for matching strings */
@@ -210,8 +212,8 @@ if (cb->at_is_special && pattern[0] == '@')
       ignore_target_hosts,
       HOST_FIND_BY_MX,     /* search only for MX, not SRV or A */
       NULL,                /* service name not relevant */
-      FALSE,               /* no widening */
-      FALSE,               /* no widening */
+      NULL,                /* srv_fail_domains not relevant */
+      NULL,                /* mx_fail_domains not relevant */
       NULL,                /* no feedback FQDN */
       &removed);           /* feedback if local removed */
 
@@ -424,6 +426,7 @@ match_check_list(uschar **listptr, int sep, tree_node **anchorptr,
 int yield = OK;
 unsigned int *original_cache_bits = *cache_ptr;
 BOOL include_unknown = FALSE;
+BOOL ignore_unknown = FALSE;
 uschar *list;
 uschar *sss;
 uschar *ot = NULL;
@@ -437,12 +440,15 @@ HDEBUG(D_any)
   if (listname[0] != 0) ot = string_sprintf("%s in %s?", name, listname);
   }
 
-/* If the list is empty, the answer is no. This case happens only when checking
-named options, so ot will be set. */
+/* If the list is empty, the answer is no. Skip the debugging output for
+an unnamed list. */
 
 if (*listptr == NULL)
   {
-  HDEBUG(D_lists) debug_printf("%s no (option unset)\n", ot);
+  HDEBUG(D_lists)
+    {
+    if (ot != NULL) debug_printf("%s no (option unset)\n", ot);
+    }
   return FAIL;
   }
 
@@ -526,6 +532,13 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
     if (Ustrcmp(ss, "+include_unknown") == 0)
       {
       include_unknown = TRUE;
+      ignore_unknown = FALSE;
+      continue;
+      }
+    if (Ustrcmp(ss, "+ignore_unknown") == 0)
+      {
+      ignore_unknown = TRUE;
+      include_unknown = FALSE;
       continue;
       }
     }
@@ -617,9 +630,11 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
             store_pool = POOL_PERM;
             p = store_get(sizeof(namedlist_cacheblock));
             p->key = string_copy(get_check_key(arg, type));
+
+
+            p->data = (*valueptr == NULL)? NULL : string_copy(*valueptr);
             store_pool = old_pool;
 
-            p->data = *valueptr;
             p->next = nb->cache_data;
             nb->cache_data = p;
             if (*valueptr != NULL)
@@ -687,11 +702,19 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
         which it was. */
 
         case ERROR:
-        HDEBUG(D_lists)
-          debug_printf("%s %s (%s)\n", ot, include_unknown? "yes":"no", error);
-        if (!include_unknown) return FAIL;
-        log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
-        return OK;
+        if (ignore_unknown)
+          {
+          HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_unknown",
+            error);
+          }
+        else
+          {
+          HDEBUG(D_lists) debug_printf("%s %s (%s)\n", ot,
+            include_unknown? "yes":"no", error);
+          if (!include_unknown) return FAIL;
+          log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
+          return OK;
+          }
         }
       }
     }
@@ -769,12 +792,20 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
         goto DEFER_RETURN;
 
         case ERROR:          /* host name lookup failed - this can only */
-        HDEBUG(D_lists)      /* be for an incoming host (not outgoing) */
-          debug_printf("%s %s (%s)\n", ot, include_unknown? "yes":"no", error);
-        fclose(f);
-        if (!include_unknown) return FAIL;
-        log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
-        return OK;
+        if (ignore_unknown)  /* be for an incoming host (not outgoing) */
+          {
+          HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_unknown",
+            error);
+          }
+        else
+         {
+          HDEBUG(D_lists) debug_printf("%s %s (%s)\n", ot,
+            include_unknown? "yes":"no", error);
+          fclose(f);
+          if (!include_unknown) return FAIL;
+          log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
+          return OK;
+          }
         }
       }
 
@@ -824,6 +855,8 @@ Arguments:
                  MCL_LOCALPART when matching a local part list (address lists
                    have their own function)
                  MCL_STRING for others (e.g. list of ciphers)
+                 MCL_NOEXPAND (whose value is greater than any of them) may
+                   be added to any value to suppress expansion of the list
   caseless       TRUE for (mostly) caseless matching - passed directly to
                    match_check_string()
   valueptr       pointer to where any lookup data is to be passed back,
@@ -848,7 +881,7 @@ cb.subject = caseless? string_copylc(s) : string_copy(s);
 cb.expand_setup = (sep > UCHAR_MAX)? 0 : -1;
 cb.use_partial = TRUE;
 cb.caseless = caseless;
-cb.at_is_special = (type == MCL_DOMAIN);
+cb.at_is_special = (type == MCL_DOMAIN || type == MCL_DOMAIN + MCL_NOEXPAND);
 if (valueptr != NULL) *valueptr = NULL;
 return  match_check_list(listptr, sep, anchorptr, &local_cache_bits,
   check_string, &cb, type, s, valueptr);
@@ -901,32 +934,24 @@ error = error;  /* Keep clever compilers from complaining */
 DEBUG(D_lists) debug_printf("address match: subject=%s pattern=%s\n",
   subject, pattern);
 
-/* Handle a regular expression, which must match the entire incoming address. */
+/* Handle a regular expression, which must match the entire incoming address.
+This may be the empty address. */
 
 if (*pattern == '^')
   return match_check_string(subject, pattern, cb->expand_setup, TRUE,
     cb->caseless, FALSE, NULL);
 
-/* If the subject is the empty string, the only pattern it can match (other
-than a regular expression) is the empty pattern. */
-
-if (*subject == 0) return (*pattern == 0)? OK : FAIL;
-
-/* Find the domain in the subject */
-
-sdomain = Ustrrchr(subject, '@');
-
-/* Handle the case of a pattern that is just a lookup. Skip over possible
-lookup names (letters, digits, hyphens). Skip over a possible * or *@ at
-the end. Then we must have a semicolon for it to be a lookup. */
+/* Handle a pattern that is just a lookup. Skip over possible lookup names
+(letters, digits, hyphens). Skip over a possible * or *@ at the end. Then we
+must have a semicolon for it to be a lookup. */
 
 for (s = pattern; isalnum(*s) || *s == '-'; s++);
 if (*s == '*') s++;
 if (*s == '@') s++;
 
-/* If it is a straight lookup, do a lookup for the whole address. Partial
-matching doesn't make sense here, so we ignore it, but write a panic log entry.
-However, *@ matching will be honoured. */
+/* If it is a straight lookup, do a lookup for the whole address. This may be
+the empty address. Partial matching doesn't make sense here, so we ignore it,
+but write a panic log entry. However, *@ matching will be honoured. */
 
 if (*s == ';')
   {
@@ -937,6 +962,16 @@ if (*s == ';')
     valueptr);
   }
 
+/* For the remaining cases, an empty subject matches only an empty pattern,
+because other patterns expect to have a local part and a domain to match
+against. */
+
+if (*subject == 0) return (*pattern == 0)? OK : FAIL;
+
+/* Find the subject's domain */
+
+sdomain = Ustrrchr(subject, '@');
+
 /* If the pattern starts with "@@" we have a split lookup, where the domain is
 looked up to obtain a list of local parts. If the subject's local part is just
 "*" (called from retry) the match always fails. */
