@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/os.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #ifdef STAND_ALONE
@@ -23,9 +25,11 @@ file has not. Some of them are tailored by macros defined in os.h files. */
 
 /* This function has the same functionality as the ANSI C signal() function,
 except that it arranges that, if the signal happens during a system call, the
-system call gets restarted. Different versions of Unix have different defaults,
-and different ways of achieving this. If the functionality is not available,
-the signal should be set to be ignored. */
+system call gets restarted. (Also, it doesn't return a result.) Different
+versions of Unix have different defaults, and different ways of setting up a
+restarting signal handler. If the functionality is not available, the signal
+should be set to be ignored. This function is used only for catching SIGUSR1.
+*/
 
 void
 os_restarting_signal(int sig, void (*handler)(int))
@@ -69,7 +73,7 @@ printf("Used SIG_IGN\n");
 #endif
 }
 
-#endif	/* OS_RESTARTING_SIGNAL */
+#endif  /* OS_RESTARTING_SIGNAL */
 
 
 #ifndef OS_NON_RESTARTING_SIGNAL
@@ -79,9 +83,10 @@ printf("Used SIG_IGN\n");
 
 /* This function has the same functionality as the ANSI C signal() function,
 except that it arranges that, if the signal happens during a system call, the
-system call gets interrupted. Different versions of Unix have different
-defaults, and different ways of achieving this. For systems for which we
-don't know what to do, just use the normal signal() function. */
+system call gets interrupted. (Also, it doesn't return a result.) Different
+versions of Unix have different defaults, and different ways of setting up a
+non-restarting signal handler. For systems for which we don't know what to do,
+just use the normal signal() function and hope for the best. */
 
 void
 os_non_restarting_signal(int sig, void (*handler)(int))
@@ -128,7 +133,7 @@ printf("Used default signal()\n");
 #endif
 }
 
-#endif	/* OS_NON_RESTARTING_SIGNAL */
+#endif  /* OS_NON_RESTARTING_SIGNAL */
 
 
 
@@ -301,6 +306,7 @@ switch (n)
   case EX_NOPERM:      return "(could mean permission denied)";
   #endif
 
+  case EX_EXECFAILED:  return "(could mean unable to exec or command does not exist)";
   case EX_UNAVAILABLE: return "(could mean service or program unavailable)";
   case EX_CANTCREAT:   return "(could mean can't create output file)";
   case EX_TEMPFAIL:    return "(could mean temporary error)";
@@ -461,7 +467,7 @@ Problems:
 
   (1) Solaris 2 has the SIOGIFNUM call to get the number of interfaces, but
   other OS (including Solaris 1) appear not to. So just screw in a largeish
-  fixed number, defined by MAXINTERFACES. This is in the config.h file and
+  fixed number, defined by MAX_INTERFACES. This is in the config.h file and
   can be changed in Local/Makefile. Unfortunately, the www addressing scheme
   means that some hosts have a very large number of virtual interfaces. Such
   hosts are recommended to set local_interfaces to avoid problems with this.
@@ -483,7 +489,7 @@ Problems:
 
 Arguments:    none
 Returns:      a chain of ip_address_items, each pointing to a textual
-              version of an IP address
+              version of an IP address, with the port field set to zero
 */
 
 
@@ -541,13 +547,16 @@ ip_address_item *
 os_common_find_running_interfaces(void)
 {
 struct V_ifconf ifc;
-struct V_ifreq ifreq, *ifr;
+struct V_ifreq ifreq;
 int vs;
 ip_address_item *yield = NULL;
 ip_address_item *last = NULL;
 ip_address_item  *next;
 char *cp;
-char buf[MAXINTERFACES*sizeof(struct V_ifreq)];
+char buf[MAX_INTERFACES*sizeof(struct V_ifreq)];
+struct sockaddr *addrp;
+size_t len = 0;
+char addrbuf[256];
 
 /* We have to create a socket in order to do ioctls on it to find out
 what we want to know. */
@@ -555,9 +564,9 @@ what we want to know. */
 if ((vs = socket(FAMILY, SOCK_DGRAM, 0)) < 0)
   {
   #if HAVE_IPV6
-  DEBUG(9) debug_printf("Unable to create IPv6 socket to find interface "
-    "addresses:\n  error %d %s\nTrying for an IPv4 socket\n", errno,
-    strerror(errno));
+  DEBUG(D_interface)
+    debug_printf("Unable to create IPv6 socket to find interface addresses:\n  "
+      "error %d %s\nTrying for an IPv4 socket\n", errno, strerror(errno));
   vs = socket(AF_INET, SOCK_DGRAM, 0);
   if (vs < 0)
   #endif
@@ -580,6 +589,20 @@ if (ioctl(vs, V_GIFCONF, (char *)&ifc) < 0)
   log_write(0, LOG_PANIC_DIE, "Unable to get interface configuration: %d %s",
     errno, strerror(errno));
 
+/* If the buffer is big enough, the ioctl sets the value of ifc.V_ifc_len to
+the amount actually used. If the buffer isn't big enough, at least on some
+operating systems, ifc.V_ifc_len still gets set to correspond to the total
+number of interfaces, even though they don't all fit in the buffer. */
+
+if (ifc.V_ifc_len > sizeof(buf))
+  {
+  ifc.V_ifc_len = sizeof(buf);
+  DEBUG(D_interface)
+    debug_printf("more than %d interfaces found: remainder not used\n"
+      "(set MAX_INTERFACES in Local/Makefile and rebuild if you want more)\n",
+      MAX_INTERFACES);
+  }
+
 /* For each interface, check it is an IP interface, get its flags, and see if
 it is up; if not, skip.
 
@@ -588,32 +611,53 @@ return a vector of ifreq structures whose size is as defined by the structure.
 BSD systems allow sockaddrs to be longer than their sizeof, which in turn makes
 the ifreq structures longer than their sizeof. The code below has its origins
 in amd and ifconfig; it uses the sa_len field of each sockaddr to determine
-each item's length. */
+each item's length.
 
-for (cp = buf; cp < buf + ifc.V_ifc_len; cp +=
-    #ifdef HAVE_SA_LEN
-    mac_max(ifr->ifr_addr.sa_len, sizeof(ifr->ifr_addr)) +
-      sizeof(ifr->ifr_name))
-    #else
-    sizeof(*ifr))
-    #endif
+This is complicated by the fact that, at least on BSD systems, the data in the
+buffer is not guaranteed to be aligned. Thus, we must first copy the basic
+struct to some aligned memory before looking at the field in the fixed part to
+find its length, and then recopy the correct length. */
+
+for (cp = buf; cp < buf + ifc.V_ifc_len; cp += len)
   {
-  ifr = (struct V_ifreq *)cp;
+  memcpy((char *)&ifreq, cp, sizeof(ifreq));
+
+  #ifndef HAVE_SA_LEN
+  len = sizeof(struct V_ifreq);
+
+  #else
+  len = ((ifreq.ifr_addr.sa_len > sizeof(ifreq.ifr_addr))?
+          ifreq.ifr_addr.sa_len : sizeof(ifreq.ifr_addr)) +
+         sizeof(ifreq.V_ifr_name);
+  if (len > sizeof(addrbuf))
+    log_write(0, LOG_PANIC_DIE, "Address for %s interface is absurdly long",
+        ifreq.V_ifr_name);
+
+  #endif
 
   /* If not an IP interface, skip */
 
-  if (ifr->V_ifr_addr.V_family != AF_INET
+  if (ifreq.V_ifr_addr.V_family != AF_INET
   #if HAVE_IPV6
-    && ifr->V_ifr_addr.V_family != AF_INET6
+    && ifreq.V_ifr_addr.V_family != AF_INET6
   #endif
     ) continue;
 
-  /* Get the interface flags, and if the interface is down, continue */
+  /* Get the interface flags, and if the interface is down, continue. Formerly,
+  we treated the inability to get the flags as a panic-die error. However, it
+  seems that on some OS (Solaris 9 being the case noted), it is possible to
+  have an interface in this list for which this call fails because the
+  interface hasn't been "plumbed" to any protocol (IPv4 or IPv6). Therefore,
+  we now just treat this case as "down" as well. */
 
-  ifreq = *ifr;
   if (ioctl(vs, V_GIFFLAGS, (char *)&ifreq) < 0)
+    {
+    continue;
+    /*************
     log_write(0, LOG_PANIC_DIE, "Unable to get flags for %s interface: %d %s",
       ifreq.V_ifr_name, errno, strerror(errno));
+    *************/
+    }
   if ((ifreq.V_ifr_flags & IFF_UP) == 0) continue;
 
   /* On some operating systems we have to get the IP address of the interface
@@ -624,16 +668,21 @@ for (cp = buf; cp < buf + ifc.V_ifc_len; cp +=
   if (ioctl(vs, V_GIFADDR, (char *)&ifreq) < 0)
     log_write(0, LOG_PANIC_DIE, "Unable to get IP address for %s interface: "
       "%d %s", ifreq.V_ifr_name, errno, strerror(errno));
+  addrp = &ifreq.V_ifr_addr;
+
   #else
-  ifreq = *ifr;
+  memcpy((char *)&ifreq, cp, sizeof(ifreq));
+  memcpy(addrbuf, (char *)&(ifreq.V_ifr_addr), len - sizeof(ifreq.V_ifr_name));
+  addrp = (struct sockaddr *)addrbuf;
   #endif
 
   /* Create a data block for the address, fill in the data, and put it on the
-  chain. This data has to survive for ever, so use malloc. */
+  chain. */
 
-  next = store_malloc(sizeof(ip_address_item));
+  next = store_get(sizeof(ip_address_item));
   next->next = NULL;
-  (void)host_ntoa(-1, &ifreq.V_ifr_addr, next->address, NULL);
+  next->port = 0;
+  (void)host_ntoa(-1, addrp, next->address, NULL);
 
   if (yield == NULL) yield = last = next; else
     {
@@ -641,7 +690,7 @@ for (cp = buf; cp < buf + ifc.V_ifc_len; cp +=
     last = next;
     }
 
-  DEBUG(9) debug_printf("Actual local interface address is %s (%s)\n",
+  DEBUG(D_interface) debug_printf("Actual local interface address is %s (%s)\n",
     last->address, ifreq.V_ifr_name);
   }
 
@@ -655,27 +704,25 @@ return yield;
 
 /* Some experimental or developing OS (e.g. GNU/Hurd) do not have the ioctls,
 and there is no other way to get a list of the (IP addresses of) local
-interfaces. Instead, we fake a local_interfaces="127.0.0.1" without setting it.
-If local_interfaces is unset, the default behaviour is to listen to all
-interfaces on a single socket. The only difference is that the interfaces will
-not be recognized as local. Hence, the user should add further local IPs to
-host_treat_as_local. However, setting local_interfaces is probably the best
-approach. */
+interfaces. We just return the loopback address(es). */
 
 ip_address_item *
 os_common_find_running_interfaces(void)
 {
-ip_address_item *yield = NULL;
-char *my_local_interfaces = "127.0.0.1"; /* local_interfaces */
+ip_address_item *yield = store_get(sizeof(address_item));
+yield->address = US"127.0.0.1";
+yield->port = 0;
+yield->next = NULL;
 
-/* We know that local_interfaces is NULL when we come here from
-host_find_interfaces(). Also we know that if we set it to something else it
-will behave differently. In particular, it won't call us again, so this is not
-a loop. */
+#if HAVE_IPV6
+yield->next = store_get(sizeof(address_item));
+yield->next->address = US"::1";
+yield->next->port = 0;
+yield->next->next = NULL;
+#endif
 
-local_interfaces = my_local_interfaces;
-yield = host_find_interfaces();
-local_interfaces = NULL;
+DEBUG(D_interface) debug_printf("Unable to find local interface addresses "
+  "on this OS: returning loopback address(es)\n");
 return yield;
 }
 
