@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/routers/ipliteral.h,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -26,7 +28,8 @@ extern ipliteral_router_options_block ipliteral_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int ipliteral_router_entry(router_instance *, address_item *,
-  address_item **, address_item **, BOOL, char *);
+  struct passwd *, BOOL, address_item **, address_item **,
+  address_item **, address_item **);
 
 extern void ipliteral_router_init(router_instance *);
 
