@@ -1,23 +1,25 @@
+/* $Cambridge: exim/exim-src/src/routers/iplookup.h,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
 /* Private structure for the private options. */
 
 typedef struct {
-  int   service;
+  int   port;
   int   protocol;
   int   timeout;
-  char *protocol_name;
-  char *hosts;
-  char *query;
-  char *response_pattern;
-  char *reroute;
-  pcre *re_response_pattern;
+  uschar *protocol_name;
+  uschar *hosts;
+  uschar *query;
+  uschar *response_pattern;
+  uschar *reroute;
+  const pcre *re_response_pattern;
   BOOL  optional;
 } iplookup_router_options_block;
 
@@ -33,7 +35,8 @@ extern iplookup_router_options_block iplookup_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int iplookup_router_entry(router_instance *, address_item *,
-  address_item **, address_item **, BOOL, char *);
+  struct passwd *, BOOL, address_item **, address_item **,
+  address_item **, address_item **);
 
 extern void iplookup_router_init(router_instance *);
 
