@@ -1,21 +1,25 @@
+/* $Cambridge: exim/exim-src/src/routers/queryprogram.h,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
 /* Private structure for the private options. */
 
 typedef struct {
-  char *command;
+  uschar *command;
   int timeout;
-  uid_t uid;
-  gid_t gid;
-  BOOL uid_set;
-  BOOL gid_set;
-  char *current_directory;
+  uid_t cmd_uid;
+  gid_t cmd_gid;
+  BOOL cmd_uid_set;
+  BOOL cmd_gid_set;
+  uschar *current_directory;
+  uschar *expand_cmd_gid;
+  uschar *expand_cmd_uid;
 } queryprogram_router_options_block;
 
 /* Data for reading the private options. */
@@ -30,7 +34,8 @@ extern queryprogram_router_options_block queryprogram_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int queryprogram_router_entry(router_instance *, address_item *,
-  address_item **, address_item **, BOOL, char *);
+  struct passwd *, BOOL, address_item **, address_item **,
+  address_item **, address_item **);
 
 extern void queryprogram_router_init(router_instance *);
 
