@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/spool_in.c,v 1.8 2005/02/17 11:58:26 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for reading spool files. When compiling for a utility (eximon),
@@ -228,7 +230,7 @@ int n;
 int rcount = 0;
 long int uid, gid;
 BOOL inheader = FALSE;
-uschar originator[64];
+uschar *p;
 
 /* Reset all the global variables to their default values. However, there is
 one exception. DO NOT change the default value of dont_deliver, because it may
@@ -238,13 +240,17 @@ for (n = 0; n < ACL_C_MAX + ACL_M_MAX; n++) acl_var[n] = NULL;
 
 authenticated_id = NULL;
 authenticated_sender = NULL;
+allow_unqualified_recipient = FALSE;
+allow_unqualified_sender = FALSE;
 body_linecount = 0;
+body_zerocount = 0;
 deliver_firsttime = FALSE;
 deliver_freeze = FALSE;
 deliver_frozen_at = 0;
 deliver_manual_thaw = FALSE;
 /* dont_deliver must NOT be reset */
 header_list = header_last = NULL;
+host_lookup_deferred = FALSE;
 host_lookup_failed = FALSE;
 interface_address = NULL;
 interface_port = 0;
@@ -264,14 +270,24 @@ sender_host_authenticated = NULL;
 sender_ident = NULL;
 sender_local = FALSE;
 sender_set_untrusted = FALSE;
+smtp_active_hostname = primary_hostname;
 tree_nonrecipients = NULL;
 
+#ifdef EXPERIMENTAL_BRIGHTMAIL
+bmi_run = 0;
+bmi_verdicts = NULL;
+#endif
+
 #ifdef SUPPORT_TLS
 tls_certificate_verified = FALSE;
 tls_cipher = NULL;
 tls_peerdn = NULL;
 #endif
 
+#ifdef WITH_CONTENT_SCAN
+spam_score_int = NULL;
+#endif
+
 /* Generate the full name and open the file. If message_subdir is already
 set, just look in the given directory. Otherwise, look in both the split
 and unsplit directories, as for the data file above. */
@@ -309,9 +325,21 @@ messages for delivery delays that have been sent. */
 
 if (Ufgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 
-if (sscanf(CS big_buffer, "%s %ld %ld", originator, &uid, &gid) != 3)
-  goto SPOOL_FORMAT_ERROR;
-originator_login = string_copy(originator);
+p = big_buffer + Ustrlen(big_buffer);
+while (p > big_buffer && isspace(p[-1])) p--;
+*p = 0;
+if (!isdigit(p[-1])) goto SPOOL_FORMAT_ERROR;
+while (p > big_buffer && isdigit(p[-1])) p--;
+gid = Uatoi(p);
+if (p <= big_buffer || *(--p) != ' ') goto SPOOL_FORMAT_ERROR;
+*p = 0;
+if (!isdigit(p[-1])) goto SPOOL_FORMAT_ERROR;
+while (p > big_buffer && isdigit(p[-1])) p--;
+uid = Uatoi(p);
+if (p <= big_buffer || *(--p) != ' ') goto SPOOL_FORMAT_ERROR;
+*p = 0;
+
+originator_login = string_copy(big_buffer);
 originator_uid = (uid_t)uid;
 originator_gid = (gid_t)gid;
 
@@ -364,15 +392,31 @@ for (;;)
     local_error_message = TRUE;
   else if (Ustrncmp(big_buffer, "-local_scan ", 12) == 0)
     local_scan_data = string_copy(big_buffer + 12);
+#ifdef WITH_CONTENT_SCAN
+  else if (Ustrncmp(big_buffer, "-spam_score_int ", 16) == 0)
+    spam_score_int = string_copy(big_buffer + 16);
+#endif
+#ifdef EXPERIMENTAL_BRIGHTMAIL
+  else if (Ustrncmp(big_buffer, "-bmi_verdicts ", 14) == 0)
+    bmi_verdicts = string_copy(big_buffer + 14);
+#endif
+  else if (Ustrcmp(big_buffer, "-host_lookup_deferred") == 0)
+    host_lookup_deferred = TRUE;
   else if (Ustrcmp(big_buffer, "-host_lookup_failed") == 0)
     host_lookup_failed = TRUE;
   else if (Ustrncmp(big_buffer, "-body_linecount", 15) == 0)
     body_linecount = Uatoi(big_buffer + 15);
+  else if (Ustrncmp(big_buffer, "-body_zerocount", 15) == 0)
+    body_zerocount = Uatoi(big_buffer + 15);
   else if (Ustrncmp(big_buffer, "-frozen", 7) == 0)
     {
     deliver_freeze = TRUE;
     deliver_frozen_at = Uatoi(big_buffer + 7);
     }
+  else if (Ustrcmp(big_buffer, "-allow_unqualified_recipient") == 0)
+    allow_unqualified_recipient = TRUE;
+  else if (Ustrcmp(big_buffer, "-allow_unqualified_sender") == 0)
+    allow_unqualified_sender = TRUE;
   else if (Ustrcmp(big_buffer, "-deliver_firsttime") == 0)
     deliver_firsttime = TRUE;
   else if (Ustrcmp(big_buffer, "-manual_thaw") == 0)
@@ -409,6 +453,8 @@ for (;;)
     interface_address = string_copy(big_buffer + 19);
     }
 
+  else if (Ustrncmp(big_buffer, "-active_hostname", 16) == 0)
+    smtp_active_hostname = string_copy(big_buffer + 17);
   else if (Ustrncmp(big_buffer, "-host_auth", 10) == 0)
     sender_host_authenticated = string_copy(big_buffer + 11);
   else if (Ustrncmp(big_buffer, "-host_name", 10) == 0)
