@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/spool_out.c,v 1.6 2005/02/17 11:58:26 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for writing spool files, and moving them about. */
@@ -174,6 +176,9 @@ if (sender_host_address != NULL)
 if (interface_address != NULL)
   fprintf(f, "-interface_address %s.%d\n", interface_address, interface_port);
 
+if (smtp_active_hostname != primary_hostname)
+  fprintf(f, "-active_hostname %s\n", smtp_active_hostname);
+
 /* Likewise for any ident information; for local messages this is
 likely to be the same as originator_login, but will be different if
 the originator was root, forcing a different ident. */
@@ -198,20 +203,33 @@ for (i = 0; i < ACL_C_MAX + ACL_M_MAX; i++)
 
 fprintf(f, "-body_linecount %d\n", body_linecount);
 
+if (body_zerocount > 0) fprintf(f, "-body_zerocount %d\n", body_zerocount);
+
 if (authenticated_id != NULL)
   fprintf(f, "-auth_id %s\n", authenticated_id);
 if (authenticated_sender != NULL)
   fprintf(f, "-auth_sender %s\n", authenticated_sender);
+
+if (allow_unqualified_recipient) fprintf(f, "-allow_unqualified_recipient\n");
+if (allow_unqualified_sender) fprintf(f, "-allow_unqualified_sender\n");
 if (deliver_firsttime) fprintf(f, "-deliver_firsttime\n");
 if (deliver_freeze) fprintf(f, "-frozen %d\n", deliver_frozen_at);
 if (dont_deliver) fprintf(f, "-N\n");
+if (host_lookup_deferred) fprintf(f, "-host_lookup_deferred\n");
 if (host_lookup_failed) fprintf(f, "-host_lookup_failed\n");
 if (sender_local) fprintf(f, "-local\n");
 if (local_error_message) fprintf(f, "-localerror\n");
 if (local_scan_data != NULL) fprintf(f, "-local_scan %s\n", local_scan_data);
+#ifdef WITH_CONTENT_SCAN
+if (spam_score_int != NULL) fprintf(f,"-spam_score_int %s\n", spam_score_int);
+#endif
 if (deliver_manual_thaw) fprintf(f, "-manual_thaw\n");
 if (sender_set_untrusted) fprintf(f, "-sender_set_untrusted\n");
 
+#ifdef EXPERIMENTAL_BRIGHTMAIL
+if (bmi_verdicts != NULL) fprintf(f, "-bmi_verdicts %s\n", bmi_verdicts);
+#endif
+
 #ifdef SUPPORT_TLS
 if (tls_certificate_verified) fprintf(f, "-tls_certificate_verified\n");
 if (tls_cipher != NULL) fprintf(f, "-tls_cipher %s\n", tls_cipher);
