@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/store.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Exim gets and frees all its store through these functions. In the original
@@ -31,6 +33,10 @@ The following different types of store are recognized:
   after accepting a message when multiple messages are received by a single
   process. Resetting happens at some other times as well, usually fairly
   locally after some specific processing that needs working store.
+
+. There is a separate pool (POOL_SEARCH) that is used only for lookup storage.
+  This means it can be freed when search_tidyup() is called to close down all
+  the lookup caching.
 */
 
 
@@ -53,6 +59,11 @@ suitably aligned. */
 
 #define STORE_BLOCK_SIZE 8192
 
+/* store_reset() will not free the following block if the last used block has
+less than this much left in it. */
+
+#define STOREPOOL_MIN_SIZE 256
+
 /* Structure describing the beginning of each big block. */
 
 typedef struct storeblock {
@@ -74,10 +85,10 @@ even if the length is zero (which is used for getting a point to reset to). */
 
 int store_pool = POOL_PERM;
 
-static storeblock *chainbase[2] = { NULL, NULL };
-static storeblock *current_block[2] = { NULL, NULL };
-static void *next_yield[2] = { NULL, NULL };
-static int yield_length[2] = { -1, -1 };
+static storeblock *chainbase[3] = { NULL, NULL, NULL };
+static storeblock *current_block[3] = { NULL, NULL, NULL };
+static void *next_yield[3] = { NULL, NULL, NULL };
+static int yield_length[3] = { -1, -1, -1 };
 
 /* pool_malloc holds the amount of memory used by the store pools; this goes up
 and down as store is reset or released. nonpool_malloc is the total got by
@@ -91,7 +102,7 @@ static int nonpool_malloc = 0;
 NULL. This enables string_cat() to optimize its store handling for very long
 strings. That's why the variable is global. */
 
-void *store_last_get[2] = { NULL, NULL};
+void *store_last_get[3] = { NULL, NULL, NULL };
 
 
 
@@ -131,20 +142,37 @@ if (size > yield_length[store_pool])
   {
   int length = (size <= STORE_BLOCK_SIZE)? STORE_BLOCK_SIZE : size;
   int mlength = length + ALIGNED_SIZEOF_STOREBLOCK;
-  storeblock *newblock;
+  storeblock *newblock = NULL;
 
-  pool_malloc += mlength;           /* Used in pools */
-  nonpool_malloc -= mlength;        /* Exclude from overall total */
+  /* Sometimes store_reset() may leave a block for us; check if we can use it */
 
-  newblock = store_malloc(mlength);
-  newblock->next = NULL;
-  newblock->length = length;
+  if (current_block[store_pool] != NULL &&
+      current_block[store_pool]->next != NULL)
+    {
+    newblock = current_block[store_pool]->next;
+    if (newblock->length < length)
+      {
+      /* Give up on this block, because it's too small */
+      store_free(newblock);
+      newblock = NULL;
+      }
+    }
 
-  if (chainbase[store_pool] == NULL) chainbase[store_pool] = newblock;
-    else current_block[store_pool]->next = newblock;
+  /* If there was no free block, get a new one */
+
+  if (newblock == NULL)
+    {
+    pool_malloc += mlength;           /* Used in pools */
+    nonpool_malloc -= mlength;        /* Exclude from overall total */
+    newblock = store_malloc(mlength);
+    newblock->next = NULL;
+    newblock->length = length;
+    if (chainbase[store_pool] == NULL) chainbase[store_pool] = newblock;
+      else current_block[store_pool]->next = newblock;
+    }
 
   current_block[store_pool] = newblock;
-  yield_length[store_pool] = length;
+  yield_length[store_pool] = newblock->length;
   next_yield[store_pool] =
     (void *)((char *)current_block[store_pool] + ALIGNED_SIZEOF_STOREBLOCK);
   }
@@ -327,7 +355,14 @@ yield_length[store_pool] = newlength - (newlength % alignment);
 next_yield[store_pool] = (char *)ptr + (newlength % alignment);
 current_block[store_pool] = b;
 
-/* Free any subsequent blocks, if any */
+/* Free any subsequent block. Do NOT free the first successor, if our
+current block has less than 256 bytes left. This should prevent us from
+flapping memory. However, keep this block only when it has the default size. */
+
+if (yield_length[store_pool] < STOREPOOL_MIN_SIZE &&
+    b->next != NULL &&
+    b->next->length == STORE_BLOCK_SIZE)
+  b = b->next;
 
 bb = b->next;
 b->next = NULL;
