@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/string.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Miscellaneous string-handling functions. Some are not required for
@@ -334,7 +336,8 @@ return ss;
 *            Copy and save string                *
 *************************************************/
 
-/*
+/* This function assumes that memcpy() is faster than strcpy().
+
 Argument: string to copy
 Returns:  copy of string in new store
 */
@@ -342,8 +345,9 @@ Returns:  copy of string in new store
 uschar *
 string_copy(uschar *s)
 {
-uschar *ss = store_get(Ustrlen(s) + 1);
-Ustrcpy(ss, s);
+int len = Ustrlen(s) + 1;
+uschar *ss = store_get(len);
+memcpy(ss, s, len);
 return ss;
 }
 
@@ -353,7 +357,8 @@ return ss;
 *     Copy and save string in malloc'd store     *
 *************************************************/
 
-/*
+/* This function assumes that memcpy() is faster than strcpy().
+
 Argument: string to copy
 Returns:  copy of string in new store
 */
@@ -361,8 +366,9 @@ Returns:  copy of string in new store
 uschar *
 string_copy_malloc(uschar *s)
 {
-uschar *ss = store_malloc(Ustrlen(s) + 1);
-Ustrcpy(ss, s);
+int len = Ustrlen(s) + 1;
+uschar *ss = store_malloc(len);
+memcpy(ss, s, len);
 return ss;
 }
 
