@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/structs.h,v 1.4 2005/02/17 11:58:26 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -219,6 +221,9 @@ typedef struct router_instance {
   uschar *driver_name;            /* Must be first */
 
   uschar *address_data;           /* Arbitrary data */
+#ifdef EXPERIMENTAL_BRIGHTMAIL
+  uschar *bmi_rule;               /* Brightmail AntiSpam rule checking */
+#endif
   uschar *cannot_route_message;   /* Used when routing fails */
   uschar *condition;              /* General condition */
   uschar *current_directory;      /* For use during delivery */
@@ -247,6 +252,11 @@ typedef struct router_instance {
   uschar *transport_name;         /* Transport name */
 
   BOOL    address_test;           /* Use this router when testing addresses */
+#ifdef EXPERIMENTAL_BRIGHTMAIL
+  BOOL    bmi_deliver_alternate;  /* TRUE => BMI said that message should be delivered to alternate location */
+  BOOL    bmi_deliver_default;    /* TRUE => BMI said that message should be delivered to default location */
+  BOOL    bmi_dont_deliver;       /* TRUE => BMI said that message should not be delivered at all */
+#endif
   BOOL    expn;                   /* Use this router when processing EXPN */
   BOOL    caseful_local_part;     /* TRUE => don't lowercase */
   BOOL    check_local_user;       /* TRUE => check local user */
@@ -329,7 +339,8 @@ typedef struct lookup_info {
     uschar *,                     /* key or query */
     int,                          /* length of key or query */
     uschar **,                    /* for returning answer */
-    uschar **);                   /* for error message */
+    uschar **,                    /* for error message */
+    BOOL *);                      /* to request cache cleanup */
   void (*close)(                  /* close function */
     void *);                      /* handle */
   void (*tidy)(void);             /* tidy function */
@@ -603,6 +614,7 @@ typedef struct retry_config {
   uschar *pattern;
   int     basic_errno;
   int     more_errno;
+  uschar *senders;
   retry_rule *rules;
 } retry_config;
 
@@ -630,8 +642,7 @@ typedef struct search_cache {
   int search_type;                /* search type */
   tree_node *up;                  /* LRU up pointer */
   tree_node *down;                /* LRU down pointer */
-  uschar *key;                    /* the key of the last lookup */
-  uschar *data;                   /* the data of the last lookup */
+  tree_node *item_cache;          /* tree of cached results */
 } search_cache;
 
 /* Structure for holding a partially decoded DNS record; the name has been
