@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/tls-gnu.c,v 1.5 2005/02/17 11:58:26 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module provides TLS (aka SSL) support for Exim using the GnuTLS
@@ -57,10 +59,10 @@ static const int kx_priority[16] = {
   0 };
 
 static int default_cipher_priority[16] = {
-  GNUTLS_CIPHER_ARCFOUR_128,
+  GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_AES_128_CBC,
   GNUTLS_CIPHER_3DES_CBC,
-  GNUTLS_CIPHER_ARCFOUR_40,
+  GNUTLS_CIPHER_ARCFOUR_128,
   0 };
 
 static int cipher_priority[16];
@@ -459,7 +461,12 @@ if (cert_expanded != NULL)
     cert_expanded, key_expanded);
   rc = gnutls_certificate_set_x509_key_file(x509_cred, CS cert_expanded,
     CS key_expanded, GNUTLS_X509_FMT_PEM);
-  if (rc < 0) return tls_error(US"cert/key setup", host, rc);
+  if (rc < 0)
+    {
+    uschar *msg = string_sprintf("cert/key setup: cert=%s key=%s",
+      cert_expanded, key_expanded);
+    return tls_error(msg, host, rc);
+    }
   }
 
 /* A certificate is mandatory in a server, but not in a client */
@@ -923,6 +930,7 @@ int rc;
 
 DEBUG(D_tls) debug_printf("initializing GnuTLS as a client\n");
 
+client_host = host;
 verify_requirement = (verify_certs == NULL)? VERIFY_NONE : VERIFY_REQUIRED;
 rc = tls_init(host, certificate, privatekey, verify_certs, verify_crl);
 if (rc != OK) return rc;
@@ -1008,6 +1016,7 @@ static void
 record_io_error(int ec, uschar *when, uschar *text)
 {
 uschar *additional = US"";
+
 if (ec == GNUTLS_E_FATAL_ALERT_RECEIVED)
   additional = string_sprintf(": %s",
     gnutls_alert_get_name(gnutls_alert_get(tls_session)));
@@ -1020,7 +1029,7 @@ if (initialized == INITIALIZED_SERVER)
 else
   log_write(0, LOG_MAIN, "TLS %s error on connection to %s [%s]: %s%s", when,
     client_host->name, client_host->address,
-    (text == 0)? text : US gnutls_strerror(ec), additional);
+    (ec == 0)? text : US gnutls_strerror(ec), additional);
 }
 
 
@@ -1117,7 +1126,6 @@ DEBUG(D_tls) debug_printf("Calling gnutls_record_recv(%lx, %lx, %u)\n",
 
 inbytes = gnutls_record_recv(tls_session, CS buff, len);
 if (inbytes > 0) return inbytes;
-
 if (inbytes == 0)
   {
   DEBUG(D_tls) debug_printf("Got TLS_EOF\n");
