@@ -1,35 +1,46 @@
+/* $Cambridge: exim/exim-src/src/transports/appendfile.h,v 1.2 2005/01/04 10:00:45 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2001 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
-  char *filename;
-  char *dirname;
-  char *prefix;
-  char *suffix;
-  char *create_file_string;
-  char *quota;
-  char *quota_size_regex;
-  char *quota_warn_threshold;
-  char *maildir_tag;
-  char *mailstore_prefix;
-  char *mailstore_suffix;
-  char *check_string;
-  char *escape_string;
-  char *file_format;
+  uschar *filename;
+  uschar *dirname;
+  uschar *dirfilename;
+  uschar *message_prefix;
+  uschar *message_suffix;
+  uschar *create_file_string;
+  uschar *quota;
+  uschar *quota_directory;
+  uschar *quota_filecount;
+  uschar *quota_size_regex;
+  uschar *quota_warn_threshold;
+  uschar *mailbox_size_string;
+  uschar *mailbox_filecount_string;
+  uschar *maildir_dir_regex;
+  uschar *maildir_tag;
+  uschar *mailstore_prefix;
+  uschar *mailstore_suffix;
+  uschar *check_string;
+  uschar *escape_string;
+  uschar *file_format;
+  int   mailbox_size_value;
+  int   mailbox_filecount_value;
   int   quota_value;
+  int   quota_filecount_value;
   int   quota_warn_threshold_value;
-  int   quota_filecount;
   int   mode;
   int   dirmode;
   int   lockfile_mode;
   int   lockfile_timeout;
   int   lock_fcntl_timeout;
+  int   lock_flock_timeout;
   int   lock_retries;
   int   lock_interval;
   int   maildir_retries;
@@ -41,19 +52,20 @@ typedef struct {
   BOOL  check_owner;
   BOOL  create_directory;
   BOOL  notify_comsat;
-  BOOL  require_lockfile;
   BOOL  use_lockfile;
   BOOL  set_use_lockfile;
   BOOL  use_fcntl;
   BOOL  set_use_fcntl;
+  BOOL  use_flock;
+  BOOL  set_use_flock;
   BOOL  use_mbx_lock;
   BOOL  set_use_mbx_lock;
+  BOOL  use_bsmtp;
   BOOL  use_crlf;
-  BOOL  from_hack;
-  BOOL  set_from_hack;
   BOOL  file_must_exist;
   BOOL  mode_fail_narrower;
   BOOL  maildir_format;
+  BOOL  maildir_use_size_file;
   BOOL  mailstore_format;
   BOOL  mbx_format;
   BOOL  quota_warn_threshold_is_percent;
@@ -78,4 +90,8 @@ extern appendfile_transport_options_block appendfile_transport_option_defaults;
 extern BOOL appendfile_transport_entry(transport_instance *, address_item *);
 extern void appendfile_transport_init(transport_instance *);
 
+/* Function that is shared with tf_maildir.c */
+
+extern int  check_dir_size(uschar *, int *, const pcre *);
+
 /* End of transports/appendfile.h */
