@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/transports/autoreply.c,v 1.2 2005/01/04 10:00:45 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -18,54 +20,42 @@ software for alternative value types. Some options are publicly visible and so
 are stored in the driver instance block. These are flagged with opt_public. */
 
 optionlist autoreply_transport_options[] = {
-  { "*expand_group",     opt_stringptr | opt_hidden | opt_public,
-      (void *)(offsetof(transport_instance, expand_gid)) },
-  { "*expand_user",      opt_stringptr | opt_hidden | opt_public,
-      (void *)(offsetof(transport_instance, expand_uid)) },
-  { "*set_group",         opt_bool | opt_hidden | opt_public,
-      (void *)(offsetof(transport_instance, gid_set)) },
-  { "*set_user",          opt_bool | opt_hidden | opt_public,
-      (void *)(offsetof(transport_instance, uid_set)) },
   { "bcc",               opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, bcc)) },
+      (void *)offsetof(autoreply_transport_options_block, bcc) },
   { "cc",                opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, cc)) },
+      (void *)offsetof(autoreply_transport_options_block, cc) },
   { "file",              opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, file)) },
+      (void *)offsetof(autoreply_transport_options_block, file) },
   { "file_expand",     opt_bool,
-      (void *)(offsetof(autoreply_transport_options_block, file_expand)) },
+      (void *)offsetof(autoreply_transport_options_block, file_expand) },
   { "file_optional",     opt_bool,
-      (void *)(offsetof(autoreply_transport_options_block, file_optional)) },
+      (void *)offsetof(autoreply_transport_options_block, file_optional) },
   { "from",              opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, from)) },
-  { "group",             opt_expand_gid | opt_public,
-      (void *)(offsetof(transport_instance, gid)) },
+      (void *)offsetof(autoreply_transport_options_block, from) },
   { "headers",           opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, headers)) },
-  { "initgroups",        opt_bool | opt_public,
-      (void *)(offsetof(transport_instance, initgroups)) },
+      (void *)offsetof(autoreply_transport_options_block, headers) },
   { "log",               opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, logfile)) },
+      (void *)offsetof(autoreply_transport_options_block, logfile) },
   { "mode",              opt_octint,
-      (void *)(offsetof(autoreply_transport_options_block, mode)) },
+      (void *)offsetof(autoreply_transport_options_block, mode) },
+  { "never_mail",        opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, never_mail) },
   { "once",              opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, oncelog)) },
+      (void *)offsetof(autoreply_transport_options_block, oncelog) },
   { "once_file_size",    opt_int,
-      (void *)(offsetof(autoreply_transport_options_block, once_file_size)) },
-  { "once_repeat",       opt_time,
-      (void *)(offsetof(autoreply_transport_options_block, once_repeat)) },
+      (void *)offsetof(autoreply_transport_options_block, once_file_size) },
+  { "once_repeat",       opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, once_repeat) },
   { "reply_to",          opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, reply_to)) },
+      (void *)offsetof(autoreply_transport_options_block, reply_to) },
   { "return_message",    opt_bool,
-      (void *)(offsetof(autoreply_transport_options_block, return_message)) },
+      (void *)offsetof(autoreply_transport_options_block, return_message) },
   { "subject",           opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, subject)) },
+      (void *)offsetof(autoreply_transport_options_block, subject) },
   { "text",              opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, text)) },
+      (void *)offsetof(autoreply_transport_options_block, text) },
   { "to",                opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, to)) },
-  { "user",              opt_expand_uid | opt_public,
-      (void *)(offsetof(transport_instance, uid)) },
+      (void *)offsetof(autoreply_transport_options_block, to) },
 };
 
 /* Size of the options list. An extern variable has to be used so that its
@@ -88,9 +78,10 @@ autoreply_transport_options_block autoreply_transport_option_defaults = {
   NULL,           /* file */
   NULL,           /* logfile */
   NULL,           /* oncelog */
+  NULL,           /* once_repeat */
+  NULL,           /* never_mail */
   0600,           /* mode */
   0,              /* once_file_size */
-  0,              /* once_repeat */
   FALSE,          /* file_expand */
   FALSE,          /* file_optional */
   FALSE           /* return message */
@@ -98,6 +89,12 @@ autoreply_transport_options_block autoreply_transport_option_defaults = {
 
 
 
+/* Type of text for the checkexpand() function */
+
+enum { cke_text, cke_hdr, cke_file };
+
+
+
 /*************************************************
 *          Initialization entry point            *
 *************************************************/
@@ -136,17 +133,20 @@ Arguments:
    s         string to expand
    addr      address that is being worked on
    name      transport name, for error text
-   text      if TRUE, don't check for non-printing characters
+   type      type, for checking content:
+               cke_text => no check
+               cke_hdr  => header, allow \n + whitespace
+               cke_file => file name, no non-printers allowed
 
 Returns:     expanded string if expansion succeeds;
              NULL otherwise
 */
 
-static char *
-checkexpand(char *s, address_item *addr, char *name, BOOL text)
+static uschar *
+checkexpand(uschar *s, address_item *addr, uschar *name, int type)
 {
-char *t;
-char *ss = expand_string(s);
+uschar *t;
+uschar *ss = expand_string(s);
 
 if (ss == NULL)
   {
@@ -156,17 +156,16 @@ if (ss == NULL)
   return NULL;
   }
 
-if (!text) for (t = ss; *t != 0; t++)
+if (type != cke_text) for (t = ss; *t != 0; t++)
   {
-  int c = (uschar)*t;
-  if (!mac_isprint(c) && c != '\t')
-    {
-    s = string_printing(s);
-    addr->transport_return = FAIL;
-    addr->message = string_sprintf("Expansion of \"%s\" in %s transport "
-      "contains non-printing character %d", s, name, c);
-    return NULL;
-    }
+  int c = *t;
+  if (mac_isprint(c)) continue;
+  if (type == cke_hdr && c == '\n' && (t[1] == ' ' || t[1] == '\t')) continue;
+  s = string_printing(s);
+  addr->transport_return = FAIL;
+  addr->message = string_sprintf("Expansion of \"%s\" in %s transport "
+    "contains non-printing character %d", s, name, c);
+  return NULL;
   }
 
 return ss;
@@ -175,6 +174,85 @@ return ss;
 
 
 
+/*************************************************
+*          Check a header line for never_mail    *
+*************************************************/
+
+/* This is called to check to, cc, and bcc for addresses in the never_mail
+list. Any that are found are removed.
+
+Arguments:
+  listptr     points to the list of addresses
+  never_mail  an address list, already expanded
+
+Returns:      nothing
+*/
+
+static void
+check_never_mail(uschar **listptr, uschar *never_mail)
+{
+uschar *s = *listptr;
+
+while (*s != 0)
+  {
+  uschar *error, *next;
+  uschar *e = parse_find_address_end(s, FALSE);
+  int terminator = *e;
+  int start, end, domain, rc;
+
+  /* Temporarily terminate the string at the address end while extracting
+  the operative address within. */
+
+  *e = 0;
+  next = parse_extract_address(s, &error, &start, &end, &domain, FALSE);
+  *e = terminator;
+
+  /* If there is some kind of syntax error, just give up on this header
+  line. */
+
+  if (next == NULL) break;
+
+  /* See if the address is on the never_mail list */
+
+  rc = match_address_list(next,         /* address to check */
+                          TRUE,         /* start caseless */
+                          FALSE,        /* don't expand the list */
+                          &never_mail,  /* the list */
+                          NULL,         /* no caching */
+                          -1,           /* no expand setup */
+                          0,            /* separator from list */
+                          NULL);        /* no lookup value return */
+
+  if (rc == OK)                         /* Remove this address */
+    {
+    DEBUG(D_transport)
+      debug_printf("discarding recipient %s (matched never_mail)\n", next);
+    if (terminator == ',') e++;
+    memmove(s, e, Ustrlen(e) + 1);
+    }
+  else                                  /* Skip over this address */
+    {
+    s = e;
+    if (terminator == ',') s++;
+    }
+  }
+
+/* Check to see if we removed the last address, leaving a terminating comma
+that needs to be removed */
+
+s = *listptr + Ustrlen(*listptr);
+while (s > *listptr && (isspace(s[-1]) || s[-1] == ',')) s--;
+*s = 0;
+
+/* Check to see if there any addresses left; if not, set NULL */
+
+s = *listptr;
+while (s != 0 && isspace(*s)) s++;
+if (*s == 0) *listptr = NULL;
+}
+
+
+
 /*************************************************
 *              Main entry point                  *
 *************************************************/
@@ -195,20 +273,20 @@ int cache_size = 0;
 int add_size = 0;
 EXIM_DB *dbm_file = NULL;
 BOOL file_expand, return_message;
-char *from, *reply_to, *to, *cc, *bcc, *subject, *headers, *text, *file;
-char *logfile, *oncelog;
-char *cache_buff = NULL;
-char *cache_time = NULL;
+uschar *from, *reply_to, *to, *cc, *bcc, *subject, *headers, *text, *file;
+uschar *logfile, *oncelog;
+uschar *cache_buff = NULL;
+uschar *cache_time = NULL;
 header_line *h;
 time_t now = time(NULL);
-time_t once_repeat;
+time_t once_repeat_sec = 0;
 FILE *f;
 FILE *ff = NULL;
 
 autoreply_transport_options_block *ob =
   (autoreply_transport_options_block *)(tblock->options_block);
 
-DEBUG(2) debug_printf("%s transport entered\n", tblock->name);
+DEBUG(D_transport) debug_printf("%s transport entered\n", tblock->name);
 
 /* Set up for the good case */
 
@@ -217,12 +295,12 @@ addr->basic_errno = 0;
 
 /* If the address is pointing to a reply block, then take all the data
 from that block. It has typically been set up by a mail filter processing
-director. Otherwise, the data must be supplied by this transport, and
+router. Otherwise, the data must be supplied by this transport, and
 it has to be expanded here. */
 
 if (addr->reply != NULL)
   {
-  DEBUG(9) debug_printf("taking data from address\n");
+  DEBUG(D_transport) debug_printf("taking data from address\n");
   from = addr->reply->from;
   reply_to = addr->reply->reply_to;
   to = addr->reply->to;
@@ -234,16 +312,16 @@ if (addr->reply != NULL)
   file = addr->reply->file;
   logfile = addr->reply->logfile;
   oncelog = addr->reply->oncelog;
-  once_repeat = addr->reply->once_repeat;
+  once_repeat_sec = addr->reply->once_repeat;
   file_expand = addr->reply->file_expand;
-  expand_forbid_exists = addr->reply->forbid_exists;
-  expand_forbid_lookup = addr->reply->forbid_lookup;
-  expand_forbid_perl = addr->reply->forbid_perl;
+  expand_forbid = addr->reply->expand_forbid;
   return_message = addr->reply->return_message;
   }
 else
   {
-  DEBUG(9) debug_printf("taking data from transport\n");
+  uschar *oncerepeat = ob->once_repeat;
+
+  DEBUG(D_transport) debug_printf("taking data from transport\n");
   from = ob->from;
   reply_to = ob->reply_to;
   to = ob->to;
@@ -255,42 +333,82 @@ else
   file = ob->file;
   logfile = ob->logfile;
   oncelog = ob->oncelog;
-  once_repeat = ob->once_repeat;
   file_expand = ob->file_expand;
   return_message = ob->return_message;
 
   if ((from  != NULL &&
-        (from = checkexpand(from, addr, tblock->name, FALSE)) == NULL) ||
+        (from = checkexpand(from, addr, tblock->name, cke_hdr)) == NULL) ||
       (reply_to    != NULL &&
-        (reply_to = checkexpand(reply_to, addr, tblock->name, FALSE)) == NULL) ||
+        (reply_to = checkexpand(reply_to, addr, tblock->name, cke_hdr)) == NULL) ||
       (to    != NULL &&
-        (to = checkexpand(to, addr, tblock->name, FALSE)) == NULL) ||
+        (to = checkexpand(to, addr, tblock->name, cke_hdr)) == NULL) ||
       (cc    != NULL &&
-        (cc = checkexpand(cc, addr, tblock->name, FALSE)) == NULL) ||
+        (cc = checkexpand(cc, addr, tblock->name, cke_hdr)) == NULL) ||
       (bcc   != NULL &&
-        (bcc = checkexpand(bcc, addr, tblock->name, FALSE)) == NULL) ||
+        (bcc = checkexpand(bcc, addr, tblock->name, cke_hdr)) == NULL) ||
       (subject   != NULL &&
-        (subject = checkexpand(subject, addr, tblock->name, FALSE)) == NULL) ||
+        (subject = checkexpand(subject, addr, tblock->name, cke_hdr)) == NULL) ||
       (headers != NULL &&
-        (headers = checkexpand(headers, addr, tblock->name, TRUE)) == NULL) ||
+        (headers = checkexpand(headers, addr, tblock->name, cke_text)) == NULL) ||
       (text  != NULL &&
-        (text = checkexpand(text, addr, tblock->name, TRUE)) == NULL) ||
+        (text = checkexpand(text, addr, tblock->name, cke_text)) == NULL) ||
       (file  != NULL &&
-        (file = checkexpand(file, addr, tblock->name, FALSE)) == NULL) ||
+        (file = checkexpand(file, addr, tblock->name, cke_file)) == NULL) ||
       (logfile != NULL &&
-        (logfile = checkexpand(logfile, addr, tblock->name, FALSE)) == NULL) ||
+        (logfile = checkexpand(logfile, addr, tblock->name, cke_file)) == NULL) ||
       (oncelog != NULL &&
-        (oncelog = checkexpand(oncelog, addr, tblock->name, FALSE)) == NULL))
+        (oncelog = checkexpand(oncelog, addr, tblock->name, cke_file)) == NULL) ||
+      (oncerepeat != NULL &&
+        (oncerepeat = checkexpand(oncerepeat, addr, tblock->name, cke_file)) == NULL))
     return FALSE;
+
+  if (oncerepeat != NULL)
+    {
+    once_repeat_sec = readconf_readtime(oncerepeat, 0, FALSE);
+    if (once_repeat_sec < 0)
+      {
+      addr->transport_return = FAIL;
+      addr->message = string_sprintf("Invalid time value \"%s\" for "
+        "\"once_repeat\" in %s transport", oncerepeat, tblock->name);
+      return FALSE;
+      }
+    }
   }
 
+/* If the never_mail option is set, we have to scan all the recipients and
+remove those that match. */
+
+if (ob->never_mail != NULL)
+  {
+  uschar *never_mail = expand_string(ob->never_mail);
+
+  if (never_mail == NULL)
+    {
+    addr->transport_return = FAIL;
+    addr->message = string_sprintf("Failed to expand \"%s\" for "
+      "\"never_mail\" in %s transport", ob->never_mail, tblock->name);
+    return FALSE;
+    }
+
+  if (to != NULL) check_never_mail(&to, never_mail);
+  if (cc != NULL) check_never_mail(&cc, never_mail);
+  if (bcc != NULL) check_never_mail(&bcc, never_mail);
+
+  if (to == NULL && cc == NULL && bcc == NULL)
+    {
+    DEBUG(D_transport)
+      debug_printf("*** all recipients removed by never_mail\n");
+    return OK;
+    }
+  }
 
 /* If the -N option is set, can't do any more. */
 
 if (dont_deliver)
   {
-  debug_printf("*** delivery by %s transport bypassed by -N option\n",
-    tblock->name);
+  DEBUG(D_transport)
+    debug_printf("*** delivery by %s transport bypassed by -N option\n",
+      tblock->name);
   return FALSE;
   }
 
@@ -310,9 +428,9 @@ if (oncelog != NULL && to != NULL)
 
   if (ob->once_file_size > 0)
     {
-    char *p;
+    uschar *p;
     struct stat statbuf;
-    cache_fd = open(oncelog, O_CREAT|O_RDWR, ob->mode);
+    cache_fd = Uopen(oncelog, O_CREAT|O_RDWR, ob->mode);
 
     if (cache_fd < 0 || fstat(cache_fd, &statbuf) != 0)
       {
@@ -329,20 +447,18 @@ if (oncelog != NULL && to != NULL)
     need to do that. */
 
     cache_size = statbuf.st_size;
-    add_size = sizeof(time_t) + (int)strlen(to) + 1;
-    store_pool = POOL_TEMP;
+    add_size = sizeof(time_t) + Ustrlen(to) + 1;
     cache_buff = store_get(cache_size + add_size);
-    store_pool = POOL_MAIN;
 
     if (read(cache_fd, cache_buff, cache_size) != cache_size)
       {
       addr->transport_return = DEFER;
       addr->basic_errno = errno;
-      addr->message = "error while reading \"once\" file";
+      addr->message = US"error while reading \"once\" file";
       goto END_OFF;
       }
 
-    DEBUG(2) debug_printf("%d bytes read from %s\n", cache_size, oncelog);
+    DEBUG(D_transport) debug_printf("%d bytes read from %s\n", cache_size, oncelog);
 
     /* Scan the data for this recipient. Each entry in the file starts with
     a time_t sized time value, followed by the address, followed by a binary
@@ -352,9 +468,9 @@ if (oncelog != NULL && to != NULL)
     p = cache_buff;
     while (p < cache_buff + cache_size)
       {
-      char *s = p + sizeof(time_t);
-      char *nextp = s + (int)strlen(s) + 1;
-      if (strcmp(to, s) == 0)
+      uschar *s = p + sizeof(time_t);
+      uschar *nextp = s + Ustrlen(s) + 1;
+      if (Ustrcmp(to, s) == 0)
         {
         memcpy(&then, p, sizeof(time_t));
         cache_time = p;
@@ -381,8 +497,8 @@ if (oncelog != NULL && to != NULL)
 
     EXIM_DATUM_INIT(key_datum);        /* Some DBM libraries need datums */
     EXIM_DATUM_INIT(result_datum);     /* to be cleared */
-    EXIM_DATUM_DATA(key_datum) = to;
-    EXIM_DATUM_SIZE(key_datum) = (int)strlen(to) + 1;
+    EXIM_DATUM_DATA(key_datum) = CS to;
+    EXIM_DATUM_SIZE(key_datum) = Ustrlen(to) + 1;
 
     if (EXIM_DBGET(dbm_file, key_datum, result_datum))
       {
@@ -404,15 +520,15 @@ if (oncelog != NULL && to != NULL)
   /* Either "then" is set zero, if no message has yet been sent, or it
   is set to the time of the last sending. */
 
-  if (then != 0 && (once_repeat <= 0 || now - then < once_repeat))
+  if (then != 0 && (once_repeat_sec <= 0 || now - then < once_repeat_sec))
     {
-    DEBUG(9) debug_printf("message previously sent to %s%s\n", to,
-      (once_repeat > 0)? " and repeat time not reached" : "");
-    log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
+    DEBUG(D_transport) debug_printf("message previously sent to %s%s\n", to,
+      (once_repeat_sec > 0)? " and repeat time not reached" : "");
+    log_fd = Uopen(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
     if (log_fd >= 0)
       {
-      char *ptr = log_buffer;
-      sprintf(ptr, "%s\n  previously sent to %.200s\n", tod_stamp(tod_log), to);
+      uschar *ptr = log_buffer;
+      sprintf(CS ptr, "%s\n  previously sent to %.200s\n", tod_stamp(tod_log), to);
       while(*ptr) ptr++;
       write(log_fd, log_buffer, ptr - log_buffer);
       close(log_fd);
@@ -420,7 +536,7 @@ if (oncelog != NULL && to != NULL)
     goto END_OFF;
     }
 
-  DEBUG(9) debug_printf("%s %s\n", (then <= 0)?
+  DEBUG(D_transport) debug_printf("%s %s\n", (then <= 0)?
     "no previous message sent to" : "repeat time reached for", to);
   }
 
@@ -428,7 +544,7 @@ if (oncelog != NULL && to != NULL)
 
 if (file != NULL)
   {
-  ff = fopen(file, "r");
+  ff = Ufopen(file, "rb");
   if (ff == NULL && !ob->file_optional)
     {
     addr->transport_return = DEFER;
@@ -449,7 +565,7 @@ if (pid < 0)
   addr->transport_return = DEFER;
   addr->message = string_sprintf("Failed to create child process to send "
     "message from %s transport: %s", tblock->name, strerror(errno));
-  DEBUG(2) debug_printf("%s\n", addr->message);
+  DEBUG(D_transport) debug_printf("%s\n", addr->message);
   return FALSE;
   }
 
@@ -457,7 +573,7 @@ if (pid < 0)
 as the -t option is used. The "headers" stuff *must* be last in case there
 are newlines in it which might, if placed earlier, screw up other headers. */
 
-f = fdopen(fd, "w");
+f = fdopen(fd, "wb");
 
 if (from != NULL) fprintf(f, "From: %s\n", from);
 if (reply_to != NULL) fprintf(f, "Reply-To: %s\n", reply_to);
@@ -474,11 +590,15 @@ for (h = header_list; h != NULL; h = h->next)
 
 if (h != NULL)
   {
-  char *s = strchr(h->text, ':') + 1;
-  while (isspace((uschar)*s)) s++;
+  uschar *s = Ustrchr(h->text, ':') + 1;
+  while (isspace(*s)) s++;
   fprintf(f, "In-Reply-To: %s", s);
   }
 
+/* Add an Auto-Submitted: header */
+
+fprintf(f, "Auto-Submitted: auto-replied\n");
+
 /* Add any specially requested headers */
 
 if (headers != NULL) fprintf(f, "%s\n", headers);
@@ -486,26 +606,26 @@ fprintf(f, "\n");
 
 if (text != NULL)
   {
-  fprintf(f, "%s", text);
-  if (text[(int)strlen(text)-1] != '\n') fprintf(f, "\n");
+  fprintf(f, "%s", CS text);
+  if (text[Ustrlen(text)-1] != '\n') fprintf(f, "\n");
   }
 
 if (ff != NULL)
   {
-  while (fgets(big_buffer, big_buffer_size, ff) != NULL)
+  while (Ufgets(big_buffer, big_buffer_size, ff) != NULL)
     {
     if (file_expand)
       {
-      char *s = expand_string(big_buffer);
-      DEBUG(9)
+      uschar *s = expand_string(big_buffer);
+      DEBUG(D_transport)
         {
         if (s == NULL)
           debug_printf("error while expanding line from file:\n  %s\n  %s\n",
             big_buffer, expand_string_message);
         }
-      fprintf(f, "%s", (s == NULL)? big_buffer : s);
+      fprintf(f, "%s", (s == NULL)? CS big_buffer : CS s);
       }
-    else fprintf(f, "%s", big_buffer);
+    else fprintf(f, "%s", CS big_buffer);
     }
   }
 
@@ -514,10 +634,10 @@ limit. */
 
 if (return_message)
   {
-  if (return_size_limit > 0)
+  if (bounce_return_size_limit > 0)
     {
     struct stat statbuf;
-    int max = (return_size_limit/DELIVER_IN_BUFFER_SIZE + 1) *
+    int max = (bounce_return_size_limit/DELIVER_IN_BUFFER_SIZE + 1) *
       DELIVER_IN_BUFFER_SIZE;
     if (fstat(deliver_datafile, &statbuf) == 0 && statbuf.st_size > max)
       {
@@ -534,13 +654,14 @@ if (return_message)
 "------ This is a copy of the message, including all the headers. ------\n\n");
 
   fflush(f);
+  transport_count = 0;
   transport_write_message(addr, fileno(f),
     (tblock->body_only? topt_no_headers : 0) |
     (tblock->headers_only? topt_no_body : 0) |
     (tblock->return_path_add? topt_add_return_path : 0) |
     (tblock->delivery_date_add? topt_add_delivery_date : 0) |
     (tblock->envelope_to_add? topt_add_envelope_to : 0),
-    return_size_limit, tblock->add_headers, tblock->remove_headers,
+    bounce_return_size_limit, tblock->add_headers, tblock->remove_headers,
     NULL, NULL, tblock->rewrite_rules, tblock->rewrite_existflags);
   }
 
@@ -563,7 +684,7 @@ between multiple simultaneous deliveries. */
 
 if (cache_fd >= 0)
   {
-  char *from = cache_buff;
+  uschar *from = cache_buff;
   int size = cache_size;
   (void)lseek(cache_fd, 0, SEEK_SET);
 
@@ -575,7 +696,7 @@ if (cache_fd >= 0)
 
     if (cache_size > 0 && size > ob->once_file_size)
       {
-      from += sizeof(time_t) + (int)strlen(from + sizeof(time_t)) + 1;
+      from += sizeof(time_t) + Ustrlen(from + sizeof(time_t)) + 1;
       size -= (from - cache_buff);
       }
     }
@@ -591,14 +712,14 @@ else if (dbm_file != NULL)
   EXIM_DATUM key_datum, value_datum;
   EXIM_DATUM_INIT(key_datum);          /* Some DBM libraries need to have */
   EXIM_DATUM_INIT(value_datum);        /* cleared datums. */
-  EXIM_DATUM_DATA(key_datum) = to;
-  EXIM_DATUM_SIZE(key_datum) = (int)strlen(to) + 1;
+  EXIM_DATUM_DATA(key_datum) = CS to;
+  EXIM_DATUM_SIZE(key_datum) = Ustrlen(to) + 1;
 
   /* Many OS define the datum value, sensibly, as a void *. However, there
   are some which still have char *. By casting this address to a char * we
   can avoid warning messages from the char * systems. */
 
-  EXIM_DATUM_DATA(value_datum) = (char *)(&now);
+  EXIM_DATUM_DATA(value_datum) = CS (&now);
   EXIM_DATUM_SIZE(value_datum) = (int)sizeof(time_t);
   EXIM_DBPUT(dbm_file, key_datum, value_datum);
   }
@@ -611,7 +732,7 @@ if (rc != 0)
   {
   if (rc == EXIT_NORECIPIENTS)
     {
-    DEBUG(1) debug_printf("%s transport: message contained no recipients\n",
+    DEBUG(D_any) debug_printf("%s transport: message contained no recipients\n",
       tblock->name);
     }
   else
@@ -634,12 +755,12 @@ writing. */
 
 if (logfile != NULL)
   {
-  int log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
+  int log_fd = Uopen(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
   if (log_fd >= 0)
     {
-    char *ptr = log_buffer;
-    DEBUG(9) debug_printf("logging message details\n");
-    sprintf(ptr, "%s\n", tod_stamp(tod_log));
+    uschar *ptr = log_buffer;
+    DEBUG(D_transport) debug_printf("logging message details\n");
+    sprintf(CS ptr, "%s\n", tod_stamp(tod_log));
     while(*ptr) ptr++;
     if (from != NULL)
       {
@@ -680,23 +801,15 @@ if (logfile != NULL)
     write(log_fd, log_buffer, ptr - log_buffer);
     close(log_fd);
     }
-  else DEBUG(2) debug_printf("Failed to open log file %s for %s "
+  else DEBUG(D_transport) debug_printf("Failed to open log file %s for %s "
     "transport: %s\n", logfile, tblock->name, strerror(errno));
   }
 
-
 END_OFF:
-if (cache_buff != NULL)
-  {
-  store_pool = POOL_TEMP;
-  store_reset(cache_buff);
-  store_pool = POOL_MAIN;
-  }
-
 if (dbm_file != NULL) EXIM_DBCLOSE(dbm_file);
 if (cache_fd > 0) close(cache_fd);
 
-DEBUG(2) debug_printf("%s transport succeeded\n", tblock->name);
+DEBUG(D_transport) debug_printf("%s transport succeeded\n", tblock->name);
 
 return FALSE;
 }
