@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/transports/autoreply.h,v 1.2 2005/01/04 10:00:45 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -20,6 +22,7 @@ typedef struct {
   uschar *logfile;
   uschar *oncelog;
   uschar *once_repeat;
+  uschar *never_mail;
   int   mode;
   off_t once_file_size;
   BOOL  file_expand;
