@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/transports/lmtp.c,v 1.4 2005/02/17 11:58:27 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -20,30 +22,16 @@ instance block so as to be publicly visible; these are flagged with opt_public.
 */
 
 optionlist lmtp_transport_options[] = {
-  { "*expand_group",     opt_stringptr | opt_hidden | opt_public,
-      (void *)(offsetof(transport_instance, expand_gid)) },
-  { "*expand_user",      opt_stringptr | opt_hidden | opt_public,
-      (void *)(offsetof(transport_instance, expand_uid)) },
-  { "*set_group",         opt_bool | opt_hidden | opt_public,
-      (void *)(offsetof(transport_instance, gid_set)) },
-  { "*set_user",          opt_bool | opt_hidden | opt_public,
-      (void *)(offsetof(transport_instance, uid_set)) },
-  { "batch",             opt_local_batch | opt_public,
-      (void *)(offsetof(transport_instance, local_batch)) },
+  { "batch_id",          opt_stringptr | opt_public,
+      (void *)offsetof(transport_instance, batch_id) },
   { "batch_max",         opt_int | opt_public,
-      (void *)(offsetof(transport_instance, batch_max)) },
+      (void *)offsetof(transport_instance, batch_max) },
   { "command",           opt_stringptr,
-      (void *)(offsetof(lmtp_transport_options_block, cmd)) },
-  { "group",             opt_expand_gid | opt_public,
-      (void *)(offsetof(transport_instance, gid)) },
-  { "initgroups",        opt_bool | opt_public,
-      (void *)(offsetof(transport_instance, initgroups)) },
-  { "retry_use_local_part", opt_bool | opt_public,
-      (void *)offsetof(transport_instance, retry_use_local_part) },
+      (void *)offsetof(lmtp_transport_options_block, cmd) },
+  { "socket",            opt_stringptr,
+      (void *)offsetof(lmtp_transport_options_block, skt) },
   { "timeout",           opt_time,
-      (void *)(offsetof(lmtp_transport_options_block, timeout)) },
-  { "user",              opt_expand_uid | opt_public,
-      (void *)(offsetof(transport_instance, uid)) },
+      (void *)offsetof(lmtp_transport_options_block, timeout) }
 };
 
 /* Size of the options list. An extern variable has to be used so that its
@@ -56,6 +44,7 @@ int lmtp_transport_options_count =
 
 lmtp_transport_options_block lmtp_transport_option_defaults = {
   NULL,           /* cmd */
+  NULL,           /* skt */
   5*60,           /* timeout */
   0               /* options */
 };
@@ -76,15 +65,12 @@ lmtp_transport_init(transport_instance *tblock)
 lmtp_transport_options_block *ob =
   (lmtp_transport_options_block *)(tblock->options_block);
 
-/* Retry_use_local_part defaults TRUE if unset */
+/* Either the command field or the socket field must be set */
 
-if (tblock->retry_use_local_part == 2) tblock->retry_use_local_part = TRUE;
-
-/* The command field must be set */
-
-if (ob->cmd == NULL)
+if ((ob->cmd == NULL) == (ob->skt == NULL))
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
-    "command must be set for the %s transport", tblock->name);
+    "one (and only one) of command or socket must be set for the %s transport",
+    tblock->name);
 
 /* If a fixed uid field is set, then a gid field must also be set. */
 
@@ -124,8 +110,8 @@ Arguments:
 Returns:       TRUE if a "QUIT" command should be sent, else FALSE
 */
 
-static BOOL check_response(int *errno_value, int more_errno, char *buffer,
-  int *yield, char **message)
+static BOOL check_response(int *errno_value, int more_errno, uschar *buffer,
+  int *yield, uschar **message)
 {
 *yield = '4';    /* Default setting is to give a temporary error */
 
@@ -155,7 +141,9 @@ end the DATA. */
 
 if (*errno_value == ERRNO_FILTER_FAIL)
   {
-  *message = string_sprintf("delivery filter process failed (%d)", more_errno);
+  *message = string_sprintf("transport filter process failed (%d)%s",
+    more_errno,
+    (more_errno == EX_EXECFAILED)? ": unable to execute command" : "");
   return FALSE;
   }
 
@@ -182,7 +170,7 @@ if (*errno_value == ERRNO_WRITEINCOMPLETE)
 
 if (buffer[0] != 0)
   {
-  char *s = string_printing(buffer);
+  uschar *s = string_printing(buffer);
   *message = string_sprintf("LMTP error after %s: %s", big_buffer, s);
   *yield = buffer[0];
   return TRUE;
@@ -226,18 +214,18 @@ lmtp_write_command(int fd, char *format, ...)
 int count, rc;
 va_list ap;
 va_start(ap, format);
-if (!string_vformat(big_buffer, big_buffer_size, format, ap))
+if (!string_vformat(big_buffer, big_buffer_size, CS format, ap))
   {
   errno = ERRNO_SMTPFORMAT;
   return FALSE;
   }
 va_end(ap);
-count = (int)strlen(big_buffer);
-DEBUG(1) debug_printf("  LMTP>> %s", big_buffer);
+count = Ustrlen(big_buffer);
+DEBUG(D_transport|D_v) debug_printf("  LMTP>> %s", big_buffer);
 rc = write(fd, big_buffer, count);
 big_buffer[count-2] = 0;     /* remove \r\n for debug and error message */
 if (rc > 0) return TRUE;
-DEBUG(1) debug_printf("write failed: %s\n", strerror(errno));
+DEBUG(D_transport) debug_printf("write failed: %s\n", strerror(errno));
 return FALSE;
 }
 
@@ -266,11 +254,11 @@ Returns:    TRUE if a valid, non-error response was received; else FALSE
 */
 
 static BOOL
-lmtp_read_response(FILE *f, char *buffer, int size, int okdigit, int timeout)
+lmtp_read_response(FILE *f, uschar *buffer, int size, int okdigit, int timeout)
 {
 int count;
-char *ptr = buffer;
-char *readptr = buffer;
+uschar *ptr = buffer;
+uschar *readptr = buffer;
 
 /* Ensure errno starts out zero */
 
@@ -298,12 +286,10 @@ for (;;)
 
     *readptr = 0;           /* In case nothing gets read */
     sigalrm_seen = FALSE;
-    os_non_restarting_signal(SIGALRM, sigalrm_handler);
     alarm(timeout);
-    rc = fgets(readptr, size-1, f);
+    rc = Ufgets(readptr, size-1, f);
     save_errno = errno;
     alarm(0);
-    signal(SIGALRM, SIG_IGN);
     errno = save_errno;
 
     if (rc != NULL) break;  /* A line has been read */
@@ -318,7 +304,7 @@ for (;;)
 
     else if (errno == EINTR)
       {
-      DEBUG(9) debug_printf("EINTR while reading LMTP response\n");
+      DEBUG(D_transport) debug_printf("EINTR while reading LMTP response\n");
       continue;
       }
 
@@ -331,7 +317,7 @@ for (;;)
   /* Adjust size in case we have to read another line, and adjust the
   count to be the length of the line we are about to inspect. */
 
-  count = (int)strlen(readptr);
+  count = Ustrlen(readptr);
   size -= count;
   count += readptr - ptr;
 
@@ -341,13 +327,13 @@ for (;;)
 
   if (ptr[count-1] != '\n')
     {
-    DEBUG(9)
+    DEBUG(D_transport)
       {
       int i;
       debug_printf("LMTP input line incomplete in one buffer:\n  ");
       for (i = 0; i < count; i++)
         {
-        int c = (uschar)(ptr[i]);
+        int c = (ptr[i]);
         if (mac_isprint(c)) debug_printf("%c", c); else debug_printf("<%d>", c);
         }
       debug_printf("\n");
@@ -359,17 +345,18 @@ for (;;)
   /* Remove any whitespace at the end of the buffer. This gets rid of CR, LF
   etc. at the end. Show it, if debugging, formatting multi-line responses. */
 
-  while (count > 0 && isspace((uschar)ptr[count-1])) count--;
+  while (count > 0 && isspace(ptr[count-1])) count--;
   ptr[count] = 0;
 
-  DEBUG(1)
+  DEBUG(D_transport|D_v)
     {
-    char *s = ptr;
-    char *t = ptr;
+    uschar *s = ptr;
+    uschar *t = ptr;
     while (*t != 0)
       {
       while (*t != 0 && *t != '\n') t++;
-      debug_printf("  %s %*s\n", (s == ptr)? "LMTP<<" : "      ", t-s, s);
+      debug_printf("  %s %*s\n", (s == ptr)? "LMTP<<" : "      ",
+        (int)(t-s), s);
       if (*t == 0) break;
       s = t = t + 1;
       }
@@ -386,11 +373,11 @@ for (;;)
 
   for(;;)
     {
-    char *p;
+    uschar *p;
     if (count < 3 ||
-       !isdigit((uschar)ptr[0]) ||
-       !isdigit((uschar)ptr[1]) ||
-       !isdigit((uschar)ptr[2]) ||
+       !isdigit(ptr[0]) ||
+       !isdigit(ptr[1]) ||
+       !isdigit(ptr[2]) ||
        (ptr[3] != '-' && ptr[3] != ' ' && ptr[3] != 0))
       {
       errno = ERRNO_SMTPFORMAT;    /* format error */
@@ -457,59 +444,104 @@ set. */
 BOOL
 lmtp_transport_entry(
   transport_instance *tblock,      /* data for this instantiation */
-  address_item *addrlist)              /* address(es) we are working on */
+  address_item *addrlist)          /* address(es) we are working on */
 {
-pid_t pid;
+pid_t pid = 0;
 FILE *out;
 lmtp_transport_options_block *ob =
   (lmtp_transport_options_block *)(tblock->options_block);
+struct sockaddr_un sockun;         /* don't call this "sun" ! */
 int timeout = ob->timeout;
 int fd_in = -1, fd_out = -1;
 int code, save_errno;
 BOOL send_data;
 BOOL yield = FALSE;
 address_item *addr;
-char buffer[256];
-char **argv;
+uschar *sockname = NULL;
+uschar **argv;
+uschar buffer[256];
 
-DEBUG(2) debug_printf("%s transport entered\n", tblock->name);
+DEBUG(D_transport) debug_printf("%s transport entered\n", tblock->name);
 
-/* Set up "transport <name>" to be put in any error messages, and then call the
-common function for creating an argument list and expanding the items. If this
-fails, copy the error information into the second and subsequent addresses. */
+/* Initialization ensures that either a command or a socket is specified, but
+not both. When a command is specified, call the common function for creating an
+argument list and expanding the items. */
 
-sprintf(buffer, "%.50s transport", tblock->name);
-if (!transport_set_up_command(&argv, ob->cmd, TRUE, PANIC, addrlist, buffer,
-     NULL))
-  return FALSE;
+if (ob->cmd != NULL)
+  {
+  DEBUG(D_transport) debug_printf("using command %s\n", ob->cmd);
+  sprintf(CS buffer, "%.50s transport", tblock->name);
+  if (!transport_set_up_command(&argv, ob->cmd, TRUE, PANIC, addrlist, buffer,
+       NULL))
+    return FALSE;
+  }
+
+/* When a socket is specified, expand the string and create a socket. */
+
+else
+  {
+  DEBUG(D_transport) debug_printf("using socket %s\n", ob->skt);
+  sockname = expand_string(ob->skt);
+  if (sockname == NULL)
+    {
+    addrlist->message = string_sprintf("Expansion of \"%s\" (socket setting "
+      "for %s transport) failed: %s", ob->skt, tblock->name,
+      expand_string_message);
+    return FALSE;
+    }
+  if ((fd_in = fd_out = socket(PF_UNIX, SOCK_STREAM, 0)) == -1)
+    {
+    addrlist->message = string_sprintf(
+      "Failed to create socket %s for %s transport: %s",
+        ob->skt, tblock->name, strerror(errno));
+    return FALSE;
+    }
+  }
 
 /* If the -N option is set, can't do any more. Presume all has gone well. */
 
 if (dont_deliver)
   {
-  debug_printf("*** delivery by %s transport bypassed by -N option",
-    tblock->name);
+  DEBUG(D_transport)
+    debug_printf("*** delivery by %s transport bypassed by -N option",
+      tblock->name);
   addrlist->transport_return = OK;
   return FALSE;
   }
 
 /* As this is a local transport, we are already running with the required
-uid/gid and current directory, so pass NULL to child_open to indicate no
-change. Request that the new process be a process group leader, so we
-can kill it and all its children on an error. */
+uid/gid and current directory. Request that the new process be a process group
+leader, so we can kill it and all its children on an error. */
 
-if ((pid = child_open(argv, NULL, 0, NULL, NULL, &fd_in, &fd_out,
-    NULL, TRUE)) < 0)
+if (ob->cmd != NULL)
   {
-  addrlist->message = string_sprintf(
-    "Failed to create child process for %s transport: %s", tblock->name,
-      strerror(errno));
-  return FALSE;
+  if ((pid = child_open(argv, NULL, 0, &fd_in, &fd_out, TRUE)) < 0)
+    {
+    addrlist->message = string_sprintf(
+      "Failed to create child process for %s transport: %s", tblock->name,
+        strerror(errno));
+    return FALSE;
+    }
+  }
+
+/* For a socket, try to make the connection */
+
+else
+  {
+  sockun.sun_family = AF_UNIX;
+  sprintf(sockun.sun_path, "%.*s", (int)(sizeof(sockun.sun_path)-1), sockname);
+  if(connect(fd_out, (struct sockaddr *)(&sockun), sizeof(sockun)) == -1)
+    {
+    addrlist->message = string_sprintf(
+      "Failed to connect to socket %s for %s transport: %s",
+        sockun.sun_path, tblock->name, strerror(errno));
+    return FALSE;
+    }
   }
 
 /* Make the output we are going to read into a file. */
 
-out = fdopen(fd_out, "r");
+out = fdopen(fd_out, "rb");
 
 /* Now we must implement the LMTP protocol. It is like SMTP, except that after
 the end of the message, a return code for every accepted RCPT TO is sent. This
@@ -517,7 +549,7 @@ allows for message+recipient checks after the message has been received. */
 
 /* First thing is to wait for an initial greeting. */
 
-strcpy(big_buffer, "initial connection");
+Ustrcpy(big_buffer, "initial connection");
 if (!lmtp_read_response(out, buffer, sizeof(buffer), '2',
   timeout)) goto RESPONSE_FAILED;
 
@@ -543,8 +575,8 @@ temporarily rejected; others may be accepted, for now. */
 send_data = FALSE;
 for (addr = addrlist; addr != NULL; addr = addr->next)
   {
-  if (!lmtp_write_command(fd_in, "RCPT TO:<%s@%s>\r\n", addr->local_part,
-       addr->domain))
+  if (!lmtp_write_command(fd_in, "RCPT TO:<%s>\r\n",
+       transport_rcpt_address(addr, tblock->rcpt_include_affixes)))
     goto WRITE_FAILED;
   if (lmtp_read_response(out, buffer, sizeof(buffer), '2', timeout))
     {
@@ -556,7 +588,12 @@ for (addr = addrlist; addr != NULL; addr = addr->next)
     if (errno != 0 || buffer[0] == 0) goto RESPONSE_FAILED;
     addr->message = string_sprintf("LMTP error after %s: %s", big_buffer,
       string_printing(buffer));
-    if (buffer[0] == '5') addr->transport_return = FAIL;
+    if (buffer[0] == '5') addr->transport_return = FAIL; else
+      {
+      int bincode = (buffer[1] - '0')*10 + buffer[2] - '0';
+      addr->basic_errno = ERRNO_RCPT4XX;
+      addr->more_errno |= bincode << 8;
+      }
     }
   }
 
@@ -571,19 +608,16 @@ if (send_data)
     goto RESPONSE_FAILED;
 
   sigalrm_seen = FALSE;
-  os_non_restarting_signal(SIGALRM, sigalrm_handler);
   transport_write_timeout = timeout;
-  strcpy(big_buffer, "sending data block");   /* For error messages */
-  DEBUG(1) debug_printf("  LMTP>> writing message and terminating \".\"\n");
+  Ustrcpy(big_buffer, "sending data block");   /* For error messages */
+  DEBUG(D_transport|D_v)
+    debug_printf("  LMTP>> writing message and terminating \".\"\n");
 
+  transport_count = 0;
   ok = transport_write_message(addrlist, fd_in, ob->options, 0,
-        tblock->add_headers, tblock->remove_headers, ".", "..",
+        tblock->add_headers, tblock->remove_headers, US".", US"..",
         tblock->rewrite_rules, tblock->rewrite_existflags);
 
-  save_errno = errno;            /* Preserve over signal call */
-  signal(SIGALRM, SIG_IGN);
-  errno = save_errno;
-
   /* Failure can either be some kind of I/O disaster (including timeout),
   or the failure of a transport filter or the expansion of added headers. */
 
@@ -593,7 +627,7 @@ if (send_data)
     goto RESPONSE_FAILED;
     }
 
-  strcpy(big_buffer, "end of data");   /* For error messages */
+  Ustrcpy(big_buffer, "end of data");   /* For error messages */
 
   /* We now expect a response for every address that was accepted above,
   in the same order. For those that get a response, their status is fixed;
@@ -660,7 +694,7 @@ RESPONSE_FAILED:
 save_errno = errno;
 addrlist->message = NULL;
 
-if(check_response(&save_errno, addrlist->more_errno,
+if (check_response(&save_errno, addrlist->more_errno,
     buffer, &code, &(addrlist->message)))
   {
   (void) lmtp_write_command(fd_in, "QUIT\r\n");
@@ -689,7 +723,7 @@ else if (errno == ERRNO_FILTER_FAIL)
 else if (errno == ERRNO_WRITEINCOMPLETE)
   addrlist->message = string_sprintf("Failed repeatedly to write data");
 else if (errno == ERRNO_SMTPFORMAT)
-  addrlist->message = "overlong LMTP command generated";
+  addrlist->message = US"overlong LMTP command generated";
 else
   addrlist->message = string_sprintf("Error %d", errno);
 
@@ -697,7 +731,7 @@ else
 
 KILL_AND_RETURN:
 
-killpg(pid, SIGKILL);
+if (pid > 0) killpg(pid, SIGKILL);
 
 /* Come here from all paths after the subprocess is created. Wait for the
 process, but with a timeout. */
@@ -709,7 +743,8 @@ RETURN:
 if (fd_in >= 0) (void) close(fd_in);
 if (fd_out >= 0) (void) fclose(out);
 
-DEBUG(2) debug_printf("%s transport yields %d\n", tblock->name, yield);
+DEBUG(D_transport)
+  debug_printf("%s transport yields %d\n", tblock->name, yield);
 
 return yield;
 }
