@@ -1,14 +1,17 @@
+/* $Cambridge: exim/exim-src/src/transports/lmtp.h,v 1.2 2005/01/04 10:00:45 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
-  char *cmd;
+  uschar *cmd;
+  uschar *skt;
   int   timeout;
   int   options;
 } lmtp_transport_options_block;
