@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/transports/pipe.c,v 1.4 2005/02/17 11:58:27 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -805,7 +807,9 @@ if (!written_ok)
         string_sprintf("Failed to expand headers_add or headers_remove: %s",
           expand_string_message);
     else if (errno == ERRNO_FILTER_FAIL)
-      addr->message = string_sprintf("Filter process failure");
+      addr->message = string_sprintf("Transport filter process failed (%d)%s",
+      addr->more_errno,
+      (addr->more_errno == EX_EXECFAILED)? ": unable to execute command" : "");
     else if (errno == ERRNO_WRITEINCOMPLETE)
       addr->message = string_sprintf("Failed repeatedly to write data");
     else
@@ -831,8 +835,7 @@ if ((rc = child_close(pid, timeout)) != 0)
     killpg(pid, SIGKILL);
     kill(outpid, SIGKILL);
     addr->transport_return = FAIL;
-    addr->message = string_sprintf("pipe delivery process timed out",
-      tblock->name);
+    addr->message = string_sprintf("pipe delivery process timed out");
     }
 
   /* Wait() failed. */
