@@ -1,31 +1,33 @@
+/* $Cambridge: exim/exim-src/src/transports/pipe.h,v 1.2 2005/01/04 10:00:45 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
-  char *cmd;
-  char *allow_commands;
-  char *environment;
-  char *path;
-  char *prefix;
-  char *suffix;
-  char *temp_errors;
-  char *check_string;
-  char *escape_string;
+  uschar *cmd;
+  uschar *allow_commands;
+  uschar *environment;
+  uschar *path;
+  uschar *message_prefix;
+  uschar *message_suffix;
+  uschar *temp_errors;
+  uschar *check_string;
+  uschar *escape_string;
   int   umask;
   int   max_output;
   int   timeout;
   int   options;
   BOOL  freeze_exec_fail;
-  BOOL  from_hack;
   BOOL  ignore_status;
   BOOL  restrict_to_path;
   BOOL  use_shell;
+  BOOL  use_bsmtp;
   BOOL  use_crlf;
 } pipe_transport_options_block;
 
