@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/tree.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2001 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for maintaining binary balanced trees and some associated
@@ -14,39 +16,6 @@ functions as well. */
 
 
 
-/*************************************************
-*      Convert address for use as tree key       *
-*************************************************/
-
-/* The domain portion of the address is forced into lower case. This function
-should not be called for pipe or file or reply addresses.
-
-Argument: points to address string
-Returns:  points to prepared address string, in a fixed bit of store
-*/
-
-static char prepared_address[512];
-
-static char *
-address_prepare(char *s)
-{
-char *p = s + (int)strlen(s);
-while (p > s && p[-1] != '@') p--;
-if (p <= s) strcpy(prepared_address, s); else
-  {
-  char *t = prepared_address;
-  char *pp = p - 2;
-  while (pp >= s && *pp != ':') pp--;
-  while (s < pp) *t++ = tolower(*s++);
-  while (s < p) *t++ = *s++;
-  while (*s) *t++ = tolower(*s++);
-  *t = 0;
-  }
-return prepared_address;
-}
-
-
-
 /*************************************************
 *        Add entry to non-recipients tree        *
 *************************************************/
@@ -55,16 +24,15 @@ return prepared_address;
 
 Arguments:
   s       string to add
-  pfr     true for a pipe/file/reply address
 
 Returns:  nothing
 */
 
 void
-tree_add_nonrecipient(char *s, BOOL pfr)
+tree_add_nonrecipient(uschar *s)
 {
-tree_node *node = store_get(sizeof(tree_node) + (int)strlen(s));
-strcpy(node->name, pfr? s : address_prepare(s));
+tree_node *node = store_get(sizeof(tree_node) + Ustrlen(s));
+Ustrcpy(node->name, s);
 node->data.ptr = NULL;
 if (!tree_insertnode(&tree_nonrecipients, node)) store_reset(node);
 }
@@ -79,45 +47,22 @@ if (!tree_insertnode(&tree_nonrecipients, node)) store_reset(node);
 
 Argument:
   s       string to add
-  pfr     TRUE for pipe/file/reply address
+  addr    the address is is a duplicate of
 
 Returns:  nothing
 */
 
 void
-tree_add_duplicate(char *s, BOOL pfr)
+tree_add_duplicate(uschar *s, address_item *addr)
 {
-tree_node *node = store_get(sizeof(tree_node) + (int)strlen(s));
-strcpy(node->name, pfr? s : address_prepare(s));
-node->data.ptr = NULL;
+tree_node *node = store_get(sizeof(tree_node) + Ustrlen(s));
+Ustrcpy(node->name, s);
+node->data.ptr = addr;
 if (!tree_insertnode(&tree_duplicates, node)) store_reset(node);
 }
 
 
 
-/*************************************************
-*            Search an address tree              *
-*************************************************/
-
-/* The key has to be prepared in order to cope with upper case bits.
-
-Arguments:
-  p         pointer to root of tree
-  name      key to search for
-  pfr       true for a pipe/file/reply address
-
-Returns:    pointer to node or NULL if not found
-*/
-
-tree_node *
-tree_search_addr(tree_node *p, char *name, BOOL pfr)
-{
-return tree_search(p, pfr? name : address_prepare(name));
-}
-
-
-
-
 /*************************************************
 *    Add entry to unusable addresses tree        *
 *************************************************/
@@ -132,10 +77,10 @@ void
 tree_add_unusable(host_item *h)
 {
 tree_node *node;
-char s[256];
-sprintf(s, "T:%.200s:%s", h->name, h->address);
-node = store_get(sizeof(tree_node) + (int)strlen(s));
-strcpy(node->name, s);
+uschar s[256];
+sprintf(CS s, "T:%.200s:%s", h->name, h->address);
+node = store_get(sizeof(tree_node) + Ustrlen(s));
+Ustrcpy(node->name, s);
 node->data.val = h->why;
 if (h->status == hstatus_unusable_expired) node->data.val += 256;
 if (!tree_insertnode(&tree_unusable, node)) store_reset(node);
@@ -202,7 +147,7 @@ write_tree(p, f);
 /* This set of routines maintains a balanced binary tree using
 the algorithm given in Knuth Vol 3 page 455.
 
-The routines make use of char * pointers as byte pointers,
+The routines make use of uschar * pointers as byte pointers,
 so as to be able to do arithmetic on them, since ANSI Standard
 C does not permit additions and subtractions on void pointers. */
 
@@ -259,7 +204,7 @@ t = q;
 
 for (;;)
   {
-  int c = strcmp(node->name, p->name);
+  int c = Ustrcmp(node->name, p->name);
   if (c == 0) return FALSE;              /* Duplicate node encountered */
 
   /* Deal with climbing down the tree, exiting from the loop
@@ -284,7 +229,7 @@ that is the place at which the new node must be inserted. */
 next node after it along the route. */
 
 s = *t;
-r = (strcmp(node->name, s->name) > 0)? s->right : s->left;
+r = (Ustrcmp(node->name, s->name) > 0)? s->right : s->left;
 
 /* Adjust balance factors along the route from s to node. */
 
@@ -292,7 +237,7 @@ p = r;
 
 while (p != node)
   {
-  if (strcmp(node->name, p->name) < 0)
+  if (Ustrcmp(node->name, p->name) < 0)
     {
     p->balance = tree_lbal;
     p = p->left;
@@ -306,15 +251,15 @@ while (p != node)
 
 /* Now the World-Famous Balancing Act */
 
-a = (strcmp(node->name, s->name) < 0)? tree_lbal : tree_rbal;
+a = (Ustrcmp(node->name, s->name) < 0)? tree_lbal : tree_rbal;
 
-if (s->balance == 0) s->balance = (char)a;        /* The tree has grown higher */
-  else if (s->balance != (char)a) s->balance = 0; /* It's become more balanced */
+if (s->balance == 0) s->balance = (uschar)a;        /* The tree has grown higher */
+  else if (s->balance != (uschar)a) s->balance = 0; /* It's become more balanced */
 else                                              /* It's got out of balance */
   {
   /* Perform a single rotation */
 
-  if (r->balance == (char)a)
+  if (r->balance == (uschar)a)
     {
     p = r;
     if (a == tree_rbal)
@@ -357,8 +302,8 @@ else                                              /* It's got out of balance */
       p->right = s;
       }
 
-    s->balance = (p->balance == (char)a)? (char)(a^tree_bmask) : 0;
-    r->balance = (p->balance == (char)(a^tree_bmask))? (char)a : 0;
+    s->balance = (p->balance == (uschar)a)? (uschar)(a^tree_bmask) : 0;
+    r->balance = (p->balance == (uschar)(a^tree_bmask))? (uschar)a : 0;
     p->balance = 0;
     }
 
@@ -385,11 +330,11 @@ Returns:    pointer to node, or NULL if not found
 */
 
 tree_node *
-tree_search(tree_node *p, char *name)
+tree_search(tree_node *p, uschar *name)
 {
 while (p != NULL)
   {
-  int c = strcmp(name, p->name);
+  int c = Ustrcmp(name, p->name);
   if (c == 0) return p;
   p = (c < 0)? p->left : p->right;
   }
@@ -397,54 +342,4 @@ return NULL;
 }
 
 
-
-/*************************************************
-*               Print tree                       *
-*************************************************/
-
-/* Recursive tree-printing subroutine. It uses a static vector
-of char to hold the line-drawing characters that need to be
-printed on every line as it moves down the page. This function
-is used only in debugging circumstances. */
-
-#define tree_printlinesize 132   /* line size for printing */
-static char tree_printline[tree_printlinesize];
-
-/* Internal recursive subroutine.
-
-Arguments:
-  p          tree node
-  pos        amount of indenting & vertical bars to pring
-  barswitch  if TRUE print | at the pos value
-
-Returns:     nothing
-*/
-
-static void
-tree_printsub(tree_node *p, int pos, int barswitch, FILE *f)
-{
-int i;
-if (p->right != NULL) tree_printsub(p->right, pos+2, 1, f);
-for (i = 0; i <= pos-1; i++) fputc(tree_printline[i], f);
-fprintf(f, "-->%s [%d]\n", p->name, p->balance);
-tree_printline[pos] = barswitch? '|' : ' ';
-if (p->left != NULL)
-  {
-  tree_printline[pos+2] = '|';
-  tree_printsub(p->left, pos+2, 0, f);
-  }
-}
-
-/* The external function, with just a tree node argument. */
-
-void
-tree_print(tree_node *p, FILE *f)
-{
-int i;
-if (f == NULL) return;
-for (i = 0; i < tree_printlinesize; i++) tree_printline[i] = ' ';
-if (p == NULL) fprintf(f, "Empty Tree\n"); else tree_printsub(p, 0, 0, f);
-fprintf(f, "---- End of tree ----\n");
-}
-
 /* End of tree.c */
